----------------------------------------------------------------------------------
-- Company: Uni Bremen ITP
-- Engineer: David Rotermund 
-- 
-- Create Date:    14:01:41 04/10/2011 
-- Design Name: 
-- Module Name:    AutoPollerCore - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;

	-- I will try to protect as much as possible against bit errors but 
	-- The read request of the block read command is not protectable 

	-- Page 0 ; TransferType "001000"; 
-- constant RequestStringForTheSize : std_logic_vector(29 downto 0) := "0010000";	
	
	-- What to do:
	-- 0.) Reserve SPI channel 
	-- 1.) Memory Page read 
	-- 		43 8'h2B reg_interface_mode
	-- 2.) If Memory Page is not 0 then set it to 0 
	-- 3.) read reg_rxbuff_bsize  
	-- 		8'h02 reg_rxbuff_bsize Receive buffer block size register
	--			[3:0] - Number of bytes, 0 to 15, in a block for the receive buffer,
	-- 4.) read reg_rxbuff_used 
	-- 		8'h01 reg_rxbuff_used Receive buffer used blocks register
	--			[6:0] - Number of blocks (0 to 64) used in RX buffer.
	-- 5.) If reg_rxbuff_used != 0 then download the data
	--			45 8'h2D reg_txrxbuff
	-- 6.) restore Memory Page via writing 
	-- 7.) Free SPI channel 

entity AutoPollerCore is
    Port ( 
				
				FIFO_IS_FULL 	: in STD_LOGIC;
				
				INPUT_DATA 		: in std_logic_vector(15 downto 0);
				INPUT_VALID		: in STD_LOGIC;
				
				OUTPUT_DATA 	: out std_logic_vector(29 downto 0);
				OUTPUT_VALID 	: out STD_LOGIC;
				
				Poll_Start 		: in STD_LOGIC;
				Poll_Active 	: out STD_LOGIC;
				
				clk : in  STD_LOGIC;
				Reset : in  STD_LOGIC
			  
			);
end AutoPollerCore;

architecture Behavioral of AutoPollerCore is

	type STATE_Type is (
--	Wait for a job	
		Idle,
-- 1 Memory Page read
-- 		43 8'h2B reg_interface_mode		
		ReadMemoryPage1_A_Req,
		ReadMemoryPage1_A_Wait,
		ReadMemoryPage1_B_Req,
		ReadMemoryPage1_B_Wait,
		
-- 2	If Memory Page is not 0 then set it to 0
-- 		43 8'h2B reg_interface_mode
		WriteMemoryPage2_A_Req,
		ReadMemoryPage2_A_Req,
		ReadMemoryPage2_A_Wait,
		ReadMemoryPage2_B_Req,
		ReadMemoryPage2_B_Wait,

-- 3 read reg_rxbuff_bsize
-- 		8'h02 reg_rxbuff_bsize Receive buffer block size register
--			[3:0] - Number of bytes, 0 to 15, in a block for the receive buffer,
		ReadBufferSize_A_Req,
		ReadBufferSize_A_Wait,
		ReadBufferSize_B_Req,
		ReadBufferSize_B_Wait,

-- 4 read reg_rxbuff_used
-- 		8'h01 reg_rxbuff_used Receive buffer used blocks register
--			[6:0] - Number of blocks (0 to 64) used in RX buffer.
--		If reg_rxbuff_used != 0 then download the data
		ReadBufferUsed_A_Req,
		ReadBufferUsed_A_Wait,
		ReadBufferUsed_B_Req,
		ReadBufferUsed_B_Wait,

-- 5a Read the Buffer
--			45 8'h2D reg_txrxbuff
		ReadTheBuffer_Start,
		ReadTheBuffer_InMediasRes,
		ReadTheBuffer_End,

-- 6	Restore the interface mode
-- 		43 8'h2B reg_interface_mode
		WriteMemoryPage3_A_Req,
		ReadMemoryPage3_A_Req,
		ReadMemoryPage3_A_Wait,
		ReadMemoryPage3_B_Req,
		ReadMemoryPage3_B_Wait,
-- 7 	End of Polling
--	Job is done
		CoolDown
	);
	signal TheMACHINE_STATE : STATE_TYPE;

	-- The signals for the read data 
	signal MemoryPageData : std_logic_vector(7 downto 0) := (others => '0');
	signal BufferSize : std_logic_vector(7 downto 0) := (others => '0');
	signal BufferUsed : std_logic_vector(7 downto 0) := (others => '0');
	signal IntermediateReadingResult : std_logic_vector(7 downto 0) := (others => '0');

	-- ReTry a read operation if something went wrong during reading
-- done	
-- 125MHz/4Mhz	= 31.25 
-- One SPI is ~ 20 bit (20*31.25 = 625 of the 125 MHz cycles)
-- Waiting for 5 * 20 bit = 3125 of the 125 MHz cycles
-- Lets use 5000 of the 125 MHz cycles
	constant ReadReTry_MaxTime : unsigned(48 downto 0) := to_unsigned(10000,49);
-- DEBUG: 
--	constant ReadReTry_MaxTime : unsigned(48 downto 0) := to_unsigned(5,49);
	signal ReadReTry_Counter : unsigned(48 downto 0) := (others => '0');
	
	-- Constants defining the command strings
-- done
	constant RequestStringFor_MemoryPage : std_logic_vector(29 downto 0) := "0000000000000000" & "1" & "0101011"  & "001000";
	-- 2 empty bytes: "0000000000000000", Address(43): "0101011", Read: "1", Transfer-Type: "001000"
	constant RequestStringFor_BufferSize : std_logic_vector(29 downto 0) := "0000000000000000" & "1" & "0000010"  & "001001";
	-- 2 empty bytes: "0000000000000000", Address(2): "0000010", Read: "1", Transfer-Type: "001001"
	constant RequestStringFor_BufferUsed : std_logic_vector(29 downto 0) := "0000000000000000" & "1" & "0000001"  & "001010";
	-- 2 empty bytes: "0000000000000000", Address(1): "0000001", Read: "1", Transfer-Type: "001010"
	constant RequestStringFor_BufferTransferStart : std_logic_vector(29 downto 0) := "0000000000000000" & "1" & "0101101"  & "000101";
	-- 2 empty bytes: "0000000000000000", Address(45): "0101101", Read: "1", Transfer-Type: "000101"
	constant RequestStringFor_BufferTransferMid : std_logic_vector(29 downto 0) := "000000000000000000000000" & "000110";
	-- 3 empty bytes: "000000000000000000000000", Transfer-Type: "000110"
	constant RequestStringFor_BufferTransferEnd : std_logic_vector(29 downto 0) := "000000000000000000000000" & "000111";
	-- 3 empty bytes: "000000000000000000000000", Transfer-Type: "000111"
	constant RequestStringFor_MemoryPageWrite : std_logic_vector(13 downto 0) := "0" & "0101011" & "000100";
	-- Address(43): "0101011", Write: "0", Transfer-Type: "001000"

	-- Signals for the block transfer mid section 
	signal Submit_Max : unsigned(15 downto 0) := (others => '0');
	signal Submit_Counter : unsigned(15 downto 0) := (others => '0');
	
begin

	TheStateMachineProzess : process(clk,reset)  
	begin
		if Reset = '1' then
			TheMACHINE_STATE <= Idle;
			MemoryPageData <= (others => '0');
			BufferSize <= (others => '0');
			IntermediateReadingResult <= (others => '0');
			ReadReTry_Counter <= (others => '0');
			Poll_Active <= '0';
			Submit_Max <= (others => '0');
			Submit_Counter	<= (others => '0');
			
			OUTPUT_DATA <= (others => '0');
			OUTPUT_VALID <= '0';
			
		elsif rising_edge(clk) then 
			TheMACHINE_STATE <= TheMACHINE_STATE;
			MemoryPageData <= MemoryPageData;
			BufferSize <= BufferSize;
			IntermediateReadingResult <= IntermediateReadingResult;
			ReadReTry_Counter <= ReadReTry_Counter;
			Poll_Active <= '1';
			Submit_Max <= Submit_Max;
			Submit_Counter	<= Submit_Counter;

			OUTPUT_DATA <= (others => '0');
			OUTPUT_VALID <= '0';			
			
			case TheMACHINE_STATE is

				--	Wait for a job	
				when Idle =>
-- done
					if Poll_Start = '0' then
						-- Wait for a job => Idle
						TheMACHINE_STATE <= Idle;
						Poll_Active <= '0';
					else
						-- There is something to do => ReadMemoryPage1_A_Req
						TheMACHINE_STATE <= ReadMemoryPage1_A_Req;
					end if;
					
				-- 1 Memory Page read
				-- 		43 8'h2B reg_interface_mode		
				when ReadMemoryPage1_A_Req =>
-- done				
					if FIFO_IS_FULL = '1' then
						-- Wait if the FIFO is full => ReadMemoryPage1_A_Req
						TheMACHINE_STATE <= ReadMemoryPage1_A_Req;
					else
						-- After producting a Request => ReadMemoryPage1_A_Wait
						TheMACHINE_STATE <= ReadMemoryPage1_A_Wait;
						OUTPUT_VALID <= '1';
						OUTPUT_DATA <= RequestStringFor_MemoryPage;
					end if;
					ReadReTry_Counter <= (others => '0');
					
				when ReadMemoryPage1_A_Wait =>
-- done	
					if (INPUT_VALID = '1') AND (INPUT_DATA(7 downto 2) = "001000") then
						-- Got an answer => ReadMemoryPage1_B_Req. 
						TheMACHINE_STATE <= ReadMemoryPage1_B_Req;
						IntermediateReadingResult <= INPUT_DATA(15 downto 8);						
					elsif (ReadReTry_Counter >= ReadReTry_MaxTime) then
						-- Waited too long and got no answer 
						-- Retry reading => ReadMemoryPage1_A_Req
						TheMACHINE_STATE <= ReadMemoryPage1_A_Req;
					else
						-- Still waiting for an answer => ReadMemoryPage1_A_Wait
						TheMACHINE_STATE <= ReadMemoryPage1_A_Wait;
						ReadReTry_Counter <= ReadReTry_Counter + 1;
					end if;

				when ReadMemoryPage1_B_Req =>
-- done					
					if FIFO_IS_FULL = '1' then
						-- Wait if the FIFO is full => ReadMemoryPage1_B_Req
						TheMACHINE_STATE <= ReadMemoryPage1_B_Req;
					else
						-- After producing the second request => ReadMemoryPage1_B_Wait
						TheMACHINE_STATE <= ReadMemoryPage1_B_Wait;
						OUTPUT_VALID <= '1';
						OUTPUT_DATA <= RequestStringFor_MemoryPage;
					end if;
					ReadReTry_Counter <= (others => '0');
				
				when ReadMemoryPage1_B_Wait =>
-- done				
					if (INPUT_VALID = '1') AND (INPUT_DATA(7 downto 2) = "001000") then
					
						if IntermediateReadingResult = INPUT_DATA(15 downto 8) then
							MemoryPageData <= IntermediateReadingResult;
							if IntermediateReadingResult(0) = '1' then
								-- Got an VAILD answer AND the important bit is 1 => WriteMemoryPage2_A_Req. 					
								-- We need to write the memory page register
								TheMACHINE_STATE <= WriteMemoryPage2_A_Req;
							else
								-- Got an VAILD answer AND the important bit is 0 => ReadBufferSize_A_Req. 					
								-- No need to write the Memory Page register
								TheMACHINE_STATE <= ReadBufferSize_A_Req;
							end if;
						else
							-- Got an UNVAILD answer, got to ReadMemoryPage1_A_Req. 
							-- Let's do it again... => ReadMemoryPage1_A_Req
							TheMACHINE_STATE <= ReadMemoryPage1_A_Req;
						end if;

					elsif (ReadReTry_Counter >= ReadReTry_MaxTime) then
						-- Waited too long and got no answer 
						-- Retry reading => ReadMemoryPage1_B_Req
						TheMACHINE_STATE <= ReadMemoryPage1_B_Req;
					else
						-- Still waiting for an answer => ReadMemoryPage1_B_Wait				
						TheMACHINE_STATE <= ReadMemoryPage1_B_Wait;
						ReadReTry_Counter <= ReadReTry_Counter + 1;
					end if;
					
				-- 2	If Memory Page is not 0 then set it to 0
				-- 		43 8'h2B reg_interface_mode
				when WriteMemoryPage2_A_Req =>
-- done					
					if FIFO_IS_FULL = '1' then
						-- Wait if the FIFO is full => WriteMemoryPage2_A_Req
						TheMACHINE_STATE <= WriteMemoryPage2_A_Req;
					else
						-- Write the MemoryPage value with bit 0
						-- and then test if this worked out by reading 
						-- it back => ReadMemoryPage2_A_Req
						TheMACHINE_STATE <= ReadMemoryPage2_A_Req;
						OUTPUT_VALID <= '1';
						OUTPUT_DATA <=  "00000000" & MemoryPageData(7 downto 1) & "0" & RequestStringFor_MemoryPageWrite;
					end if;
					ReadReTry_Counter <= (others => '0');
										
				when ReadMemoryPage2_A_Req =>
-- done					
					if FIFO_IS_FULL = '1' then
						-- Wait if the FIFO is full => ReadMemoryPage2_A_Req
						TheMACHINE_STATE <= ReadMemoryPage2_A_Req;
					else
						-- After producing the first request => ReadMemoryPage2_A_Wait
						TheMACHINE_STATE <= ReadMemoryPage2_A_Wait;
						OUTPUT_VALID <= '1';
						OUTPUT_DATA <= RequestStringFor_MemoryPage;
					end if;
					ReadReTry_Counter <= (others => '0');
					
				when ReadMemoryPage2_A_Wait =>
-- done
					if (INPUT_VALID = '1') AND (INPUT_DATA(7 downto 2) = "001000") then				
					
						if INPUT_DATA(15 downto 8) = (MemoryPageData(7 downto 1) & "0") then 
							-- Got a correct answer => ReadMemoryPage2_B_Req. 
							TheMACHINE_STATE <= ReadMemoryPage2_B_Req;
							IntermediateReadingResult <= INPUT_DATA(15 downto 8);	
						else
							-- Got a wrong answer: write again => WriteMemoryPage2_A_Req
							TheMACHINE_STATE <= WriteMemoryPage2_A_Req;
						end if;
					elsif (ReadReTry_Counter >= ReadReTry_MaxTime) then
						-- Waited too long and got no answer 
						-- Retry reading => ReadMemoryPage2_A_Req
						TheMACHINE_STATE <= ReadMemoryPage2_A_Req;
					else
						-- Still waiting for an answer => ReadMemoryPage2_A_Wait
						TheMACHINE_STATE <= ReadMemoryPage2_A_Wait;
						ReadReTry_Counter <= ReadReTry_Counter + 1;
					end if;

				when ReadMemoryPage2_B_Req =>
-- done				
					if FIFO_IS_FULL = '1' then
						-- Wait if the FIFO is full => ReadMemoryPage2_B_Req
						TheMACHINE_STATE <= ReadMemoryPage2_B_Req;
					else
						-- After producing the second request => ReadMemoryPage2_B_Wait
						TheMACHINE_STATE <= ReadMemoryPage2_B_Wait;				
						OUTPUT_VALID <= '1';
						OUTPUT_DATA <= RequestStringFor_MemoryPage;
					end if;
					ReadReTry_Counter <= (others => '0');
	
				when ReadMemoryPage2_B_Wait =>
-- done				
					if (INPUT_VALID = '1') AND (INPUT_DATA(7 downto 2) = "001000") then

						if IntermediateReadingResult = INPUT_DATA(15 downto 8) then
							-- Got an VAILD answer => ReadBufferSize_A_Req.
							TheMACHINE_STATE <= ReadBufferSize_A_Req;
						else
							-- Got an UNVAILD answer, => WriteMemoryPage2_A_Req. 
							-- Let's write and read again...					
							TheMACHINE_STATE <= WriteMemoryPage2_A_Req;
						end if;
					elsif (ReadReTry_Counter >= ReadReTry_MaxTime) then
						-- Waited too long and got no answer 
						-- Retry reading => ReadMemoryPage2_B_Req
						TheMACHINE_STATE <= ReadMemoryPage2_B_Req;
					else
						-- Still waiting for an answer => ReadMemoryPage2_B_Wait
						TheMACHINE_STATE <= ReadMemoryPage2_B_Wait;
						ReadReTry_Counter <= ReadReTry_Counter + 1;
					end if;

				-- 3 read reg_rxbuff_bsize
				-- 		8'h02 reg_rxbuff_bsize Receive buffer block size register
				--			[3:0] - Number of bytes, 0 to 15, in a block for the receive buffer,
				when ReadBufferSize_A_Req =>
-- done
					if FIFO_IS_FULL = '1' then
						-- Wait if the FIFO is full => ReadBufferSize_A_Req
						TheMACHINE_STATE <= ReadBufferSize_A_Req;
					else
						-- After producing the first request => ReadBufferSize_A_Wait
						TheMACHINE_STATE <= ReadBufferSize_A_Wait;
						OUTPUT_VALID <= '1';
						OUTPUT_DATA <= RequestStringFor_BufferSize;
					end if;
					ReadReTry_Counter <= (others => '0');

				when ReadBufferSize_A_Wait =>
-- done
					if (INPUT_VALID = '1') AND (INPUT_DATA(7 downto 2) = "001001") then
						-- Got an answer => ReadBufferSize_B_Req.
						TheMACHINE_STATE <= ReadBufferSize_B_Req;
						IntermediateReadingResult <= INPUT_DATA(15 downto 8);	
					elsif (ReadReTry_Counter >= ReadReTry_MaxTime) then
						-- Waited too long and got no answer 
						-- Retry reading => ReadBufferSize_A_Req
						TheMACHINE_STATE <= ReadBufferSize_A_Req;
					else
						-- Still waiting for an answer => ReadBufferSize_A_Wait
						TheMACHINE_STATE <= ReadBufferSize_A_Wait;
						ReadReTry_Counter <= ReadReTry_Counter + 1;
					end if;
					
				when ReadBufferSize_B_Req =>
-- done
					if FIFO_IS_FULL = '1' then
						-- Wait if the FIFO is full => ReadBufferSize_B_Req
						TheMACHINE_STATE <= ReadBufferSize_B_Req;
					else
						-- After producing the second request => ReadBufferSize_B_Wait
						TheMACHINE_STATE <= ReadBufferSize_B_Wait;
						OUTPUT_VALID <= '1';
						OUTPUT_DATA <= RequestStringFor_BufferSize;
					end if;
					ReadReTry_Counter <= (others => '0');
					
				when ReadBufferSize_B_Wait =>
-- done
					if (INPUT_VALID = '1') AND (INPUT_DATA(7 downto 2) = "001001") then
					
						if IntermediateReadingResult = INPUT_DATA(15 downto 8) then 
						
							if unsigned(IntermediateReadingResult(3 downto 0)) > to_unsigned(0,4) then 
								-- Got an VAILD answer => ReadBufferUsed_A_Req.
								TheMACHINE_STATE <= ReadBufferUsed_A_Req;
							else
								if MemoryPageData(0) = '1' then
									-- Transfer is done, restore the Memory page if there the bit was 1
									-- => WriteMemoryPage3_A_Req
									TheMACHINE_STATE <= WriteMemoryPage3_A_Req;
								else
									-- Transfer is done, don't restore the Memory page if there the bit was 0
									-- => CoolDown
									TheMACHINE_STATE <= CoolDown;
								end if;						
							end if;
							BufferSize <= IntermediateReadingResult;
-- DEBUG: 
--							BufferSize <= "00001111";
						else
							-- Got an UNVAILD answer, got to ReadBufferSize_A_Req. 
							-- => ReadBufferSize_A_Req						
							TheMACHINE_STATE <= ReadBufferSize_A_Req;
						end if;
					elsif (ReadReTry_Counter >= ReadReTry_MaxTime) then
						-- Waited too long and got no answer 
						-- Retry reading => ReadBufferSize_B_Req
						TheMACHINE_STATE <= ReadBufferSize_B_Req;
					else
						-- Still waiting for an answer => ReadBufferSize_B_Wait
						TheMACHINE_STATE <= ReadBufferSize_B_Wait;
						ReadReTry_Counter <= ReadReTry_Counter + 1;
					end if;
					
				-- 4 read reg_rxbuff_used
				-- 		8'h01 reg_rxbuff_used Receive buffer used blocks register
				--			[6:0] - Number of blocks (0 to 64) used in RX buffer.
				--		If reg_rxbuff_used != 0 then download the data
				when ReadBufferUsed_A_Req =>
-- done
					if FIFO_IS_FULL = '1' then
						-- Wait if the FIFO is full => ReadBufferUsed_A_Req
						TheMACHINE_STATE <= ReadBufferUsed_A_Req;
					else
						-- After producing the first request => ReadBufferUsed_A_Wait
						TheMACHINE_STATE <= ReadBufferUsed_A_Wait;
						OUTPUT_VALID <= '1';
						OUTPUT_DATA <= RequestStringFor_BufferUsed;
					end if;
					ReadReTry_Counter <= (others => '0');
					
				when ReadBufferUsed_A_Wait =>
-- done
					if (INPUT_VALID = '1') AND (INPUT_DATA(7 downto 2) = "001010") then
						-- Got an answer => ReadBufferUsed_B_Req.
						TheMACHINE_STATE <= ReadBufferUsed_B_Req;
						IntermediateReadingResult <= INPUT_DATA(15 downto 8);
					elsif (ReadReTry_Counter >= ReadReTry_MaxTime ) then
						-- Waited too long and got no answer
						-- Retry reading => ReadBufferUsed_A_Req
						TheMACHINE_STATE <= ReadBufferUsed_A_Req;
					else
						-- Still waiting for an answer => ReadBufferUsed_A_Wait
						TheMACHINE_STATE <= ReadBufferUsed_A_Wait;
						ReadReTry_Counter <= ReadReTry_Counter + 1;
					end if;
					
				when ReadBufferUsed_B_Req =>
-- done				
					if FIFO_IS_FULL = '1' then
						-- Wait if the FIFO is full => ReadBufferUsed_B_Req
						TheMACHINE_STATE <= ReadBufferUsed_B_Req;
					else
						-- After producing the second request => ReadBufferUsed_B_Wait
						TheMACHINE_STATE <= ReadBufferUsed_B_Wait;
						OUTPUT_VALID <= '1';
						OUTPUT_DATA <= RequestStringFor_BufferUsed;
					end if;
					ReadReTry_Counter <= (others => '0');
					
				when ReadBufferUsed_B_Wait =>
-- done	
					if (INPUT_VALID = '1') AND (INPUT_DATA(7 downto 2) = "001010") then
						if IntermediateReadingResult = INPUT_DATA(15 downto 8) then

							if unsigned(IntermediateReadingResult(6 downto 0)) > to_unsigned(0,7) then 
								-- Got a VAILD answer => ReadTheBuffer_Start
								TheMACHINE_STATE <= ReadTheBuffer_Start;
							else
								if MemoryPageData(0) = '1' then
									-- Transfer is done, restore the Memory page if there the bit was 1
									-- => WriteMemoryPage3_A_Req
									TheMACHINE_STATE <= WriteMemoryPage3_A_Req;
								else
									-- Transfer is done, don't restore the Memory page if there the bit was 0
									-- => CoolDown
									TheMACHINE_STATE <= CoolDown;
								end if;
							end if;
						
							BufferUsed <= IntermediateReadingResult;

-- DEBUG:
--							BufferUsed <= "00001010";
							
						else 
							-- Got a UNVAILD answer 
							-- => ReadBufferUsed_A_Req
							TheMACHINE_STATE <= ReadBufferUsed_A_Req;
						end if;

					elsif (ReadReTry_Counter >= ReadReTry_MaxTime) then
						-- Waited too long and got no answer 
						-- Retry reading => ReadBufferUsed_B_Req						
						TheMACHINE_STATE <= ReadBufferUsed_B_Req;
					else
						-- Still waiting for an answer => ReadBufferUsed_B_Wait
						TheMACHINE_STATE <= ReadBufferUsed_B_Wait;
						ReadReTry_Counter <= ReadReTry_Counter + 1;
					end if;
					
				-- 5a Read the Buffer
				--			45 8'h2D reg_txrxbuff
				when ReadTheBuffer_Start =>
-- done
					if FIFO_IS_FULL = '1' then
						-- Wait if the FIFO is full => ReadTheBuffer_Start
						TheMACHINE_STATE <= ReadTheBuffer_Start;
					else
						-- Write the starting request of the block read
						-- go to the request for the next value
						-- => ReadTheBuffer_InMediasRes
						TheMACHINE_STATE <= ReadTheBuffer_InMediasRes;
						OUTPUT_VALID <= '1';
						OUTPUT_DATA <= RequestStringFor_BufferTransferStart;
						Submit_Max <= resize((unsigned(BufferSize(3 downto 0)) * unsigned(BufferUsed(6 downto 0))) - 3 ,16);
					end if;
					Submit_Counter <= (others => '0');

				when ReadTheBuffer_InMediasRes =>
-- done
					if FIFO_IS_FULL = '1' then	
						-- Wait if the FIFO is full => ReadTheBuffer_InMediasRes
						TheMACHINE_STATE <= ReadTheBuffer_InMediasRes;
					else
						OUTPUT_VALID <= '1';
						OUTPUT_DATA <= RequestStringFor_BufferTransferMid;
						if Submit_Counter >= Submit_Max then
							-- Go the end of the block requests 
							-- => ReadTheBuffer_End
							TheMACHINE_STATE <= ReadTheBuffer_End;
						else
							-- Write the next in medias res request value 
							-- => ReadTheBuffer_InMediasRes
							TheMACHINE_STATE <= ReadTheBuffer_InMediasRes;
							Submit_Counter <= Submit_Counter + 1;
						end if;
					end if;
					
				when ReadTheBuffer_End =>
-- done					
					if FIFO_IS_FULL = '1' then
						-- Wait if the FIFO is full => ReadTheBuffer_End
						TheMACHINE_STATE <= ReadTheBuffer_End;
					else
						OUTPUT_VALID <= '1';
						OUTPUT_DATA <= RequestStringFor_BufferTransferEnd;
						
						if MemoryPageData(0) = '1' then
							-- Transfer is done, restore the Memory page if there the bit was 1
							-- => WriteMemoryPage3_A_Req
							TheMACHINE_STATE <= WriteMemoryPage3_A_Req;
						else
							-- Transfer is done, don't restore the Memory page if there the bit was 0
							-- => CoolDown
							TheMACHINE_STATE <= CoolDown;
						end if;
					end if;
					
				-- 6	Restore the interface mode
				-- 		43 8'h2B reg_interface_mode
				when WriteMemoryPage3_A_Req => 
-- done						
					if FIFO_IS_FULL = '1' then
						-- Wait if the FIFO is full => WriteMemoryPage3_A_Req
						TheMACHINE_STATE <= WriteMemoryPage3_A_Req;
					else
						-- After producing the write request, let's read it back 
						-- => ReadMemoryPage3_A_Req
						TheMACHINE_STATE <= ReadMemoryPage3_A_Req;
						OUTPUT_VALID <= '1';
						OUTPUT_DATA <=  "00000000" & MemoryPageData(7 downto 0) & RequestStringFor_MemoryPageWrite;						
					end if;
					ReadReTry_Counter <= (others => '0');
					
				when ReadMemoryPage3_A_Req =>
-- done
					if FIFO_IS_FULL = '1' then
						-- Wait if the FIFO is full => ReadMemoryPage3_A_Req
						TheMACHINE_STATE <= ReadMemoryPage3_A_Req;
					else
						-- After producing the first request => ReadMemoryPage3_A_Wait
						TheMACHINE_STATE <= ReadMemoryPage3_A_Wait;
						OUTPUT_VALID <= '1';
						OUTPUT_DATA <= RequestStringFor_MemoryPage;
					end if;
					ReadReTry_Counter <= (others => '0');
					
				when ReadMemoryPage3_A_Wait =>
-- done	
					if (INPUT_VALID = '1') AND (INPUT_DATA(7 downto 2) = "001000") then
						if INPUT_DATA(15 downto 8) = MemoryPageData then
							-- Got a correct answer => ReadMemoryPage3_B_Req
							TheMACHINE_STATE <= ReadMemoryPage3_B_Req;
						else
							-- Got a wrong answer => WriteMemoryPage3_A_Req
							TheMACHINE_STATE <= WriteMemoryPage3_A_Req;
						end if;
						
					elsif (ReadReTry_Counter >= ReadReTry_MaxTime ) then
						-- Waited too long and got no answer 
						-- Retry reading => ReadMemoryPage3_A_Req
						TheMACHINE_STATE <= ReadMemoryPage3_A_Req;
					else
					-- Still waiting for an answer => ReadMemoryPage3_A_Wait
						TheMACHINE_STATE <= ReadMemoryPage3_A_Wait;
						ReadReTry_Counter <= ReadReTry_Counter + 1;
					end if;
					
				when ReadMemoryPage3_B_Req =>
-- done
					if FIFO_IS_FULL = '1' then
						-- Wait if the FIFO is full => ReadMemoryPage3_B_Req
						TheMACHINE_STATE <= ReadMemoryPage3_B_Req;
					else
						-- After producing the second request => ReadMemoryPage3_B_Wait
						TheMACHINE_STATE <= ReadMemoryPage3_B_Wait;
						OUTPUT_VALID <= '1';
						OUTPUT_DATA <= RequestStringFor_MemoryPage;
					end if;
					ReadReTry_Counter <= (others => '0');
					
				when ReadMemoryPage3_B_Wait =>
-- done
					if (INPUT_VALID = '1') AND (INPUT_DATA(7 downto 2) = "001000") then
					
						if INPUT_DATA(15 downto 8) = MemoryPageData then
							-- Got an VAILD answer => CoolDown
							TheMACHINE_STATE <= CoolDown;						
						else
							-- Got an UNVAILD answer write and read again 
							-- => WriteMemoryPage3_A_Req
							TheMACHINE_STATE <= WriteMemoryPage3_A_Req;
						end if;

					elsif (ReadReTry_Counter >= ReadReTry_MaxTime) then
						-- Waited too long and got no answer 
						-- Retry reading => ReadMemoryPage3_B_Req
						TheMACHINE_STATE <= ReadMemoryPage3_B_Req;
					else
						-- Still waiting for an answer => ReadMemoryPage3_B_Wait
						TheMACHINE_STATE <= ReadMemoryPage3_B_Wait;
						ReadReTry_Counter <= ReadReTry_Counter + 1;
					end if;
					
				-- 7 	End of Polling
				--	Job is done
				when CoolDown =>
-- done
					-- Let's go home => Idle
					TheMACHINE_STATE <= Idle;
					Poll_Active <= '0';
				
				-- Check for something unexpected 
				when others => 
-- done				
					-- Let's go home => Idle
					TheMACHINE_STATE <= Idle;
					Poll_Active <= '0';					
			end case;
			
		end if;
	end process TheStateMachineProzess;

end architecture Behavioral;

