----------------------------------------------------------------------------------
-- Company: Uni Bremen ITP
-- Engineer: David Rotermund
-- 
-- Create Date:    14:25:04 08/03/2011 
-- Design Name: 
-- Module Name:    AutomaticCore - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;

entity AutomaticCore is
    Port ( 
		-- Did the clock ring? Then we need to do somehing on 0->1
		AutomaticClock_Event 		: in std_logic_vector (1 downto 0);
		
		-- Have we an ouput channel? 
		AutomaticInputOngoing 		: in std_logic;
		
		-- Request an output channel:
		AutomaticChannelRequired 	: out std_logic;
		
		AutomaticClock_ResetSignal : out std_logic_vector (1 downto 0);
		
		-- SPI Commands out...
		Data_Out 						: out std_logic_vector (29 downto 0);
		Data_Out_Valid					: out std_logic;
		
		-- Infos about the results... 
		Data_In							: in std_logic_vector (15 downto 0);
		Data_In_Valid					: in std_logic;
		Data_IN_Empty					: in std_logic;
		Data_IN_ReadEnable			: out std_logic;
		
		-- SPI FIFO Check
		SPI_FIFO_FULL 					: in std_logic;
		
		-- Reset the core 
		Reset 							: in std_logic;
		-- Clock to the core (125 MHz)
		clk 								: in STD_LOGIC
		);
end AutomaticCore;

architecture Behavioral of AutomaticCore is
	
	signal MyAutomaticChannelRequired : std_logic := '0';
	signal NeedAChannel : std_logic := '0';
	signal MyAutomaticClock_ResetSignal : std_logic_vector (1 downto 0) := "00";
	
	signal Counter : unsigned (2 downto 0) := "000";

	signal Poll_Start : STD_LOGIC := '0';
	signal Poll_Active : STD_LOGIC;	
	signal Poll_Active_Last : STD_LOGIC := '0';	
	signal AutomaticInputOngoing_Last : std_logic := '0';
	
begin
	AutomaticClock_ResetSignal <= MyAutomaticClock_ResetSignal;
	AutomaticChannelRequired <= MyAutomaticChannelRequired;
	
	-- Always read the FIFO (It is only for crossing the data from the 8MHz into the 125MHz domain)
	Data_IN_ReadEnable <= '1';
	
	MyWorkIdentifier : process (clk, Reset)
	begin

		if Reset = '1' then
			NeedAChannel <= '0';
			MyAutomaticChannelRequired <= '0';
			MyAutomaticClock_ResetSignal <= "00";
			
		elsif rising_edge(clk) then
		
			AutomaticInputOngoing_Last <= AutomaticInputOngoing;
			Poll_Active_Last <= Poll_Active;
			
			-- Detect a Clock Event 
			if (AutomaticClock_Event(1) = '0') AND (AutomaticClock_Event(0) = '1') then
				NeedAChannel <= '1';
			-- Detect the Reset Event
			elsif (MyAutomaticClock_ResetSignal(0) = '1') AND (MyAutomaticClock_ResetSignal(1) = '0') then
				NeedAChannel <= '0';
			else
			-- Stick to the last change 
				NeedAChannel <= NeedAChannel;
			end if;

			-- There was a clock event, now we need to do something with it...
			if NeedAChannel = '1' then
				-- There is no open channel, we need one
				if AutomaticInputOngoing = '0' then
					MyAutomaticChannelRequired <= '1';
				else
				-- We have the channel. Don't need to request it further			
					MyAutomaticChannelRequired <= '0';
				end if;
			else
				-- There is no need for a channel 
				MyAutomaticChannelRequired <= '0';
			end if;

			-- Start the AutomaticPoller
			if (AutomaticInputOngoing_Last = '0') AND (AutomaticInputOngoing = '1') then
				Poll_Start <= '1';
			elsif (Poll_Active_Last = '0') AND (Poll_Active = '1') then
				Poll_Start <= '0';
			else 
				Poll_Start <= Poll_Start;
			end if;

			-- Tell everybody that the AutomaticPoller is done
			if (Poll_Active_Last = '1') AND (Poll_Active = '0') then
				MyAutomaticClock_ResetSignal <= MyAutomaticClock_ResetSignal(0) & "1";
			else
				MyAutomaticClock_ResetSignal <= MyAutomaticClock_ResetSignal(0) & "0";
			end if;
		
		end if;
	end process MyWorkIdentifier;

	MyAutoPollerCore : entity work.AutoPollerCore 
    Port MAP ( 
				
				FIFO_IS_FULL 	=> SPI_FIFO_FULL,
				
				INPUT_DATA 		=> Data_In,
				INPUT_VALID		=> Data_In_Valid,
				
				OUTPUT_DATA 	=> Data_Out,
				OUTPUT_VALID 	=> Data_Out_Valid,
				
				Poll_Start => Poll_Start,
				Poll_Active => Poll_Active,
				
				clk => clk,
				Reset => Reset
			);

end architecture Behavioral;

