library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;
-- Errors:
-- 0: No Error
-- 1: Input Data too long
-- 2: Input Data too short
-- 3: Strange state related to the input data 
-- 4: CRC Error
-- 5: Zarlink Timeout Error

entity Command_BlockRead is
	 Generic (
		Module_Name : STD_LOGIC_VECTOR (15 downto 0) := (others => '0');
		Module_HasMemory : STD_LOGIC := '0';
		-- Needs to be at least 1 => 
		Module_LengthInWord : integer := 7;
		Module_UsesZarlink : STD_LOGIC := '1'
	 );
    Port ( 
		-- Connection to the first Input-Filter-Layer 
		In_Data						: in STD_LOGIC_VECTOR(15 downto 0);
		In_Valid 					: in STD_LOGIC;
		In_Wait						: out STD_LOGIC;
		
		-- More information about the packets 
		Packet_Start				: in STD_LOGIC;
		Packet_End					: in STD_LOGIC;
		Packet_Valid				: in STD_LOGIC;
		Packet_Unknown				: in STD_LOGIC;
		
		-- Connection to the Zarlink 
		Zarlink_Data_Out			: out STD_LOGIC_VECTOR(119 downto 0); -- Tri-State
		Zarlink_WriteEn			: out STD_LOGIC; -- Tri-State
		Zarlink_Data_In			: in 	STD_LOGIC_VECTOR(111 downto 0);
		Zarlink_Valid_In			: in 	STD_LOGIC;
		Zarlink_BlockMode			: out	STD_LOGIC; -- Tri-State 
		Zarlink_Wait				: in  STD_LOGIC;
		Zarlink_RequestChannel  : out STD_LOGIC;
		Zarlink_GetChannel		: in  STD_LOGIC;
		Zarlink_Name				: out STD_LOGIC_VECTOR (15 downto 0); -- Tri-State
		Zarlink_Error 				: in	STD_LOGIC;
		
		-- Connection to the Network
		Network_Data				: out STD_LOGIC_VECTOR(15 downto 0); -- Tri-State 
		Network_WriteEn			: out STD_LOGIC; -- Tri-State
		Network_Wait				: in  STD_LOGIC;
		Network_RequestChannel  : out STD_LOGIC; 
		Network_GetChannel		: in  STD_LOGIC;
		CRC_Reset					: out STD_LOGIC;
		CRC_00						: in 	STD_LOGIC_VECTOR(15 downto 0);
		CRC_01						: in 	STD_LOGIC_VECTOR(15 downto 0);
		CRC_10						: in 	STD_LOGIC_VECTOR(15 downto 0);
		CRC_11						: in 	STD_LOGIC_VECTOR(15 downto 0);

		-- The usual ports 
		Reset 						: in  STD_LOGIC;			
		Clk 							: in  STD_LOGIC
	 );
end Command_BlockRead;

architecture Behavioral of Command_BlockRead is

-- Handle the Tri-States 	
	-- Zarlink Outputs
	signal Zarlink_Data_Out_Internal	: STD_LOGIC_VECTOR(119 downto 0) := (others => '0');
	signal Zarlink_WriteEn_Internal : STD_LOGIC := '0';
	signal Zarlink_BlockMode_Internal : STD_LOGIC := '0';
	
	-- Network Outputs
	signal Network_Data_Internal : STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
	signal Network_WriteEn_Internal : STD_LOGIC := '0';
	signal Network_GetChannel_Internal : STD_LOGIC := '0';
	signal CRC_Reset_Internal : STD_LOGIC := '0';
	-- Internal Memory 
	signal Input_Memory : STD_LOGIC_VECTOR ((Module_LengthInWord*16)-1 downto 0) := (others => '0');
	signal Input_WordCounter : unsigned (15 downto 0) := (others => '0');
	
	signal ErrorState : STD_LOGIC_VECTOR (15 downto 0) := (others => '0');

	signal CRC_01_Copy : STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
	signal CRC_10_Copy : STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
	signal CRC_11_Copy : STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
	
	-- 0x55AA
	constant NET_HEADER : STD_LOGIC_VECTOR(15 downto 0) := std_logic_vector(to_unsigned(21930,16));
	-- 0x8000
	constant NET_RESPONSEID : STD_LOGIC_VECTOR(15 downto 0) := std_logic_vector(to_unsigned(32768,16));
	-- Fill the 14 Byte Block with dec13
	constant FILL_UP_BYTE : STD_LOGIC_VECTOR(7 downto 0) := std_logic_vector(to_unsigned(13,8));
	-- Not constant because the length can change 
	signal NET_LENGTH : STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
	
	type STATE_TYPE is (
		-- Reset State 
		STATE_RESET,
		-- Idle State 
		STATE_IDLE,
		-- The Packet Starts and we acquire the data from the stream
		STATE_GETDATA,
		-- Wait for the valid check 
		STATE_INPUTDATA_VALID,
		-- Request the Zalink Channel
		STATE_ZARLINK_PREPARE,
		-- Comunicate with the Zarlink
		STATE_ZARLINK_COMMUNICATE,
		STATE_ZARLINK_COMMUNICATE_WAIT,
		-- Prepare the network for transmission
		STATE_NETWORK_PREPARE,
		-- Communicate with the network (HEADER)
		STATE_NETWORK_COMMUNICATE_HEADER,
		-- Communicate with the network (LENGTH)
		STATE_NETWORK_COMMUNICATE_LENGTH,
		-- Communicate with the network (RESPONSEID)
		STATE_NETWORK_COMMUNICATE_RESPONSEID,
		-- Communicate with the network (ID)
		STATE_NETWORK_COMMUNICATE_ID,
		-- Communicate with the network (ERROR)
		STATE_NETWORK_COMMUNICATE_ERROR,
		-- Communicate with the network (BODY)
		STATE_NETWORK_COMMUNICATE_BODY,
		STATE_NETWORK_COMMUNICATE_BODY_2,
		STATE_NETWORK_COMMUNICATE_BODY_3,
		STATE_NETWORK_COMMUNICATE_BODY_4,
		STATE_NETWORK_COMMUNICATE_BODY_5,
		STATE_NETWORK_COMMUNICATE_BODY_6,
		STATE_NETWORK_COMMUNICATE_BODY_7,
		-- Communicate with the network - WAIT A
		STATE_NETWORK_COMMUNICATE_WAIT_A,
		-- Communicate with the network - WAIT B
		STATE_NETWORK_COMMUNICATE_WAIT_B,		
		-- Communicate with the network - WAIT C
		STATE_NETWORK_COMMUNICATE_WAIT_C,		
		-- Communicate with the network (CRC 00)
		STATE_NETWORK_COMMUNICATE_CRC00,
		-- Communicate with the network (CRC 01)
		STATE_NETWORK_COMMUNICATE_CRC01,
		-- Communicate with the network (CRC 10)
		STATE_NETWORK_COMMUNICATE_CRC10,
		-- Communicate with the network (CRC 11)
		STATE_NETWORK_COMMUNICATE_CRC11
	);
	signal FSM_STATE : STATE_TYPE;

	signal ZDOR	: STD_LOGIC_VECTOR(111 downto 0) := (others => '0');
	signal Z_RD : STD_LOGIC := '0';
	signal Z_Address : STD_LOGIC_VECTOR(6 downto 0) := (others => '0');
	signal Z_BM : STD_LOGIC := '0';
	signal Z_WE : STD_LOGIC := '0';
	signal Z_Done : STD_LOGIC := '0';
	
begin
-- Resort the Zarlink_Data_Out_Internal in Zarlink-Fashion
Zarlink_Data_Out_Internal <= 
		 ZDOR(104)&ZDOR(105)&ZDOR(106)&ZDOR(107)&ZDOR(108)&ZDOR(109)&ZDOR(110)&ZDOR(111)
		&ZDOR(96)&ZDOR(97)&ZDOR(98)&ZDOR(99)&ZDOR(100)&ZDOR(101)&ZDOR(102)&ZDOR(103)
		&ZDOR(88)&ZDOR(89)&ZDOR(90)&ZDOR(91)&ZDOR(92)&ZDOR(93)&ZDOR(94)&ZDOR(95)
		&ZDOR(80)&ZDOR(81)&ZDOR(82)&ZDOR(83)&ZDOR(84)&ZDOR(85)&ZDOR(86)&ZDOR(87)
		&ZDOR(72)&ZDOR(73)&ZDOR(74)&ZDOR(75)&ZDOR(76)&ZDOR(77)&ZDOR(78)&ZDOR(79)
		&ZDOR(64)&ZDOR(65)&ZDOR(66)&ZDOR(67)&ZDOR(68)&ZDOR(69)&ZDOR(70)&ZDOR(71)
		&ZDOR(56)&ZDOR(57)&ZDOR(58)&ZDOR(59)&ZDOR(60)&ZDOR(61)&ZDOR(62)&ZDOR(63)
		&ZDOR(48)&ZDOR(49)&ZDOR(50)&ZDOR(51)&ZDOR(52)&ZDOR(53)&ZDOR(54)&ZDOR(55)
		&ZDOR(40)&ZDOR(41)&ZDOR(42)&ZDOR(43)&ZDOR(44)&ZDOR(45)&ZDOR(46)&ZDOR(47)
		&ZDOR(32)&ZDOR(33)&ZDOR(34)&ZDOR(35)&ZDOR(36)&ZDOR(37)&ZDOR(38)&ZDOR(39)
		&ZDOR(24)&ZDOR(25)&ZDOR(26)&ZDOR(27)&ZDOR(28)&ZDOR(29)&ZDOR(30)&ZDOR(31)
		&ZDOR(16)&ZDOR(17)&ZDOR(18)&ZDOR(19)&ZDOR(20)&ZDOR(21)&ZDOR(22)&ZDOR(23)
		&ZDOR(8)&ZDOR(9)&ZDOR(10)&ZDOR(11)&ZDOR(12)&ZDOR(13)&ZDOR(14)&ZDOR(15)
		&ZDOR(0)&ZDOR(1)&ZDOR(2)&ZDOR(3)&ZDOR(4)&ZDOR(5)&ZDOR(6)&ZDOR(7)
		&Z_Address(0)&Z_Address(1)&Z_Address(2)&Z_Address(3)&Z_Address(4)&Z_Address(5)&Z_Address(6)&Z_RD;
Zarlink_BlockMode_Internal	<= Z_BM;
Zarlink_WriteEn_Internal <= Z_WE;

-- Tri - State Buffers: 
-- Zarlink Tri-State Output
Zarlink_Data_Out <= Zarlink_Data_Out_Internal when Zarlink_GetChannel = '1' else (others => 'Z');
Zarlink_WriteEn <= Zarlink_WriteEn_Internal when Zarlink_GetChannel = '1' else 'Z';
Zarlink_BlockMode <= Zarlink_BlockMode_Internal when Zarlink_GetChannel = '1' else 'Z';
Zarlink_Name <= Module_Name when Zarlink_GetChannel = '1' else (others => 'Z');

-- Network Tri-State Output
Network_Data <= Network_Data_Internal when Network_GetChannel_Internal = '1' else (others => 'Z');
Network_WriteEn <= Network_WriteEn_Internal when Network_GetChannel_Internal = '1' else 'Z';
CRC_Reset <= CRC_Reset_Internal when Network_GetChannel_Internal = '1' else 'Z';

	FSM : process(Reset,Clk)
	begin
		if (Reset = '1') then
			-- Default:
				ZDOR <= (others => '0');
				Z_RD <= '0';
				Z_Address <= (others => '0');						
				Z_BM <= '0';
				Z_WE <= '0';
				Z_Done <= '0';
				Zarlink_RequestChannel <= '0';
				
				Network_Data_Internal <= (others => '0');
				Network_WriteEn_Internal <= '0';
				Network_RequestChannel <= '0';
				CRC_Reset_Internal <= '0';
				
				ErrorState <= (others => '0');
				
				In_Wait <= '0';
				Input_Memory <= (others => '0');
				FSM_STATE <= STATE_RESET;
				Input_WordCounter <= (others => '0');

				CRC_01_Copy <= (others => '0');
				CRC_10_Copy <= (others => '0');
				CRC_11_Copy <= (others => '0');
				NET_LENGTH <= std_logic_vector(to_unsigned(32,16));
				Network_GetChannel_Internal <= '0';
		elsif rising_edge(Clk) then
				Network_GetChannel_Internal <= Network_GetChannel;
			-- Default:
				ZDOR <= (others => '0');
				Z_RD <= '0';
				Z_Address <= (others => '0');						
				Z_BM <= '0';
				Z_WE <= '0';
				Z_Done <= Z_Done;
				Zarlink_RequestChannel <= '0';
				
				Network_Data_Internal <= (others => '0');
				Network_WriteEn_Internal <= '0';
				Network_RequestChannel <= '0';
				CRC_Reset_Internal <= '0';
				
				ErrorState <= ErrorState;
				In_Wait <= '0';
				Input_Memory <= Input_Memory;
				Input_WordCounter <= (others => '0');
				CRC_01_Copy <= (others => '0');
				CRC_10_Copy <= (others => '0');
				CRC_11_Copy <= (others => '0');
				
				NET_LENGTH <= NET_LENGTH;
				
				case FSM_STATE is
					when STATE_RESET => 
						FSM_STATE <= STATE_IDLE;
						Input_Memory <= (others => '0');	
						ErrorState <= (others => '0');
					when STATE_IDLE => 
						
						-- Is there packet with my name? 
						if (Packet_Start = '1') AND (In_Valid = '1') AND (In_Data = Module_Name) then
							-- yes 
							FSM_STATE <= STATE_GETDATA;
						else
							-- no
							FSM_STATE <= STATE_IDLE;
						end if;
						Input_Memory <= (others => '0');	
						ErrorState <= (others => '0');
						
					-- There is a putative packet with my name... 
					-- Store my data !!!
					when STATE_GETDATA => 
						-- Is there valid data? 
						if In_Valid = '1' then
							-- Yes
							Input_WordCounter <= Input_WordCounter + 1;
							-- Can we still store data? 
							if (Input_WordCounter < to_unsigned(Module_LengthInWord,16)) AND (Module_HasMemory = '1') then
								-- Yes
								-- Then store the data !!!!
								Input_Memory <= In_Data & Input_Memory((Module_LengthInWord*16)-1 downto 16) ;
							else
								-- No
								Input_WordCounter <= Input_WordCounter;
								Input_Memory <= (others => '0');
							end if;
						else
							-- No
							Input_WordCounter <= Input_WordCounter;
						end if;
						
						-- Has the Packet ended? 
						if Packet_End = '0' then
							-- No
							FSM_STATE <= STATE_GETDATA;
						else
							In_Wait <= '1';
							-- Yes
							-- Was it valid?
							if (Packet_End = '1') AND (Packet_Valid = '1') AND 
								((Input_WordCounter = to_unsigned(Module_LengthInWord,16) ) OR Module_HasMemory = '0') then 
								-- Yes => Continue
								FSM_STATE <= STATE_INPUTDATA_VALID;
							else
								-- No
								-- I will report this!!!!
								if (Packet_Valid = '0') AND ((Input_WordCounter = to_unsigned(Module_LengthInWord,16))  OR Module_HasMemory = '0' ) then
									ErrorState <= std_logic_vector(to_unsigned(4,16));
								elsif Input_WordCounter > to_unsigned(Module_LengthInWord,16) then 
									ErrorState <= std_logic_vector(to_unsigned(1,16));
								elsif Input_WordCounter < to_unsigned(Module_LengthInWord,16) then
									ErrorState <= std_logic_vector(to_unsigned(2,16));
								else
									ErrorState <= std_logic_vector(to_unsigned(3,16));
								end if;
								FSM_STATE <= STATE_NETWORK_PREPARE;
							end if;
						end if;
					
					-- We have a valid packet!!!
					-- Lets do something with it. 
					-- Do Network or Zarlink...
					when STATE_INPUTDATA_VALID	=>
						
						----------------------------------------
						-- Here we can assign some variables... 
						----------------------------------------
						
						if Module_UsesZarlink = '1' then
							FSM_STATE <= STATE_ZARLINK_PREPARE;
						else
							FSM_STATE <= STATE_NETWORK_PREPARE;
						end if;
						
						In_Wait <= '1';
					
					--	We need the Zarlink
					-- So... Lets get it. 
					when STATE_ZARLINK_PREPARE =>
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';
						if Zarlink_GetChannel = '1' then
							-- We have the channel to the zarlink. 
							FSM_STATE <= STATE_ZARLINK_COMMUNICATE;
						else
							-- Continue waiting
							FSM_STATE <= STATE_ZARLINK_PREPARE;
						end if;
						Z_Done <= '0';
						
					-- We have the link to the Zarlink
					-- Lets comunicate...
					when STATE_ZARLINK_COMMUNICATE =>
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';
						if (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							FSM_STATE <= STATE_ZARLINK_COMMUNICATE_WAIT;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_ZARLINK_COMMUNICATE;
							Z_Address <= std_logic_vector(to_unsigned(45,7));
							ZDOR <= (others => '0');
							Z_RD <= '1';
							Z_BM <= '1';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_ZARLINK_COMMUNICATE;						
						end if;
					
					when STATE_ZARLINK_COMMUNICATE_WAIT	=>
						In_Wait <= '1';
						Zarlink_RequestChannel <= '1';					
						if (Zarlink_Valid_In = '0') AND (Zarlink_Error = '0') then
							FSM_STATE <= STATE_ZARLINK_COMMUNICATE_WAIT;
						else
							Input_Memory <= Zarlink_Data_In;
							if Zarlink_Error = '1' then
								ErrorState <= std_logic_vector(to_unsigned(5,16));
							else
								NULL;
							end if;
							FSM_STATE <= STATE_NETWORK_PREPARE;
														
						end if;
					
					--	We need the Network
					-- So... Lets get it. 
					when STATE_NETWORK_PREPARE => 
						In_Wait <= '1';
						Network_RequestChannel <= '1';
						if Network_GetChannel_Internal = '1' then
							-- We have the channel to the network.
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_HEADER;
							CRC_Reset_Internal <= '1';
						else
							-- Continue waiting
							FSM_STATE <= STATE_NETWORK_PREPARE;
						end if;	
					
					-- Communicate with the network (HEADER)
					when STATE_NETWORK_COMMUNICATE_HEADER =>
						In_Wait <= '1';
						Network_RequestChannel <= '1';					
						if Network_Wait = '0' then
							Network_Data_Internal <= NET_HEADER;
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_LENGTH;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_HEADER;
						end if;
						
					-- Communicate with the network (LENGTH)
					when STATE_NETWORK_COMMUNICATE_LENGTH =>
						In_Wait <= '1';
						Network_RequestChannel <= '1';						
						if Network_Wait = '0' then
							Network_Data_Internal <= NET_LENGTH;
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_RESPONSEID;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_LENGTH;
						end if;						
					
					-- Communicate with the network (RESPONSEID)
					when STATE_NETWORK_COMMUNICATE_RESPONSEID => 
						In_Wait <= '1';
						Network_RequestChannel <= '1';						
						if Network_Wait = '0' then
							Network_Data_Internal <= NET_RESPONSEID;
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_ID;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_RESPONSEID;
						end if;
						
					-- Communicate with the network (ID)
					when STATE_NETWORK_COMMUNICATE_ID =>
						In_Wait <= '1';
						Network_RequestChannel <= '1';						
						if Network_Wait = '0' then
							Network_Data_Internal <= Module_Name;
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_ERROR;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_ID;
						end if;
					
					-- Communicate with the network (ERROR)
					when STATE_NETWORK_COMMUNICATE_ERROR => 
						In_Wait <= '1';
						Network_RequestChannel <= '1';						
						if Network_Wait = '0' then
							Network_Data_Internal <= ErrorState;
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_BODY;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_ERROR;
						end if;
						
					-- Communicate with the network (BODY)
					when STATE_NETWORK_COMMUNICATE_BODY =>
						In_Wait <= '1';
						Network_RequestChannel <= '1';						
						if Network_Wait = '0' then
							Network_Data_Internal <= Input_Memory(111 downto 104) & Input_Memory(103 downto 96) ;
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_BODY_2;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_BODY;
						end if;

					when STATE_NETWORK_COMMUNICATE_BODY_2 =>
						In_Wait <= '1';
						Network_RequestChannel <= '1';						
						if Network_Wait = '0' then
							Network_Data_Internal <= Input_Memory(95 downto 88) & Input_Memory(87 downto 80);
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_BODY_3;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_BODY_2;
						end if;

					when STATE_NETWORK_COMMUNICATE_BODY_3 =>
						In_Wait <= '1';
						Network_RequestChannel <= '1';						
						if Network_Wait = '0' then
							Network_Data_Internal <= Input_Memory(79 downto 72) & Input_Memory(71 downto 64);
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_BODY_4;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_BODY_3;
						end if;

					when STATE_NETWORK_COMMUNICATE_BODY_4 =>
						In_Wait <= '1';
						Network_RequestChannel <= '1';						
						if Network_Wait = '0' then
							Network_Data_Internal <= Input_Memory(63 downto 56) & Input_Memory(55 downto 48);
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_BODY_5;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_BODY_4;
						end if;

					when STATE_NETWORK_COMMUNICATE_BODY_5 =>
						In_Wait <= '1';
						Network_RequestChannel <= '1';						
						if Network_Wait = '0' then
							Network_Data_Internal <= Input_Memory(47 downto 40) & Input_Memory(39 downto 32);
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_BODY_6;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_BODY_5;
						end if;

					when STATE_NETWORK_COMMUNICATE_BODY_6 =>
						In_Wait <= '1';
						Network_RequestChannel <= '1';						
						if Network_Wait = '0' then
							Network_Data_Internal <= Input_Memory(31 downto 24) & Input_Memory(23 downto 16);
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_BODY_7;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_BODY_6;
						end if;


					when STATE_NETWORK_COMMUNICATE_BODY_7 =>
						In_Wait <= '1';
						Network_RequestChannel <= '1';						
						if Network_Wait = '0' then
							Network_Data_Internal <= Input_Memory(15 downto 8) & Input_Memory(7 downto 0);
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_WAIT_A;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_BODY_7;
						end if;
					
					-- Wait for the CRC - Values I.
					when STATE_NETWORK_COMMUNICATE_WAIT_A =>
						In_Wait <= '1';
						Network_RequestChannel <= '1';						
						FSM_STATE <= STATE_NETWORK_COMMUNICATE_WAIT_B;

					-- Wait for the CRC - Values II.
					when STATE_NETWORK_COMMUNICATE_WAIT_B =>
						In_Wait <= '1';
						Network_RequestChannel <= '1';	
						FSM_STATE <= STATE_NETWORK_COMMUNICATE_WAIT_C;						
					
					-- Wait for the CRC - Values III.
					when STATE_NETWORK_COMMUNICATE_WAIT_C =>
						In_Wait <= '1';
						Network_RequestChannel <= '1';						
						FSM_STATE <= STATE_NETWORK_COMMUNICATE_CRC00;
						
					-- Communicate with the network (CRC 00)
					when STATE_NETWORK_COMMUNICATE_CRC00 => 
						In_Wait <= '1';
						Network_RequestChannel <= '1';						
						CRC_01_Copy <= CRC_01;
						CRC_10_Copy <= CRC_10;
						CRC_11_Copy <= CRC_11;
						
						if Network_Wait = '0' then
							Network_Data_Internal <= CRC_00;
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_CRC01;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_CRC00;
						end if;
						
					-- Communicate with the network (CRC 01)
					when STATE_NETWORK_COMMUNICATE_CRC01 =>
						In_Wait <= '1';
						Network_RequestChannel <= '1';						
						CRC_01_Copy <= CRC_01_Copy;
						CRC_10_Copy <= CRC_10_Copy;
						CRC_11_Copy <= CRC_11_Copy;		
						
						if Network_Wait = '0' then
							Network_Data_Internal <= CRC_01_Copy;
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_CRC10;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_CRC01;
						end if;
						
					-- Communicate with the network (CRC 10)
					when STATE_NETWORK_COMMUNICATE_CRC10 => 
						In_Wait <= '1';
						Network_RequestChannel <= '1';						
						CRC_10_Copy <= CRC_10_Copy;
						CRC_11_Copy <= CRC_11_Copy;	
						if Network_Wait = '0' then
							Network_Data_Internal <= CRC_10_Copy;
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_CRC11;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_CRC10;
						end if;
						
					-- Communicate with the network (CRC 11)
					when STATE_NETWORK_COMMUNICATE_CRC11 => 
						In_Wait <= '1';
						Network_RequestChannel <= '1';						

						CRC_11_Copy <= CRC_11_Copy;		
						
						if Network_Wait = '0' then
							Network_Data_Internal <= CRC_11_Copy;
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_IDLE;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_CRC11;
						end if;						
						
					when others => 
						FSM_STATE <= STATE_RESET;
				
				end case;
		

		end if;
	end process FSM;

end architecture Behavioral;

