library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;
-- Errors:
-- 0: No Error
-- 1: Input Data too long
-- 2: Input Data too short
-- 3: Strange state related to the input data 
-- 4: CRC Error
-- 6: Unkown Packet Typ

entity Command_Unknown is
    Port ( 
		-- Connection to the first Input-Filter-Layer 
		In_Data						: in STD_LOGIC_VECTOR(15 downto 0);
		In_Valid 					: in STD_LOGIC;
		In_Wait						: out STD_LOGIC;
		
		-- More information about the packets 
		Packet_Start				: in STD_LOGIC;
		Packet_End					: in STD_LOGIC;
		Packet_Valid				: in STD_LOGIC;
		Packet_Unknown				: in STD_LOGIC;
		
		-- Connection to the Network
		Network_Data				: out STD_LOGIC_VECTOR(15 downto 0); -- Tri-State 
		Network_WriteEn			: out STD_LOGIC; -- Tri-State
		Network_Wait				: in  STD_LOGIC;
		Network_RequestChannel  : out STD_LOGIC; 
		Network_GetChannel		: in  STD_LOGIC;
		CRC_Reset					: out STD_LOGIC;
		CRC_00						: in 	STD_LOGIC_VECTOR(15 downto 0);
		CRC_01						: in 	STD_LOGIC_VECTOR(15 downto 0);
		CRC_10						: in 	STD_LOGIC_VECTOR(15 downto 0);
		CRC_11						: in 	STD_LOGIC_VECTOR(15 downto 0);

		-- The usual ports 
		Reset 						: in  STD_LOGIC;			
		Clk 							: in  STD_LOGIC
	 );
end Command_Unknown;

architecture Behavioral of Command_Unknown is

-- Handle the Tri-States 	

	
	-- Network Outputs
	signal Network_Data_Internal : STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
	signal Network_WriteEn_Internal : STD_LOGIC := '0';
	signal Network_GetChannel_Internal : STD_LOGIC := '0';
	signal CRC_Reset_Internal : STD_LOGIC := '0';
	-- Internal Memory 
	signal Input_Memory : STD_LOGIC_VECTOR (15 downto 0) := (others => '0');

	
	constant ErrorState : STD_LOGIC_VECTOR (15 downto 0) := std_logic_vector(to_unsigned(6,16)); 

	signal CRC_01_Copy : STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
	signal CRC_10_Copy : STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
	signal CRC_11_Copy : STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
	
	-- 0x55AA
	constant NET_HEADER : STD_LOGIC_VECTOR(15 downto 0) := std_logic_vector(to_unsigned(21930,16));
	-- 0x8000
	constant NET_RESPONSEID : STD_LOGIC_VECTOR(15 downto 0) := std_logic_vector(to_unsigned(32768,16));
	-- Not constant because the length can change 
	signal NET_LENGTH : STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
	
	type STATE_TYPE is (
		-- Reset State 
		STATE_RESET,
		-- Idle State 
		STATE_IDLE,
		-- Wait for the valid check 
		STATE_INPUTDATA_VALID,
		-- Prepare the network for transmission
		STATE_NETWORK_PREPARE,
		-- Communicate with the network (HEADER)
		STATE_NETWORK_COMMUNICATE_HEADER,
		-- Communicate with the network (LENGTH)
		STATE_NETWORK_COMMUNICATE_LENGTH,
		-- Communicate with the network (RESPONSEID)
		STATE_NETWORK_COMMUNICATE_RESPONSEID,
		-- Communicate with the network (ID)
		STATE_NETWORK_COMMUNICATE_ID,
		-- Communicate with the network (ERROR)
		STATE_NETWORK_COMMUNICATE_ERROR,
		-- Communicate with the network (BODY)
		STATE_NETWORK_COMMUNICATE_BODY,
		-- Communicate with the network - WAIT A
		STATE_NETWORK_COMMUNICATE_WAIT_A,
		-- Communicate with the network - WAIT B
		STATE_NETWORK_COMMUNICATE_WAIT_B,		
		-- Communicate with the network - WAIT C
		STATE_NETWORK_COMMUNICATE_WAIT_C,		
		-- Communicate with the network (CRC 00)
		STATE_NETWORK_COMMUNICATE_CRC00,
		-- Communicate with the network (CRC 01)
		STATE_NETWORK_COMMUNICATE_CRC01,
		-- Communicate with the network (CRC 10)
		STATE_NETWORK_COMMUNICATE_CRC10,
		-- Communicate with the network (CRC 11)
		STATE_NETWORK_COMMUNICATE_CRC11
	);
	signal FSM_STATE : STATE_TYPE;

	
begin

-- Network Tri-State Output
Network_Data <= Network_Data_Internal when Network_GetChannel_Internal = '1' else (others => 'Z');
Network_WriteEn <= Network_WriteEn_Internal when Network_GetChannel_Internal = '1' else 'Z';
CRC_Reset <= CRC_Reset_Internal when Network_GetChannel_Internal = '1' else 'Z';

	FSM : process(Reset,Clk)
	begin
		if (Reset = '1') then
			-- Default:
				Network_Data_Internal <= (others => '0');
				Network_WriteEn_Internal <= '0';
				Network_RequestChannel <= '0';
				CRC_Reset_Internal <= '0';
				
				In_Wait <= '0';
				Input_Memory <= (others => '0');
				FSM_STATE <= STATE_RESET;

				CRC_01_Copy <= (others => '0');
				CRC_10_Copy <= (others => '0');
				CRC_11_Copy <= (others => '0');
				NET_LENGTH <= std_logic_vector(to_unsigned(18,16));
				Network_GetChannel_Internal <= '0';
		elsif rising_edge(Clk) then
				Network_GetChannel_Internal <= Network_GetChannel;
			-- Default:
				
				Network_Data_Internal <= (others => '0');
				Network_WriteEn_Internal <= '0';
				Network_RequestChannel <= '0';
				CRC_Reset_Internal <= '0';

				In_Wait <= '0';
				Input_Memory <= Input_Memory;

				CRC_01_Copy <= (others => '0');
				CRC_10_Copy <= (others => '0');
				CRC_11_Copy <= (others => '0');
				
				NET_LENGTH <= NET_LENGTH;
				
				
				case FSM_STATE is
					when STATE_RESET => 
						FSM_STATE <= STATE_IDLE;
						Input_Memory <= (others => '0');	

					when STATE_IDLE => 
						
						-- Is there a packet? 
						if Packet_Unknown = '1' then
							-- yes 
							FSM_STATE <= STATE_NETWORK_PREPARE;
							Input_Memory <= In_Data;
						else
							-- no
							FSM_STATE <= STATE_IDLE;
							Input_Memory <= (others => '0');	
						end if;
					
						
					--	We need the Network
					-- So... Lets get it. 
					when STATE_NETWORK_PREPARE => 
						In_Wait <= '1';
						Network_RequestChannel <= '1';
						if Network_GetChannel_Internal = '1' then
							-- We have the channel to the network.
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_HEADER;
							CRC_Reset_Internal <= '1';
						else
							-- Continue waiting
							FSM_STATE <= STATE_NETWORK_PREPARE;
						end if;	
					
					-- Communicate with the network (HEADER)
					when STATE_NETWORK_COMMUNICATE_HEADER =>
						In_Wait <= '1';
						Network_RequestChannel <= '1';					
						if Network_Wait = '0' then
							Network_Data_Internal <= NET_HEADER;
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_LENGTH;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_HEADER;
						end if;
						
					-- Communicate with the network (LENGTH)
					when STATE_NETWORK_COMMUNICATE_LENGTH =>
						In_Wait <= '1';
						Network_RequestChannel <= '1';						
						if Network_Wait = '0' then
							Network_Data_Internal <= NET_LENGTH;
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_RESPONSEID;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_LENGTH;
						end if;						
					
					-- Communicate with the network (RESPONSEID)
					when STATE_NETWORK_COMMUNICATE_RESPONSEID => 
						In_Wait <= '1';
						Network_RequestChannel <= '1';						
						if Network_Wait = '0' then
							Network_Data_Internal <= NET_RESPONSEID;
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_ID;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_RESPONSEID;
						end if;
						
					-- Communicate with the network (ID)
					when STATE_NETWORK_COMMUNICATE_ID =>
						In_Wait <= '1';
						Network_RequestChannel <= '1';						
						if Network_Wait = '0' then
							Network_Data_Internal <= Input_Memory;
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_ERROR;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_ID;
						end if;
					
					-- Communicate with the network (ERROR)
					when STATE_NETWORK_COMMUNICATE_ERROR => 
						In_Wait <= '1';
						Network_RequestChannel <= '1';						
						if Network_Wait = '0' then
							Network_Data_Internal <= ErrorState;
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_BODY;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_ERROR;
						end if;
						
					-- Communicate with the network (BODY)
					when STATE_NETWORK_COMMUNICATE_BODY =>
						In_Wait <= '1';
						Network_RequestChannel <= '1';						
						FSM_STATE <= STATE_NETWORK_COMMUNICATE_WAIT_A;
						-- Do your worst... 
					
					-- Wait for the CRC - Values I.
					when STATE_NETWORK_COMMUNICATE_WAIT_A =>
						In_Wait <= '1';
						Network_RequestChannel <= '1';						
						FSM_STATE <= STATE_NETWORK_COMMUNICATE_WAIT_B;

					-- Wait for the CRC - Values II.
					when STATE_NETWORK_COMMUNICATE_WAIT_B =>
						In_Wait <= '1';
						Network_RequestChannel <= '1';	
						FSM_STATE <= STATE_NETWORK_COMMUNICATE_WAIT_C;						
					
					-- Wait for the CRC - Values III.
					when STATE_NETWORK_COMMUNICATE_WAIT_C =>
						In_Wait <= '1';
						Network_RequestChannel <= '1';						
						FSM_STATE <= STATE_NETWORK_COMMUNICATE_CRC00;
						
					-- Communicate with the network (CRC 00)
					when STATE_NETWORK_COMMUNICATE_CRC00 => 
						In_Wait <= '1';
						Network_RequestChannel <= '1';						
						CRC_01_Copy <= CRC_01;
						CRC_10_Copy <= CRC_10;
						CRC_11_Copy <= CRC_11;
						
						if Network_Wait = '0' then
							Network_Data_Internal <= CRC_00;
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_CRC01;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_CRC00;
						end if;
						
					-- Communicate with the network (CRC 01)
					when STATE_NETWORK_COMMUNICATE_CRC01 =>
						In_Wait <= '1';
						Network_RequestChannel <= '1';						
						CRC_01_Copy <= CRC_01_Copy;
						CRC_10_Copy <= CRC_10_Copy;
						CRC_11_Copy <= CRC_11_Copy;		
						
						if Network_Wait = '0' then
							Network_Data_Internal <= CRC_01_Copy;
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_CRC10;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_CRC01;
						end if;
						
					-- Communicate with the network (CRC 10)
					when STATE_NETWORK_COMMUNICATE_CRC10 => 
						In_Wait <= '1';
						Network_RequestChannel <= '1';						
						CRC_10_Copy <= CRC_10_Copy;
						CRC_11_Copy <= CRC_11_Copy;	
						if Network_Wait = '0' then
							Network_Data_Internal <= CRC_10_Copy;
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_CRC11;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_CRC10;
						end if;
						
					-- Communicate with the network (CRC 11)
					when STATE_NETWORK_COMMUNICATE_CRC11 => 
						In_Wait <= '1';
						Network_RequestChannel <= '1';						

						CRC_11_Copy <= CRC_11_Copy;		
						
						if Network_Wait = '0' then
							Network_Data_Internal <= CRC_11_Copy;
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_IDLE;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_CRC11;
						end if;						
						
					when others => 
						FSM_STATE <= STATE_RESET;
				
				end case;
		

		end if;
	end process FSM;

end architecture Behavioral;

