----------------------------------------------------------------------------------
-- Company: Uni Bremen ITP
-- Engineer: David Rotermund 
-- 
-- Create Date:    11:23:57 02/21/2011 
-- Design Name: 	
-- Module Name:    DataProzessingCore - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;

-- Core for handling incomming and outgoing data between the network core (16 bit) and the SPI Interface Core (30 bit in / 16 bit out)

-- Important Infos: 

-- TheInput_Data:
-- (least significant bit is one the right hand side)
-- Transfertyp : "000000" (Read/Write one Byte) | "000001" (Read/Write first Byte of a block)
-- A= Adress , B = First Bit , C = Second Bit (not used yet, e.g. for 8 Bit Addresses) 
-- String 0:
-- B1 | A7(RD) A6 A5 A4 A3 A2 A1 A0 | T5 T4 T3 T2 T1 T0 | Page0(=0)
-- String 1:
-- B1 C7 C6 C5 C4 C3 C2 C1 C0 | B7 B6 B5 B4 B3 B2  | Page1(=1)

-- Transfertyp : "000010" (Read/Write Bytes in the middle of a block) | "000011" (Read/Write last Byte of a block)
-- A= Adress , B = First Bit , C = Second Bit (not used yet, e.g. for 8 Bit Addresses) 
-- String 0:
-- A= Adress (no used) , B = First Bit , C = Second Bit (not used) 
-- C1 | B7 B6 B5 B4 B3 B2 B1 B0 | T5 T4 T3 T2 T1 T0 | Page0(=0)

-- TheOutput_Data:
-- A= Adress (no used) , B = First Bit 
-- B7 B6 B5 B4 B3 B2 B1 B0 | T5 T4 T3 T2 T1 T0 | 0 0


entity DataProzessingCore is
    Port ( 
		-- The Ports
			SPI_ZL_CS 				: out std_logic; 								-- SPI Enable
			SPI_SDI 					: out std_logic; 								-- SPI Data input into the Zarlink 
			SPI_SDO 					: in std_logic; 								-- SPI Data from the Zarlink
			SPI_CLK 					: out std_logic;								-- SPI Clock (4MHz)
			
		-- The Input 	
			TheInput_Data 			: in std_logic_vector(15 downto 0);		-- Data from the network core
			TheInput_Valid 		: in std_logic;								-- Data from the network core is valid (=1)
			TheInput_IsAlmostFull : out std_logic;								 
			TheInput_ProgFull 	: out std_logic;								-- Less than one network frame free?
		-- The Output 	
			TheOutput_Data 		: out std_logic_vector(15 downto 0);	-- Data to the network core
			TheOutput_Done 		: in std_logic;								-- Read enable (=1)
			TheOutput_Empty		: out std_logic;
			TheOutput_Valid		: out std_logic;
			
		-- Rest 
			Reset 					: in std_logic;								-- Reset port (reset = 1)
		
		--	The Clocks
			Clk8MHz					: in std_LOGIC; 								-- 8 MHz Clock
			clk 						: in STD_LOGIC 								-- 125 MHz (or >> 8MHz) for the external fifo
	 );
end entity DataProzessingCore;

architecture Behavioral of DataProzessingCore is

	-- Fifo for dealing with the incoming data from the network core
	component EchoFIFO
	port (
		clk: in std_logic;
		rst: in std_logic;
		din: in std_logic_vector(15 downto 0);
		wr_en: in std_logic;
		rd_en: in std_logic;
		dout: out std_logic_vector(15 downto 0);
		full: out std_logic;
		almost_full: out std_logic;
		empty: out std_logic;
		valid: out std_logic;
		overflow : OUT STD_LOGIC;
		prog_full : OUT STD_LOGIC
	);
	end component EchoFIFO;

	-- Fifo for the repacked data for the SPI interface core 
	component FIFO_30Bit_1000BlockCheck
	port (
		rst: in std_logic;
		wr_clk: in std_logic;
		rd_clk: in std_logic;
		din: in std_logic_vector(29 downto 0);
		wr_en: in std_logic;
		rd_en: in std_logic;
		dout: out std_logic_vector(29 downto 0);
		full: out std_logic;
		empty: out std_logic;
		valid: out std_logic;
		prog_full: out std_logic
	);
	end component FIFO_30Bit_1000BlockCheck;

	-- Fifo for the Output data (to the network core)
	component FIFO_16Bit_1000BlockCheck
	port (
		rst: in std_logic;
		wr_clk: in std_logic;
		rd_clk: in std_logic;
		din: in std_logic_vector(15 downto 0);
		wr_en: in std_logic;
		rd_en: in std_logic;
		dout: out std_logic_vector(15 downto 0);
		full: out std_logic;
		empty: out std_logic;
		valid: out std_logic;
		prog_full: out std_logic
	);
	end component FIFO_16Bit_1000BlockCheck;

	component AutoPoller_FIFO_16Bit 
	PORT (
		rst : IN STD_LOGIC;
		wr_clk : IN STD_LOGIC;
		rd_clk : IN STD_LOGIC;
		din : IN STD_LOGIC_VECTOR(15 DOWNTO 0);
		wr_en : IN STD_LOGIC;
		rd_en : IN STD_LOGIC;
		dout : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
		full : OUT STD_LOGIC;
		empty : OUT STD_LOGIC;
		valid : OUT STD_LOGIC
	);
	END component AutoPoller_FIFO_16Bit;

	-- FIFO IN
	signal FIFO_IN_ENABLE : std_logic := '0';
	signal FIFO_IN_DATAOUT : std_logic_vector(15 downto 0) := (others => '0');
	signal FIFO_IN_IsFull : std_logic := '0';
	signal FIFO_IN_IsAlmostFull : std_logic := '0';	
	signal FIFO_IN_IsEmpty : std_logic := '0';
	signal FIFO_IN_IsValid : std_logic := '0';
	signal FIFO_IN_overflow : STD_LOGIC := '0';


	-- FIFO OUT
	signal FIFO_OUT_DATAIN : std_logic_vector(15 downto 0) := (others => '0');
	signal FIFO_OUT_DATAIN_Valid : std_logic := '0';
	signal FIFO_OUT_IsFull : std_logic := '0';
	signal FIFO_OUT_IsEmpty : std_logic := '0';
	signal FIFO_OUT_IsValid : std_logic := '0';
	signal FIFO_OUT_1000BLOCKTOFULLL : std_logic := '0';

	-- SPI FIFO
	signal SPI_FIFO_READENABLE : std_logic := '0';
	signal SPI_FIFO_DATAOUT : std_logic_vector(29 downto 0) := (others => '0');
	signal SPI_FIFO_FULL : std_logic := '0';
	signal SPI_FIFO_EMPTY : std_logic := '0';
	signal SPI_FIFO_DATAOUTVALID : std_logic := '0';
	signal SPI_FIFO_1000BLOCKTOFULLL : std_logic := '0';
		
	-- Puzzle the data... 
	signal SPI_WriteStreamAway : std_logic := '0';
	signal SPI_IsFirstSegment : std_logic := '0';
	signal SPI_Data : std_logic_vector(29 downto 0);
	signal SPI_TransmitType : std_logic_vector(5 downto 0) := (others => '0');
	
	signal ClockCounter : unsigned (127 downto 0) := (others => '0');
	constant ClockCounterCycles : unsigned (127 downto 0) := to_unsigned(1,128);
	signal ClockSlow_1MHz : std_logic := '0';
	signal ClockSlow_2MHz : std_logic := '0';
	signal ClockSelected : std_logic := '0';
	signal ClockMode : std_logic_vector (1 downto 0) := "00";
	
	-- Automatic Clock 
	signal AutomaticClock_CycleLength : std_logic_vector (46 downto 0) := (others => '0');
	signal AutomaticClock_SegmentA : std_logic := '0';
	signal AutomaticClock_SegmentB : std_logic := '0';
	signal AutomaticClock_Counter : unsigned (48 downto 0) := (others => '0');
	signal AutomaticClock_Event : std_logic_vector (1 downto 0) := "00";
	signal AutomaticClock_ResetSignal : std_logic_vector (1 downto 0) := "00";
	
	-- Arbiter 
	signal PrimaryInputOngoing :std_logic := '0';
	signal AutomaticInputOngoing :std_logic := '0';
	signal AutomaticChannelRequired : std_logic := '0';
	
	-- Automatic Core 
	signal AutomaticCore_Data_Out : std_logic_vector (29 downto 0) := (others => '0');
	signal AutomaticCore_Data_Out_Valid : std_logic := '0';	

	-- Automatic FIFO
	signal MyAutoPoller_FIFO_rd_en : STD_LOGIC;
	signal MyAutoPoller_FIFO_dout : STD_LOGIC_VECTOR(15 DOWNTO 0);
	signal MyAutoPoller_FIFO_full : STD_LOGIC;
	signal MyAutoPoller_FIFO_empty : STD_LOGIC;
	signal MyAutoPoller_FIFO_valid : STD_LOGIC;	
	
	signal AutomaticCore_Enable : std_logic := '0';

begin
	TheOutput_Empty <= FIFO_OUT_IsEmpty;

	
	-- Ask the network core to collect all data from the output fifo
	TheOutput_Valid <= FIFO_OUT_IsValid;
--	-- Allows the routing of the selected clock signal 
	ClockModeProcess : process(ClockMode,ClockSlow_1MHz,ClockSlow_2MHz,Clk8MHz) 
	begin
		-- 3:1 Mux
		case ClockMode is
			-- 1 MHz SPI clock signal
			when "00" => 
				ClockSelected <= ClockSlow_1MHz;
			when "01" => 
			-- 2 MHz SPI clock signal
				ClockSelected <= ClockSlow_2MHz;
			when others =>
			-- 4 MHz SPI clock signal
				ClockSelected <= Clk8MHz;
		end case; 
	end process ClockModeProcess;
	
	AutomaticClockProcess : process (clk,Reset)
	begin
		-- On reset 
		if Reset = '1' then
			-- Empty the counter 
			AutomaticClock_Counter <= (others => '0');
			-- There is no clock event
			AutomaticClock_Event <= (others => '0');
		elsif rising_edge(clk) then
			-- Is there a clock value set or is the clock disabled by zeros
			if (unsigned(AutomaticClock_CycleLength) /= to_unsigned(0,47)) AND (AutomaticCore_Enable = '1') then
			
				-- Reset value of the Counter Cycle not reached yet? 
				if AutomaticClock_Counter < resize(unsigned(AutomaticClock_CycleLength),49) then
					-- Bounderies not meet yet => No event, add one to the counter
					AutomaticClock_Event <= AutomaticClock_Event(0) & '0';
					AutomaticClock_Counter <= AutomaticClock_Counter + 1;
				else
					-- Reset value of the Counter Cycle was reached!
					if (AutomaticClock_ResetSignal(0) = '1') AND (AutomaticClock_ResetSignal(1) = '0')  then
						-- Please reset the counter
						AutomaticClock_Counter <= (others => '0');
					else
						-- Waiting until the other process is done 
						AutomaticClock_Counter <= AutomaticClock_Counter;
					end if;
					-- Hold the Event Alive 
					AutomaticClock_Event <= AutomaticClock_Event(0) & '1';
				end if;
			else
				-- There is noting to do... No Automatic counter configured 
				AutomaticClock_Counter <= (others => '0');
				AutomaticClock_Event <= AutomaticClock_Event(0) & '0';
			end if;
		end if;
	end process AutomaticClockProcess;

	-- Breaking down the clock to a 1MHz SPI clock (means 2 MHz raw clock)
	ClockDown_1MHz : process (Clk8MHz,Reset)
	begin
		if Reset = '1' then
			ClockSlow_1MHz <= '0';
			ClockCounter <= (others => '0');
			
		elsif rising_edge(Clk8MHz) then
		
			if ClockCounter < ClockCounterCycles then
				ClockCounter <= ClockCounter + 1;
				ClockSlow_1MHz <= ClockSlow_1MHz;
			else
				ClockCounter <= (others => '0');
				ClockSlow_1MHz <= NOT ClockSlow_1MHz;
			end if;
			
		
		end if;
	end process ClockDown_1MHz;
	
	-- Breaking down the clock to a 2MHz SPI clock (means 4 MHz raw clock)
	ClockDown_2MHz : process (Clk8MHz,Reset)
		begin
			if Reset = '1' then
				ClockSlow_2MHz <= '0';
			elsif rising_edge(Clk8MHz) then
				ClockSlow_2MHz <= NOT ClockSlow_2MHz;
			end if;
		end process ClockDown_2MHz;


	-- Make from two Inputs one but in a correct way... 
	SortIt : process (clk,Reset)
		variable SPI_TransmitType_VAR : std_logic_vector(5 downto 0) := (others => '0');
	begin
		if Reset = '1' then
			SPI_WriteStreamAway <= '0';
			SPI_IsFirstSegment <= '0';
			FIFO_IN_ENABLE <= '0';
			SPI_Data <= (others => '0');
			SPI_TransmitType <= (others => '0');		
			ClockMode <= "11";
			AutomaticClock_SegmentA <= '0';
			AutomaticClock_SegmentB <= '0';
			AutomaticClock_CycleLength <= (others => '0');
			PrimaryInputOngoing <= '0';
			AutomaticInputOngoing <= '0';
			AutomaticCore_Enable <= '0';
			
		elsif rising_edge(clk) then
			ClockMode <= ClockMode;
			AutomaticClock_SegmentA <= AutomaticClock_SegmentA;
			AutomaticClock_SegmentB <= AutomaticClock_SegmentB;
			AutomaticClock_CycleLength <= AutomaticClock_CycleLength;
			AutomaticCore_Enable <= AutomaticCore_Enable;
			-- (FIFO_IN_ENABLE = '1') reduces the effective speed by two
			-- Nevertheless it seems necessary to fix a problem with 
			-- the valid flag ang the read_enable flag.
			-- The Valid Flag goes up and reading (read_enable flag => 1) starts.
			-- There is a delay between setting the read_enable flag and reading, 
			-- and the valid flages only changes after reading... Thus data is
			-- declared valid which isn't valid... 
			-- Other solutions are surely possible but 
			-- the data stream is reduced to 4 MHz at the end anyway...
			if ((FIFO_IN_IsEmpty = '1') OR (SPI_FIFO_FULL = '1') OR (FIFO_IN_ENABLE = '1') ) AND (((AutomaticInputOngoing = '0') AND (AutomaticChannelRequired = '0')) OR (PrimaryInputOngoing = '1')) then
				-- Waiting for data... do nothing... 
				SPI_WriteStreamAway <= '0';
				SPI_IsFirstSegment <= SPI_IsFirstSegment;
				FIFO_IN_ENABLE <= '0';
				SPI_Data <= SPI_Data;
				SPI_TransmitType <= SPI_TransmitType;
				PrimaryInputOngoing <= PrimaryInputOngoing;
				AutomaticInputOngoing <= AutomaticInputOngoing;

			elsif (AutomaticInputOngoing = '0') AND (PrimaryInputOngoing = '0') AND (AutomaticChannelRequired = '1') then				
				-- Automatic Clock... Init
				SPI_IsFirstSegment <= '0';
				FIFO_IN_ENABLE <= '0';

				-- Connection to the Automatic Core...
				SPI_WriteStreamAway <= AutomaticCore_Data_Out_Valid;
				-- SPI_WriteStreamAway <= '1'; 
				SPI_Data <= AutomaticCore_Data_Out;
				-- SPI_Data <= (others => '1');

				SPI_TransmitType <= (others => '0');
				PrimaryInputOngoing <= '0';
				AutomaticInputOngoing <= '1';
				AutomaticClock_SegmentA <= '0';
				AutomaticClock_SegmentB <= '0';				
				
			elsif AutomaticInputOngoing = '1' then
				-- Automatic Clock... Hold or End
				SPI_IsFirstSegment <= '0';
				FIFO_IN_ENABLE <= '0';

				-- Connection to the Automatic Core
				SPI_WriteStreamAway <= AutomaticCore_Data_Out_Valid;
				-- SPI_WriteStreamAway <= '1'; 
				SPI_Data <= AutomaticCore_Data_Out;
				-- SPI_Data <= (others => '1');

				SPI_TransmitType <= (others => '0');
				PrimaryInputOngoing <= '0';
				
				-- Up -> Down => Off 
				if (AutomaticClock_ResetSignal(0) = '1') AND (AutomaticClock_ResetSignal(1) = '0') then
					AutomaticInputOngoing <= '0';
				else
					AutomaticInputOngoing <= AutomaticInputOngoing;
				end if;
				
				AutomaticClock_SegmentA <= '0';
				AutomaticClock_SegmentB <= '0';								
					
			elsif (FIFO_IN_IsValid = '1') AND (AutomaticInputOngoing = '0') then
				SPI_TransmitType_VAR := SPI_TransmitType;
				AutomaticInputOngoing <= '0';
				-- Page0 : FIFO_IN_DATAOUT(0) = '0'
				-- Page1 : FIFO_IN_DATAOUT(0) = '1'
				if FIFO_IN_DATAOUT(0) = '0' then
					-- First Bit Page
					SPI_TransmitType_VAR := FIFO_IN_DATAOUT(6 downto 1);
				else
					SPI_TransmitType_VAR := SPI_TransmitType_VAR;
				end if;
				
				case SPI_TransmitType_VAR is
					when "000000" | "000001"=>
						-- 16 & puzzle & go
						
						-- The data is from page 1 and there is a page0 already stored 
						if (FIFO_IN_DATAOUT(0) = '1') AND (SPI_IsFirstSegment = '1') then
							SPI_WriteStreamAway <= '1';
							SPI_IsFirstSegment <= '0';
							FIFO_IN_ENABLE <= '1';
							SPI_Data(14 downto 0) <= SPI_Data(14 downto 0);
							SPI_Data(29 downto 15) <= FIFO_IN_DATAOUT(15 downto 1);
							SPI_TransmitType_VAR := (others => '0');
							
							if SPI_TransmitType_VAR = "000001" then
								PrimaryInputOngoing <= '1';
							else
								PrimaryInputOngoing <= '0';
							end if;
							
						else
							SPI_WriteStreamAway <= '0';
							SPI_IsFirstSegment <= '1';
							FIFO_IN_ENABLE <= '1';
							SPI_Data(14 downto 0) <= FIFO_IN_DATAOUT(15 downto 1);
							SPI_Data(29 downto 15) <= (others => '0');
							SPI_TransmitType_VAR := SPI_TransmitType_VAR;	
							PrimaryInputOngoing <= '1';							
						end if;
						
					when "000010" | "000011" =>
						-- 8
						SPI_WriteStreamAway <= '1';
						SPI_IsFirstSegment <= '0';
						FIFO_IN_ENABLE <= '1';
						SPI_Data(14 downto 0) <= FIFO_IN_DATAOUT(15 downto 1);
						SPI_Data(29 downto 15) <= (others => '0');
						SPI_TransmitType_VAR := (others => '0');	
						
						if SPI_TransmitType_VAR = "000010" then
							PrimaryInputOngoing <= '1';
						else
							PrimaryInputOngoing <= '0';
						end if;
							
					when "001101" =>
						-- SPI Speed 
						ClockMode <= FIFO_IN_DATAOUT(15 downto 14);
						SPI_WriteStreamAway <= '0';
						SPI_IsFirstSegment <= '0';
						FIFO_IN_ENABLE <= '1';
						SPI_Data <= (others => '0');
						SPI_TransmitType_VAR := (others => '0');	
						PrimaryInputOngoing <= '0';						
					when "001011" =>
						-- Clock A
						AutomaticCore_Enable <= '0';
						SPI_WriteStreamAway <= '0';
						SPI_IsFirstSegment <= '0';
						SPI_Data <= (others => '0');
						FIFO_IN_ENABLE <= '1';
						AutomaticClock_CycleLength(46 downto 24) <= AutomaticClock_CycleLength(46 downto 24);
						-- 16 & puzzle & go
						-- The data is from page 1 and there is a page0 already stored 
						if (FIFO_IN_DATAOUT(0) = '1') AND (AutomaticClock_SegmentA = '1') then
							AutomaticClock_CycleLength(8 downto 0) <= AutomaticClock_CycleLength(8 downto 0);
							AutomaticClock_CycleLength(23 downto 9) <= FIFO_IN_DATAOUT(15 downto 1);
							AutomaticClock_SegmentA <='0';
							AutomaticClock_SegmentB <='0';
							SPI_TransmitType_VAR := (others => '0');		
							PrimaryInputOngoing <= '0';
						else
							AutomaticClock_CycleLength(8 downto 0) <= FIFO_IN_DATAOUT(15 downto 7);
							AutomaticClock_CycleLength(23 downto 9) <= (others => '0');
							AutomaticClock_SegmentA <='1';
							AutomaticClock_SegmentB <='0';
							SPI_TransmitType_VAR := SPI_TransmitType_VAR;			
							PrimaryInputOngoing <= '1';
						end if;
					when "001100" =>
						-- Clock B
						SPI_WriteStreamAway <= '0';
						SPI_IsFirstSegment <= '0';
						SPI_Data <= (others => '0');
						FIFO_IN_ENABLE <= '1';
						AutomaticClock_CycleLength(23 downto 0) <= AutomaticClock_CycleLength(23 downto 0);
						-- 16 & puzzle & go
						-- The data is from page 1 and there is a page0 already stored 
						if (FIFO_IN_DATAOUT(0) = '1') AND (AutomaticClock_SegmentB = '1') then
							AutomaticClock_CycleLength(32 downto 24) <= AutomaticClock_CycleLength(32 downto 24);
							AutomaticClock_CycleLength(46 downto 33) <= FIFO_IN_DATAOUT(14 downto 1);
							AutomaticClock_SegmentA <='0';
							AutomaticClock_SegmentB <='0';
							SPI_TransmitType_VAR := (others => '0');	
							PrimaryInputOngoing <= '0';							
							AutomaticCore_Enable <= FIFO_IN_DATAOUT(15);									
						else
							AutomaticClock_CycleLength(32 downto 24) <= FIFO_IN_DATAOUT(15 downto 7);
							AutomaticClock_CycleLength(46 downto 33) <= (others => '0');
							AutomaticClock_SegmentA <='0';
							AutomaticClock_SegmentB <='1';
							SPI_TransmitType_VAR := SPI_TransmitType_VAR;
							PrimaryInputOngoing <= '1';	
							AutomaticCore_Enable <= '0';									
						end if;
					when others =>
						-- Protect us from the unknown by resets... 
						SPI_WriteStreamAway <= '0';
						SPI_IsFirstSegment <= '0';
						FIFO_IN_ENABLE <= '1';
						SPI_Data <= (others => '0');
						SPI_TransmitType_VAR := (others => '0');
						PrimaryInputOngoing <= '0';				
					end case;
					
					SPI_TransmitType <= SPI_TransmitType_VAR;
			else
				-- Something is broken. Get rid of it... 
				SPI_WriteStreamAway <= '0';
				SPI_IsFirstSegment <= '0';
				FIFO_IN_ENABLE <= '1';
				SPI_Data <= (others => '0');
				SPI_TransmitType <= (others => '0');
				PrimaryInputOngoing <= '0';
				AutomaticInputOngoing <= '0';
			end if;
			
		end if;
	end process SortIt;


--	-- 16 bit Input Buffer
--		MyEchoFIFO_IN : EchoFIFO
--		port map (
--			clk => clk,
--			rst => Reset,
--			din => TheInput_Data,
--			wr_en => TheInput_Valid,
--			rd_en => FIFO_IN_ENABLE,
--			dout => FIFO_IN_DATAOUT,
--			almost_full => TheInput_IsAlmostFull,
--			full => FIFO_IN_IsFull,
--			empty => FIFO_IN_IsEmpty,
--			valid => FIFO_IN_IsValid,
--			overflow => FIFO_IN_overflow,
--			prog_full => TheInput_ProgFull
--			);


--	-- 30bit x 2k Input Fifo (OneBlockFree = '1' < 1000 free )
--		SPI_IN_FIFO : FIFO_30Bit_1000BlockCheck
--		port map (
--			rst => Reset,
--			wr_clk => clk,
--			rd_clk => ClockSelected,
--			din => SPI_Data,
--			wr_en => SPI_WriteStreamAway,
--			rd_en => SPI_FIFO_READENABLE,
--			dout => SPI_FIFO_DATAOUT,
--			full => SPI_FIFO_FULL,
--			empty => SPI_FIFO_EMPTY,
--			valid => SPI_FIFO_DATAOUTVALID,
--			prog_full => SPI_FIFO_1000BLOCKTOFULLL
--		);

--		-- 16Bit x 4 k FIFO (OneBlockFree = '1' < 1000 free )
--		FIFO_OUT : FIFO_16Bit_1000BlockCheck
--		port map (
--			rst => Reset,
--			wr_clk => ClockSelected,
--			rd_clk => clk,
--			din => FIFO_OUT_DATAIN,
--			wr_en => FIFO_OUT_DATAIN_Valid,
--			rd_en => TheOutput_Done,
--			dout => TheOutput_Data,
--			full => FIFO_OUT_IsFull,
--			empty => FIFO_OUT_IsEmpty,
--			valid => FIFO_OUT_IsValid,
--			prog_full => FIFO_OUT_1000BLOCKTOFULLL
--		);
		
	-- FIFO for the AutoPoller Core; crossing clock domains
	MyAutoPoller_FIFO_16Bit : AutoPoller_FIFO_16Bit 
	PORT MAP (
		rst => Reset,
		wr_clk => ClockSelected,
		rd_clk => clk,
		din => FIFO_OUT_DATAIN,
		wr_en => FIFO_OUT_DATAIN_Valid,
		rd_en => MyAutoPoller_FIFO_rd_en,
		dout => MyAutoPoller_FIFO_dout,
		full => MyAutoPoller_FIFO_full,
		empty => MyAutoPoller_FIFO_empty,
		valid => MyAutoPoller_FIFO_valid
	);


--	-- The SPI Interface core... 
--	MySimpleSPICore : entity work.SimpleSPICore 
--    Port MAP( 
--		-- ZARLINK SPI - PORTS
--		SPI_CS_B => SPI_ZL_CS,
--		SPI_SDI => SPI_SDI,
--		SPI_SDO => SPI_SDO,
--		SPI_CLK => SPI_CLK,
--
--		-- Input
--		INPUT_FIFO_DATAIN	=> SPI_FIFO_DATAOUT,
--		INPUT_FIFO_DATAIN_VALID	=> SPI_FIFO_DATAOUTVALID,
--		INPUT_FIFO_READENABLE => SPI_FIFO_READENABLE,
--		INPUT_FIFO_EMPTY => SPI_FIFO_EMPTY,
--
--		-- Output 
--		OUTPUT_FIFO_WRITEENABLE	=> FIFO_OUT_DATAIN_Valid,
--		OUTPUT_FIFO_DATAOUT		=> FIFO_OUT_DATAIN,
--		OUTPUT_FIFO_1000BLOCKTOFULLL => FIFO_OUT_1000BLOCKTOFULLL,		
--		
--		Reset	=> Reset,
--		CoreClock => ClockSelected
--	 );

	MyAutomaticCore : entity work.AutomaticCore
	Port MAP(
		-- Did the clock ring? Then we need to do somehing on 0->1
		AutomaticClock_Event 		=> AutomaticClock_Event,
		
		-- Have we an ouput channel? 
		AutomaticInputOngoing 		=> AutomaticInputOngoing,
		
		-- Request an output channel: 0->1
		AutomaticChannelRequired 	=> AutomaticChannelRequired,
		
		AutomaticClock_ResetSignal => AutomaticClock_ResetSignal,
		
		-- SPI Commands out...
		Data_Out 						=> AutomaticCore_Data_Out,
		Data_Out_Valid					=> AutomaticCore_Data_Out_Valid,

		-- Infos about the results... 
		Data_In							=> MyAutoPoller_FIFO_dout,
		Data_In_Valid					=> MyAutoPoller_FIFO_valid,
		Data_IN_Empty					=> MyAutoPoller_FIFO_empty,
		Data_IN_ReadEnable			=> MyAutoPoller_FIFO_rd_en,
		
		-- SPI FIFO Check
		SPI_FIFO_FULL 					=> SPI_FIFO_FULL,
		
		-- Reset the core 
		Reset 							=> Reset,
		-- Clock to the core (125 MHz)
		clk 								=> clk
	);
	
end architecture Behavioral;

