library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;

entity EmergencyInitFSM is
    Port ( 
		-- Start the whole ride 
		Start_EmergencyConnect 			: in STD_LOGIC;
		Start_LinkReadyCheck				: in STD_LOGIC;
		Start_PageModeCheck				: in STD_LOGIC;
		Start_SetPageMode_0				: in STD_LOGIC;
		Start_GetTXBufferUsed			: in STD_LOGIC;
		Start_GetRXBufferUsed			: in STD_LOGIC;
		Start_GetTXBufferSize			: in STD_LOGIC;
		Start_GetRXBufferSize			: in STD_LOGIC;
		Start_GetTXBufferMaxLength		: in STD_LOGIC;
		
		-- Input Parameter 
		RX_MODE 								: in STD_LOGIC_VECTOR(1 downto 0); -- 11 for 4FSK
		TX_MODE 								: in STD_LOGIC_VECTOR(1 downto 0); -- 11 for 4FSK
		SPI_SPEED							: in STD_LOGIC_VECTOR(1 downto 0); -- 11 for 4MHz
		CHANNEL_NUMBER 					: in STD_LOGIC_VECTOR(7 downto 0); -- Default 5
		RX_BUFFERSIZE 						: in STD_LOGIC_VECTOR(7 downto 0); -- Buffersize (14) + 16 (Error Correction) !!!Don't use 15!!!
		TX_BUFFERSIZE 						: in STD_LOGIC_VECTOR(7 downto 0); -- Buffersize (14) !!!Don't use 15!!!
		TX_BUFFER_MAXLENGTH 				: in STD_LOGIC_VECTOR(7 downto 0); -- Default: 31
		IMD_ID0 								: in STD_LOGIC_VECTOR(7 downto 0); -- Personal default: 0xaa
		IMD_ID1 								: in STD_LOGIC_VECTOR(7 downto 0); -- Personal default: 0xaa
		IMD_ID2 								: in STD_LOGIC_VECTOR(7 downto 0); -- Personal default: 0xaa

		DISABLE_THE_WATCHDOG				: IN STD_LOGIC; -- Do it => 1 ; Don't do it => 0
		CLEAN_IRQ_AUXSTATUS				: IN STD_LOGIC; -- Do it => 1 ; Don't do it => 0

		-- Commands generated by this core 
		Command_Out_Data			: out STD_LOGIC_VECTOR(15 downto 0);
		Command_Out_Valid			: out STD_LOGIC;

		-- Data received by this core 
		Command_In_Data			: in STD_LOGIC_VECTOR(7 downto 0);
		Command_In_Valid			: in STD_LOGIC;
		
		-- Information about this FSM (Is in idle mode?)
		IsInIDLEMode				: out STD_LOGIC;

		-- Information about the RadioLink (Is it ready?)
		IsLinkReady					: out STD_LOGIC;
		
		-- Information about the PageMode (Is it page 0?) 
		IsPageMode0					: out STD_LOGIC;
		
		-- Information about the state of the buffer 
		-- Start_GetTXBufferUsed
		ACTUAL_TXBufferUsed		: out STD_LOGIC_VECTOR(7 downto 0);
		-- Start_GetRXBufferUsed
		ACTUAL_RXBufferUsed		: out STD_LOGIC_VECTOR(7 downto 0);
		-- Start_GetTXBufferSize
		ACTUAL_TXBufferSize		: out STD_LOGIC_VECTOR(7 downto 0);
		-- Start_GetRXBufferSize
		ACTUAL_RXBufferSize		: out STD_LOGIC_VECTOR(7 downto 0);
		-- Start_GetTXBufferMaxLength
		ACTUAL_TXBufferMaxLength: out STD_LOGIC_VECTOR(7 downto 0);		
		
		-- The default stuff... 
		Reset 	: in STD_LOGIC;
		Clk 		: in  STD_LOGIC
	 
	 );
end EmergencyInitFSM;

architecture Behavioral of EmergencyInitFSM is

	-- The states of the SPI state machine
	type STATE_Type is (
		INIT, -- Reset all signals to a defined state
		IDLE, -- Wait until the Emergency Init is required 

		-- Start_EmergencyConnect
		WRITE_PAGEMODE_INIT,	-- Set PageMode0 and SPI Speed	
		RESET_ZARLINK, -- Reset the zarlink chip write dec82 to register dec95
		DISABLE_WATCHDOG, -- Disable the watchdog write 0x0F(=dec15) to register 26
		READ_PAGEMODE, -- Set Page to 0, read register 43
		WAIT_PAGEMODE, 
		WRITE_PAGEMODE, -- Set Page to 0
		READ_REG_IRQ_RAWSTATUS2, -- read register dec50
		WAIT_IRQ_RADIO_READY, 
		READ_MAC_MODUSER, -- read register dec23 => MAC_MODUSER(7 downto 0)
		WAIT_MAC_MODUSER,
		WRITE_MAC_MODUSER, -- write register dec23 MAC_MODUSER(7 downto 4) & RX_MODE & TX_MODE
		WRITE_REG_MACCHANNEL, -- write CHANNEL_NUMBER to register dec25
		WRITE_REG_TXBUFF_BSIZE, -- write TX_BUFFERSIZE to register dec4
		WRITE_REG_RXBUFF_BSIZE, -- write RX_BUFFERSIZE to register dec2
		WRITE_REG_TXBUFF_MAXPACKSIZE, -- write TX_BUFFER_MAXLENGTH to register dec5
		WRITE_IMD_ID0_A, -- write IMD_ID0 to register dec66
		WRITE_IMD_ID1_A, -- write IMD_ID1 to register dec67
		WRITE_IMD_ID2_A, -- write IMD_ID2 to register dec68
		WRITE_IMD_ID0_B, -- write IMD_ID0 to register dec19
		WRITE_IMD_ID1_B, -- write IMD_ID1 to register dec20
		WRITE_IMD_ID2_B, -- write IMD_ID2 to register dec21
		READ_REQ_IRQ_ENABLESET1, -- read REQ_IRQ_ENABLESET1(7 downto 0) from register dec48
		WAIT_REQ_IRQ_ENABLESET1,
		WRITE_REQ_IRQ_ENABLESET1, -- write REQ_IRQ_ENABLESET1(7 downto 5) & "1" & REQ_IRQ_ENABLESET1(3 downto 0) to register dec48
		WRITE_REQ_MAX_INITCOM, -- write 0 to dec42
		WRITE_REG_IRQ_AUXSTATUS, -- Clean IRQ AUXSTATUS (optional) 0 to register dec56

		-- Start_LinkReadyCheck
		READ_REG_IRQ_RAWSTATUS2_LINKREADY, -- Get RawStatus2 for LinkReady Check
		WAIT_IRQ_LINK_READY, -- Check if the Link is ready 

		-- Start_PageModeCheck
		READ_PAGEMODE_B, -- read register 43 (for IsPageMode0)
		WAIT_PAGEMODE_B,  -- for IsPageMode0	

		---Start_SetPageMode_0
		READ_PAGEMODE_C, -- Set Page to 0, read register 43
		WAIT_PAGEMODE_C, 
		WRITE_PAGEMODE_C, -- Set Page to 0
		
		-- Start_GetTXBufferUsed		
		READ_PAGEMODE_D, -- Set Page to 0, read register 43
		WAIT_PAGEMODE_D, 
		WRITE_PAGEMODE_D, -- Set Page to 0
		READ_REG_TXBUFF_USED, -- Read from register 3
		WAIT_REG_TXBUFF_USED,
		
		-- Start_GetRXBufferUsed		
		READ_PAGEMODE_E, -- Set Page to 0, read register 43
		WAIT_PAGEMODE_E, 
		WRITE_PAGEMODE_E, -- Set Page to 0
		READ_REG_RXBUFF_USED, -- Read from register 1
		WAIT_REG_RXBUFF_USED,

		-- Start_GetRXBufferSize		
		READ_PAGEMODE_F, -- Set Page to 0, read register 43
		WAIT_PAGEMODE_F, 
		WRITE_PAGEMODE_F, -- Set Page to 0
		READ_REG_RXBUFF_SIZE, -- Read from register 2
		WAIT_REG_RXBUFF_SIZE,

		-- Start_GetTXBufferSize		
		READ_PAGEMODE_G, -- Set Page to 0, read register 43
		WAIT_PAGEMODE_G, 
		WRITE_PAGEMODE_G, -- Set Page to 0
		READ_REG_TXBUFF_SIZE, -- Read from register 4
		WAIT_REG_TXBUFF_SIZE,

		-- Start_GetTXBufferMaxLength		
		READ_PAGEMODE_H, -- Set Page to 0, read register 43
		WAIT_PAGEMODE_H, 
		WRITE_PAGEMODE_H, -- Set Page to 0
		READ_REG_TXBUFF_MAXLENGTH, -- Read from register 5
		WAIT_REG_TXBUFF_MAXLENGTH
		
		);
	signal FSM_STATE : STATE_TYPE;

	signal MAC_MODUSER 	: std_logic_vector (7 downto 0) := (others => '0');	
	signal REQ_IRQ_ENABLESET1 : std_logic_vector (7 downto 0) := (others => '0');
	signal PAGEMODE : std_logic_vector (7 downto 0) := (others => '0');
	
	constant TimeOut_Threshold : unsigned (31 downto 0) := to_unsigned(1000,32);
	signal TimeOut_Counter : unsigned (31 downto 0) := (others => '0');
	signal IsLinkReady_Buffer : std_logic := '0';
	signal IsPageMode0_Buffer : std_logic := '0';
	signal ACTUAL_TXBufferUsed_Buffer : std_logic_vector (7 downto 0) := (others => '0');
	signal ACTUAL_RXBufferUsed_Buffer : std_logic_vector (7 downto 0) := (others => '0');
	signal ACTUAL_TXBufferSize_Buffer : STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
	signal ACTUAL_RXBufferSize_Buffer : STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
	signal ACTUAL_TXBufferMaxLength_Buffer : STD_LOGIC_VECTOR(7 downto 0) := (others => '0');		
	
begin
	IsLinkReady <= IsLinkReady_Buffer;
	IsPageMode0 <= IsPageMode0_Buffer;
	ACTUAL_TXBufferUsed <= ACTUAL_TXBufferUsed_Buffer;
	ACTUAL_RXBufferUsed <= ACTUAL_RXBufferUsed_Buffer;
	ACTUAL_TXBufferSize <= ACTUAL_TXBufferSize_Buffer;
	ACTUAL_RXBufferSize <= ACTUAL_RXBufferSize_Buffer;
	ACTUAL_TXBufferMaxLength <= ACTUAL_TXBufferMaxLength_Buffer;
	
	FSM_Process: process (Reset, Clk)
		variable ReadBit : std_logic := '0';
		variable Address : std_logic_vector(6 downto 0) := (others => '0');
		variable Data : std_logic_vector(7 downto 0) := (others => '0');
	begin
	
		if Reset = '1' then
			
			Command_Out_Data <= (others => '0');
			Command_Out_Valid <= '0';
			FSM_STATE <= INIT;
			MAC_MODUSER <= (others => '0');
			REQ_IRQ_ENABLESET1 <= (others => '0');
			PAGEMODE <= (others => '0');
			TimeOut_Counter <= (others => '0');
			IsInIDLEMode <= '0';
			IsLinkReady_Buffer <= '0';
			IsPageMode0_Buffer <= '0';
			ACTUAL_TXBufferUsed_Buffer <= (others => '0');
			ACTUAL_RXBufferUsed_Buffer <= (others => '0');
			ACTUAL_TXBufferSize_Buffer	<= (others => '0');
			ACTUAL_RXBufferSize_Buffer	<= (others => '0');
			ACTUAL_TXBufferMaxLength_Buffer <= (others => '0');		
			
		elsif rising_edge(Clk) then
		
			-- Default:
			Command_Out_Data <= (others => '0');
			Command_Out_Valid <= '0';
			FSM_STATE <= FSM_STATE;			
			MAC_MODUSER <= MAC_MODUSER;
			REQ_IRQ_ENABLESET1 <= REQ_IRQ_ENABLESET1;
			PAGEMODE <= PAGEMODE;
			TimeOut_Counter <= (others => '0');
			IsInIDLEMode <= '0';
			IsLinkReady_Buffer <= IsLinkReady_Buffer;
			IsPageMode0_Buffer <= IsPageMode0_Buffer;
			ACTUAL_TXBufferUsed_Buffer <= ACTUAL_TXBufferUsed_Buffer;
			ACTUAL_RXBufferUsed_Buffer <= ACTUAL_RXBufferUsed_Buffer;
			ACTUAL_TXBufferSize_Buffer <= ACTUAL_TXBufferSize_Buffer;
			ACTUAL_RXBufferSize_Buffer <= ACTUAL_RXBufferSize_Buffer;
			ACTUAL_TXBufferMaxLength_Buffer <= ACTUAL_TXBufferMaxLength_Buffer;
			
			case FSM_STATE is

				when INIT => -- Reset all signals to a defined state
					FSM_STATE <= IDLE;			
					MAC_MODUSER <= (others => '0');
					REQ_IRQ_ENABLESET1 <= (others => '0');
					PAGEMODE <= (others => '0');
			
				when IDLE => -- Wait until the Emergency Init is required 
					if Start_EmergencyConnect = '1' then
						FSM_STATE <= WRITE_PAGEMODE_INIT;
					elsif Start_LinkReadyCheck = '1' then 
						FSM_STATE <= READ_REG_IRQ_RAWSTATUS2_LINKREADY;
					elsif Start_PageModeCheck = '1' then
						FSM_STATE <= READ_PAGEMODE_B;
					elsif Start_SetPageMode_0 = '1' then
						FSM_STATE <= READ_PAGEMODE_C;
					elsif Start_GetTXBufferUsed = '1' then
						FSM_STATE <= READ_PAGEMODE_D;
					elsif Start_GetRXBufferUsed = '1' then
						FSM_STATE <= READ_PAGEMODE_E;
					elsif Start_GetRXBufferSize = '1' then
						FSM_STATE <= READ_PAGEMODE_F;
					elsif Start_GetTXBufferSize = '1' then
						FSM_STATE <= READ_PAGEMODE_G;
					elsif Start_GetTXBufferMaxLength = '1' then
						FSM_STATE <= READ_PAGEMODE_H;
					else
						FSM_STATE <= IDLE;
					end if;
					MAC_MODUSER <= (others => '0');
					REQ_IRQ_ENABLESET1 <= (others => '0');
					PAGEMODE <= (others => '0');
					IsInIDLEMode <= '1';

-- Start_EmergencyConnect

				when WRITE_PAGEMODE_INIT => -- Set Page to 0, write PAGEMODE(7 downto 1) & "0" to register 43 
					FSM_STATE <= RESET_ZARLINK;
					ReadBit := '0';
					Data := "0" & SPI_SPEED & "0000" & "0";
					Address := std_logic_vector(to_unsigned(43,7));
					Command_Out_Data <= Data(0) & Data(1) & Data(2) & Data(3) & Data(4) & Data(5) & Data(6) & Data(7) 	
											 & Address(0) & Address(1) & Address(2) & Address(3) & Address(4) & Address(5) & Address(6) & ReadBit;
					Command_Out_Valid <= DISABLE_THE_WATCHDOG;

				when RESET_ZARLINK => -- Reset the zarlink chip write dec82 to register dec95
					FSM_STATE <= DISABLE_WATCHDOG;
					ReadBit := '0';
					Data := std_logic_vector(to_unsigned(82,8));
					Address := std_logic_vector(to_unsigned(95,7));
					Command_Out_Data <= Data(0) & Data(1) & Data(2) & Data(3) & Data(4) & Data(5) & Data(6) & Data(7) 	
											 & Address(0) & Address(1) & Address(2) & Address(3) & Address(4) & Address(5) & Address(6) & ReadBit;
					Command_Out_Valid <= '1';
					
				when DISABLE_WATCHDOG => -- Disable the watchdog write 0x0F(=dec15) to register 26
					FSM_STATE <= READ_PAGEMODE;
					ReadBit := '0';
					Data := std_logic_vector(to_unsigned(15,8));
					Address := std_logic_vector(to_unsigned(26,7));
					Command_Out_Data <= Data(0) & Data(1) & Data(2) & Data(3) & Data(4) & Data(5) & Data(6) & Data(7) 	
											 & Address(0) & Address(1) & Address(2) & Address(3) & Address(4) & Address(5) & Address(6) & ReadBit;
					Command_Out_Valid <= DISABLE_THE_WATCHDOG;
				
				when READ_PAGEMODE => -- Set Page to 0, read register 43
					FSM_STATE <= WAIT_PAGEMODE;
					ReadBit := '1';
					Data := (others => '0');
					Address := std_logic_vector(to_unsigned(43,7));
					Command_Out_Data <= Data(0) & Data(1) & Data(2) & Data(3) & Data(4) & Data(5) & Data(6) & Data(7) 	
											 & Address(0) & Address(1) & Address(2) & Address(3) & Address(4) & Address(5) & Address(6) & ReadBit;
					Command_Out_Valid <= DISABLE_THE_WATCHDOG;
					
				when WAIT_PAGEMODE => 
					if Command_In_Valid = '1' then
						FSM_STATE <= WRITE_PAGEMODE;
						PAGEMODE <= Command_In_Data;
					elsif TimeOut_Counter >= TimeOut_Threshold then
						FSM_STATE <= INIT;
					else
						FSM_STATE <= WAIT_PAGEMODE;
					end if;
					TimeOut_Counter <= TimeOut_Counter + 1;
				
				when WRITE_PAGEMODE => -- Set Page to 0, write PAGEMODE(7 downto 1) & "0" to register 43 
					FSM_STATE <= READ_REG_IRQ_RAWSTATUS2;
					ReadBit := '0';
					Data := PAGEMODE(7) & SPI_SPEED & PAGEMODE(4 downto 1) & "0";
					Address := std_logic_vector(to_unsigned(43,7));
					Command_Out_Data <= Data(0) & Data(1) & Data(2) & Data(3) & Data(4) & Data(5) & Data(6) & Data(7) 	
											 & Address(0) & Address(1) & Address(2) & Address(3) & Address(4) & Address(5) & Address(6) & ReadBit;
					Command_Out_Valid <= DISABLE_THE_WATCHDOG;

				when READ_REG_IRQ_RAWSTATUS2 => -- read register dec50
					FSM_STATE <= WAIT_IRQ_RADIO_READY;
					ReadBit := '1';
					Data := (others => '0');
					Address := std_logic_vector(to_unsigned(50,7));
					Command_Out_Data <= Data(0) & Data(1) & Data(2) & Data(3) & Data(4) & Data(5) & Data(6) & Data(7) 	
											 & Address(0) & Address(1) & Address(2) & Address(3) & Address(4) & Address(5) & Address(6) & ReadBit;
					Command_Out_Valid <= DISABLE_THE_WATCHDOG;
					
				when WAIT_IRQ_RADIO_READY => 
					
					if (Command_In_Valid = '1') AND (Command_In_Data(3) = '1') then
						FSM_STATE <= READ_MAC_MODUSER;
					elsif (Command_In_Valid = '1') AND (Command_In_Data(3) = '0') then
						FSM_STATE <= READ_REG_IRQ_RAWSTATUS2;
					elsif TimeOut_Counter >= TimeOut_Threshold then
						FSM_STATE <= INIT;
					else
						FSM_STATE <= WAIT_IRQ_RADIO_READY;
					end if;
					TimeOut_Counter <= TimeOut_Counter + 1;
					
				when READ_MAC_MODUSER => -- read register dec23 => MAC_MODUSER(7 downto 0)
					FSM_STATE <= WAIT_MAC_MODUSER;		
					ReadBit := '1';
					Data := (others => '0');
					Address := std_logic_vector(to_unsigned(23,7));
					Command_Out_Data <= Data(0) & Data(1) & Data(2) & Data(3) & Data(4) & Data(5) & Data(6) & Data(7) 	
											 & Address(0) & Address(1) & Address(2) & Address(3) & Address(4) & Address(5) & Address(6) & ReadBit;
					Command_Out_Valid <= '1';	
					
				when WAIT_MAC_MODUSER =>
					if Command_In_Valid = '1' then
						FSM_STATE <= WRITE_MAC_MODUSER;
						PAGEMODE <= Command_In_Data;
					elsif TimeOut_Counter >= TimeOut_Threshold then
						FSM_STATE <= INIT;
					else
						FSM_STATE <= WAIT_MAC_MODUSER;
					end if;
					TimeOut_Counter <= TimeOut_Counter + 1;

				when WRITE_MAC_MODUSER => -- write register dec23 MAC_MODUSER(7 downto 4) & RX_MODE & TX_MODE
					FSM_STATE <= WRITE_REG_MACCHANNEL;
					ReadBit := '0';
					Data := MAC_MODUSER(7 downto 4) & RX_MODE & TX_MODE;
					Address := std_logic_vector(to_unsigned(23,7));
					Command_Out_Data <= Data(0) & Data(1) & Data(2) & Data(3) & Data(4) & Data(5) & Data(6) & Data(7) 	
											 & Address(0) & Address(1) & Address(2) & Address(3) & Address(4) & Address(5) & Address(6) & ReadBit;
					Command_Out_Valid <= '1';
					
				when WRITE_REG_MACCHANNEL => -- write CHANNEL_NUMBER to register dec25
					FSM_STATE <= WRITE_REG_TXBUFF_BSIZE;		
					ReadBit := '0';
					Data := CHANNEL_NUMBER;
					Address := std_logic_vector(to_unsigned(25,7));
					Command_Out_Data <= Data(0) & Data(1) & Data(2) & Data(3) & Data(4) & Data(5) & Data(6) & Data(7) 	
											 & Address(0) & Address(1) & Address(2) & Address(3) & Address(4) & Address(5) & Address(6) & ReadBit;
					Command_Out_Valid <= '1';
					
				when WRITE_REG_TXBUFF_BSIZE => -- write TX_BUFFERSIZE to register dec4
					FSM_STATE <= WRITE_REG_RXBUFF_BSIZE;
					ReadBit := '0';
					Data := TX_BUFFERSIZE;
					Address := std_logic_vector(to_unsigned(4,7));
					Command_Out_Data <= Data(0) & Data(1) & Data(2) & Data(3) & Data(4) & Data(5) & Data(6) & Data(7) 	
											 & Address(0) & Address(1) & Address(2) & Address(3) & Address(4) & Address(5) & Address(6) & ReadBit;
					Command_Out_Valid <= '1';					
					
				when WRITE_REG_RXBUFF_BSIZE => -- write RX_BUFFERSIZE to register dec2
					FSM_STATE <= WRITE_REG_TXBUFF_MAXPACKSIZE;
					ReadBit := '0';
					Data := RX_BUFFERSIZE;
					Address := std_logic_vector(to_unsigned(2,7));
					Command_Out_Data <= Data(0) & Data(1) & Data(2) & Data(3) & Data(4) & Data(5) & Data(6) & Data(7) 	
											 & Address(0) & Address(1) & Address(2) & Address(3) & Address(4) & Address(5) & Address(6) & ReadBit;
					Command_Out_Valid <= '1';						
					
				when WRITE_REG_TXBUFF_MAXPACKSIZE => -- write TX_BUFFER_MAXLENGTH to register dec5
					FSM_STATE <= WRITE_IMD_ID0_A;	
					ReadBit := '0';
					Data := TX_BUFFER_MAXLENGTH;
					Address := std_logic_vector(to_unsigned(5,7));
					Command_Out_Data <= Data(0) & Data(1) & Data(2) & Data(3) & Data(4) & Data(5) & Data(6) & Data(7) 	
											 & Address(0) & Address(1) & Address(2) & Address(3) & Address(4) & Address(5) & Address(6) & ReadBit;
					Command_Out_Valid <= '1';	
					
				when WRITE_IMD_ID0_A => -- write IMD_ID0 to register dec66
					FSM_STATE <= WRITE_IMD_ID1_A;
					ReadBit := '0';
					Data := IMD_ID0;
					Address := std_logic_vector(to_unsigned(66,7));
					Command_Out_Data <= Data(0) & Data(1) & Data(2) & Data(3) & Data(4) & Data(5) & Data(6) & Data(7) 	
											 & Address(0) & Address(1) & Address(2) & Address(3) & Address(4) & Address(5) & Address(6) & ReadBit;
					Command_Out_Valid <= '1';	
					
				when WRITE_IMD_ID1_A => -- write IMD_ID1 to register dec67
					FSM_STATE <= WRITE_IMD_ID2_A;
					ReadBit := '0';
					Data := IMD_ID1;
					Address := std_logic_vector(to_unsigned(67,7));
					Command_Out_Data <= Data(0) & Data(1) & Data(2) & Data(3) & Data(4) & Data(5) & Data(6) & Data(7) 	
											 & Address(0) & Address(1) & Address(2) & Address(3) & Address(4) & Address(5) & Address(6) & ReadBit;
					Command_Out_Valid <= '1';	
					
				when WRITE_IMD_ID2_A => -- write IMD_ID2 to register dec68
					FSM_STATE <= WRITE_IMD_ID0_B;
					ReadBit := '0';
					Data := IMD_ID2;
					Address := std_logic_vector(to_unsigned(68,7));
					Command_Out_Data <= Data(0) & Data(1) & Data(2) & Data(3) & Data(4) & Data(5) & Data(6) & Data(7) 	
											 & Address(0) & Address(1) & Address(2) & Address(3) & Address(4) & Address(5) & Address(6) & ReadBit;
					Command_Out_Valid <= '1';						
					
				when WRITE_IMD_ID0_B => -- write IMD_ID0 to register dec19
					FSM_STATE <= WRITE_IMD_ID1_B;
					ReadBit := '0';
					Data := IMD_ID0;
					Address := std_logic_vector(to_unsigned(19,7));
					Command_Out_Data <= Data(0) & Data(1) & Data(2) & Data(3) & Data(4) & Data(5) & Data(6) & Data(7) 	
											 & Address(0) & Address(1) & Address(2) & Address(3) & Address(4) & Address(5) & Address(6) & ReadBit;
					Command_Out_Valid <= '1';						
					
				when WRITE_IMD_ID1_B => -- write IMD_ID1 to register dec20
					FSM_STATE <= WRITE_IMD_ID2_B;
					ReadBit := '0';
					Data := IMD_ID1;
					Address := std_logic_vector(to_unsigned(20,7));
					Command_Out_Data <= Data(0) & Data(1) & Data(2) & Data(3) & Data(4) & Data(5) & Data(6) & Data(7) 	
											 & Address(0) & Address(1) & Address(2) & Address(3) & Address(4) & Address(5) & Address(6) & ReadBit;
					Command_Out_Valid <= '1';					
					
				when WRITE_IMD_ID2_B => -- write IMD_ID2 to register dec21
					FSM_STATE <= READ_REQ_IRQ_ENABLESET1;
					ReadBit := '0';
					Data := IMD_ID2;
					Address := std_logic_vector(to_unsigned(21,7));
					Command_Out_Data <= Data(0) & Data(1) & Data(2) & Data(3) & Data(4) & Data(5) & Data(6) & Data(7) 	
											 & Address(0) & Address(1) & Address(2) & Address(3) & Address(4) & Address(5) & Address(6) & ReadBit;
					Command_Out_Valid <= '1';					
					
				when READ_REQ_IRQ_ENABLESET1 => -- read REQ_IRQ_ENABLESET1(7 downto 0) from register dec48
					FSM_STATE <= WRITE_REQ_IRQ_ENABLESET1;
					ReadBit := '1';
					Data := (others => '0');
					Address := std_logic_vector(to_unsigned(48,7));
					Command_Out_Data <= Data(0) & Data(1) & Data(2) & Data(3) & Data(4) & Data(5) & Data(6) & Data(7) 	
											 & Address(0) & Address(1) & Address(2) & Address(3) & Address(4) & Address(5) & Address(6) & ReadBit;
					Command_Out_Valid <= '1';	

				when WAIT_REQ_IRQ_ENABLESET1 =>
					if Command_In_Valid = '1' then
						FSM_STATE <= WRITE_REQ_IRQ_ENABLESET1;
						REQ_IRQ_ENABLESET1 <= Command_In_Data;
					elsif TimeOut_Counter >= TimeOut_Threshold then
						FSM_STATE <= INIT;
					else
						FSM_STATE <= WAIT_REQ_IRQ_ENABLESET1;
					end if;
					TimeOut_Counter <= TimeOut_Counter + 1;
					
				when WRITE_REQ_IRQ_ENABLESET1 => -- write REQ_IRQ_ENABLESET1(7 downto 5) & "1" & REQ_IRQ_ENABLESET1(3 downto 0) to register dec48
					FSM_STATE <= WRITE_REQ_MAX_INITCOM;
					ReadBit := '0';
					Data := REQ_IRQ_ENABLESET1(7 downto 5) & "1" & REQ_IRQ_ENABLESET1(3 downto 0);
					Address := std_logic_vector(to_unsigned(48,7));
					Command_Out_Data <= Data(0) & Data(1) & Data(2) & Data(3) & Data(4) & Data(5) & Data(6) & Data(7) 	
											 & Address(0) & Address(1) & Address(2) & Address(3) & Address(4) & Address(5) & Address(6) & ReadBit;
					Command_Out_Valid <= '1';					
					
				when WRITE_REQ_MAX_INITCOM  => -- write 0 to dec42			
					FSM_STATE <= WRITE_REG_IRQ_AUXSTATUS;
					ReadBit := '0';
					Data := (others => '0');
					Address := std_logic_vector(to_unsigned(42,7));
					Command_Out_Data <= Data(0) & Data(1) & Data(2) & Data(3) & Data(4) & Data(5) & Data(6) & Data(7) 	
											 & Address(0) & Address(1) & Address(2) & Address(3) & Address(4) & Address(5) & Address(6) & ReadBit;
					Command_Out_Valid <= '1';		
					
				when WRITE_REG_IRQ_AUXSTATUS => -- Clean IRQ AUXSTATUS (optional) 0 to register dec56	
					FSM_STATE <= INIT;
					ReadBit := '0';
					Data := (others => '0');
					Address := std_logic_vector(to_unsigned(56,7));
					Command_Out_Data <= Data(0) & Data(1) & Data(2) & Data(3) & Data(4) & Data(5) & Data(6) & Data(7) 	
											 & Address(0) & Address(1) & Address(2) & Address(3) & Address(4) & Address(5) & Address(6) & ReadBit;
					Command_Out_Valid <= CLEAN_IRQ_AUXSTATUS;	

-- Start_LinkReadyCheck
				when READ_REG_IRQ_RAWSTATUS2_LINKREADY => -- read register dec50
					FSM_STATE <= WAIT_IRQ_RADIO_READY;
					ReadBit := '1';
					Data := (others => '0');
					Address := std_logic_vector(to_unsigned(50,7));
					Command_Out_Data <= Data(0) & Data(1) & Data(2) & Data(3) & Data(4) & Data(5) & Data(6) & Data(7) 	
											 & Address(0) & Address(1) & Address(2) & Address(3) & Address(4) & Address(5) & Address(6) & ReadBit;
					Command_Out_Valid <= DISABLE_THE_WATCHDOG;
					
				when WAIT_IRQ_LINK_READY => -- Check if the link is ready 
					
					if (Command_In_Valid = '1') then
						FSM_STATE <= INIT;
						IsLinkReady_Buffer <= Command_In_Data(2);
					elsif TimeOut_Counter >= TimeOut_Threshold then
						FSM_STATE <= INIT;
					else
						FSM_STATE <= WAIT_IRQ_LINK_READY;
					end if;
					TimeOut_Counter <= TimeOut_Counter + 1;

-- Start_PageModeCheck
				when READ_PAGEMODE_B => -- Set Page to 0, read register 43 (for IsPageMode0)
					FSM_STATE <= WAIT_PAGEMODE_B;
					ReadBit := '1';
					Data := (others => '0');
					Address := std_logic_vector(to_unsigned(43,7));
					Command_Out_Data <= Data(0) & Data(1) & Data(2) & Data(3) & Data(4) & Data(5) & Data(6) & Data(7) 	
											 & Address(0) & Address(1) & Address(2) & Address(3) & Address(4) & Address(5) & Address(6) & ReadBit;
					Command_Out_Valid <= DISABLE_THE_WATCHDOG;
					
				when WAIT_PAGEMODE_B =>  --(for IsPageMode0)
					if Command_In_Valid = '1' then
						FSM_STATE <= INIT;
						IsPageMode0_Buffer <= Command_In_Data(0);
					elsif TimeOut_Counter >= TimeOut_Threshold then
						FSM_STATE <= INIT;
					else
						FSM_STATE <= WAIT_PAGEMODE_B;
					end if;
					TimeOut_Counter <= TimeOut_Counter + 1;

---Start_SetPageMode_0
				when READ_PAGEMODE_C => -- Set Page to 0, read register 43
					FSM_STATE <= WAIT_PAGEMODE_C;
					ReadBit := '1';
					Data := (others => '0');
					Address := std_logic_vector(to_unsigned(43,7));
					Command_Out_Data <= Data(0) & Data(1) & Data(2) & Data(3) & Data(4) & Data(5) & Data(6) & Data(7) 	
											 & Address(0) & Address(1) & Address(2) & Address(3) & Address(4) & Address(5) & Address(6) & ReadBit;
					Command_Out_Valid <= DISABLE_THE_WATCHDOG;
					
				when WAIT_PAGEMODE_C => 
					if Command_In_Valid = '1' then
						FSM_STATE <= WRITE_PAGEMODE_C;
						PAGEMODE <= Command_In_Data;
					elsif TimeOut_Counter >= TimeOut_Threshold then
						FSM_STATE <= INIT;
					else
						FSM_STATE <= WAIT_PAGEMODE_C;
					end if;
					TimeOut_Counter <= TimeOut_Counter + 1;
				
				when WRITE_PAGEMODE_C => -- Set Page to 0, write PAGEMODE(7 downto 1) & "0" to register 43 
					FSM_STATE <= INIT;
					ReadBit := '0';
					Data := PAGEMODE(7 downto 1) & "0";
					Address := std_logic_vector(to_unsigned(43,7));
					Command_Out_Data <= Data(0) & Data(1) & Data(2) & Data(3) & Data(4) & Data(5) & Data(6) & Data(7) 	
											 & Address(0) & Address(1) & Address(2) & Address(3) & Address(4) & Address(5) & Address(6) & ReadBit;
					Command_Out_Valid <= DISABLE_THE_WATCHDOG;

-- Start_GetTXBufferUsed
				when READ_PAGEMODE_D => -- Set Page to 0, read register 43
					FSM_STATE <= WAIT_PAGEMODE_D;
					ReadBit := '1';
					Data := (others => '0');
					Address := std_logic_vector(to_unsigned(43,7));
					Command_Out_Data <= Data(0) & Data(1) & Data(2) & Data(3) & Data(4) & Data(5) & Data(6) & Data(7) 	
											 & Address(0) & Address(1) & Address(2) & Address(3) & Address(4) & Address(5) & Address(6) & ReadBit;
					Command_Out_Valid <= DISABLE_THE_WATCHDOG;
					
				when WAIT_PAGEMODE_D => 
					if Command_In_Valid = '1' then
						FSM_STATE <= WRITE_PAGEMODE_D;
						PAGEMODE <= Command_In_Data;
					elsif TimeOut_Counter >= TimeOut_Threshold then
						FSM_STATE <= INIT;
					else
						FSM_STATE <= WAIT_PAGEMODE_D;
					end if;
					TimeOut_Counter <= TimeOut_Counter + 1;
				
				when WRITE_PAGEMODE_D => -- Set Page to 0, write PAGEMODE(7 downto 1) & "0" to register 43 
					FSM_STATE <= READ_REG_TXBUFF_USED;
					ReadBit := '0';
					Data := PAGEMODE(7 downto 1) & "0";
					Address := std_logic_vector(to_unsigned(43,7));
					Command_Out_Data <= Data(0) & Data(1) & Data(2) & Data(3) & Data(4) & Data(5) & Data(6) & Data(7) 	
											 & Address(0) & Address(1) & Address(2) & Address(3) & Address(4) & Address(5) & Address(6) & ReadBit;
					Command_Out_Valid <= DISABLE_THE_WATCHDOG;

				when READ_REG_TXBUFF_USED => -- Read from register dec3
					FSM_STATE <= WAIT_REG_TXBUFF_USED;
					ReadBit := '1';
					Data := (others => '0');
					Address := std_logic_vector(to_unsigned(3,7));
					Command_Out_Data <= Data(0) & Data(1) & Data(2) & Data(3) & Data(4) & Data(5) & Data(6) & Data(7) 	
											 & Address(0) & Address(1) & Address(2) & Address(3) & Address(4) & Address(5) & Address(6) & ReadBit;
					Command_Out_Valid <= '1';	

				when WAIT_REG_TXBUFF_USED =>
					if Command_In_Valid = '1' then
						FSM_STATE <= INIT;
						ACTUAL_TXBufferUsed_Buffer <= Command_In_Data;
					elsif TimeOut_Counter >= TimeOut_Threshold then
						FSM_STATE <= INIT;
					else
						FSM_STATE <= WAIT_REG_TXBUFF_USED;
					end if;
					TimeOut_Counter <= TimeOut_Counter + 1;

-- Start_GetTXBufferUsed
				when READ_PAGEMODE_E => -- Set Page to 0, read register 43
					FSM_STATE <= WAIT_PAGEMODE_E;
					ReadBit := '1';
					Data := (others => '0');
					Address := std_logic_vector(to_unsigned(43,7));
					Command_Out_Data <= Data(0) & Data(1) & Data(2) & Data(3) & Data(4) & Data(5) & Data(6) & Data(7) 	
											 & Address(0) & Address(1) & Address(2) & Address(3) & Address(4) & Address(5) & Address(6) & ReadBit;
					Command_Out_Valid <= DISABLE_THE_WATCHDOG;
					
				when WAIT_PAGEMODE_E => 
					if Command_In_Valid = '1' then
						FSM_STATE <= WRITE_PAGEMODE_E;
						PAGEMODE <= Command_In_Data;
					elsif TimeOut_Counter >= TimeOut_Threshold then
						FSM_STATE <= INIT;
					else
						FSM_STATE <= WAIT_PAGEMODE_E;
					end if;
					TimeOut_Counter <= TimeOut_Counter + 1;
				
				when WRITE_PAGEMODE_E => -- Set Page to 0, write PAGEMODE(7 downto 1) & "0" to register 43 
					FSM_STATE <= READ_REG_RXBUFF_USED;
					ReadBit := '0';
					Data := PAGEMODE(7 downto 1) & "0";
					Address := std_logic_vector(to_unsigned(43,7));
					Command_Out_Data <= Data(0) & Data(1) & Data(2) & Data(3) & Data(4) & Data(5) & Data(6) & Data(7) 	
											 & Address(0) & Address(1) & Address(2) & Address(3) & Address(4) & Address(5) & Address(6) & ReadBit;
					Command_Out_Valid <= DISABLE_THE_WATCHDOG;

				when READ_REG_RXBUFF_USED => -- Read from register dec1
					FSM_STATE <= WAIT_REG_RXBUFF_USED;
					ReadBit := '1';
					Data := (others => '0');
					Address := std_logic_vector(to_unsigned(1,7));
					Command_Out_Data <= Data(0) & Data(1) & Data(2) & Data(3) & Data(4) & Data(5) & Data(6) & Data(7) 	
											 & Address(0) & Address(1) & Address(2) & Address(3) & Address(4) & Address(5) & Address(6) & ReadBit;
					Command_Out_Valid <= '1';	

				when WAIT_REG_RXBUFF_USED =>
					if Command_In_Valid = '1' then
						FSM_STATE <= INIT;
						ACTUAL_TXBufferUsed_Buffer <= Command_In_Data;
					elsif TimeOut_Counter >= TimeOut_Threshold then
						FSM_STATE <= INIT;
					else
						FSM_STATE <= WAIT_REG_RXBUFF_USED;
					end if;
					TimeOut_Counter <= TimeOut_Counter + 1;

-- Start_GetRXBufferSize

				when READ_PAGEMODE_F => -- Set Page to 0, read register 43
					FSM_STATE <= WAIT_PAGEMODE_F;
					ReadBit := '1';
					Data := (others => '0');
					Address := std_logic_vector(to_unsigned(43,7));
					Command_Out_Data <= Data(0) & Data(1) & Data(2) & Data(3) & Data(4) & Data(5) & Data(6) & Data(7) 	
											 & Address(0) & Address(1) & Address(2) & Address(3) & Address(4) & Address(5) & Address(6) & ReadBit;
					Command_Out_Valid <= DISABLE_THE_WATCHDOG;
					
				when WAIT_PAGEMODE_F => 
					if Command_In_Valid = '1' then
						FSM_STATE <= WRITE_PAGEMODE_F;
						PAGEMODE <= Command_In_Data;
					elsif TimeOut_Counter >= TimeOut_Threshold then
						FSM_STATE <= INIT;
					else
						FSM_STATE <= WAIT_PAGEMODE_F;
					end if;
					TimeOut_Counter <= TimeOut_Counter + 1;
				
				when WRITE_PAGEMODE_F => -- Set Page to 0, write PAGEMODE(7 downto 1) & "0" to register 43 
					FSM_STATE <= READ_REG_RXBUFF_SIZE;
					ReadBit := '0';
					Data := PAGEMODE(7 downto 1) & "0";
					Address := std_logic_vector(to_unsigned(43,7));
					Command_Out_Data <= Data(0) & Data(1) & Data(2) & Data(3) & Data(4) & Data(5) & Data(6) & Data(7) 	
											 & Address(0) & Address(1) & Address(2) & Address(3) & Address(4) & Address(5) & Address(6) & ReadBit;
					Command_Out_Valid <= DISABLE_THE_WATCHDOG;

				when READ_REG_RXBUFF_SIZE => -- Read from register dec2
					FSM_STATE <= WAIT_REG_RXBUFF_SIZE;
					ReadBit := '1';
					Data := (others => '0');
					Address := std_logic_vector(to_unsigned(2,7));
					Command_Out_Data <= Data(0) & Data(1) & Data(2) & Data(3) & Data(4) & Data(5) & Data(6) & Data(7) 	
											 & Address(0) & Address(1) & Address(2) & Address(3) & Address(4) & Address(5) & Address(6) & ReadBit;
					Command_Out_Valid <= '1';	

				when WAIT_REG_RXBUFF_SIZE =>
					if Command_In_Valid = '1' then
						FSM_STATE <= INIT;
						ACTUAL_RXBufferSize_Buffer <= Command_In_Data;
					elsif TimeOut_Counter >= TimeOut_Threshold then
						FSM_STATE <= INIT;
					else
						FSM_STATE <= WAIT_REG_RXBUFF_SIZE;
					end if;
					TimeOut_Counter <= TimeOut_Counter + 1;


-- Start_GetTXBufferSize

				when READ_PAGEMODE_G => -- Set Page to 0, read register 43
					FSM_STATE <= WAIT_PAGEMODE_G;
					ReadBit := '1';
					Data := (others => '0');
					Address := std_logic_vector(to_unsigned(43,7));
					Command_Out_Data <= Data(0) & Data(1) & Data(2) & Data(3) & Data(4) & Data(5) & Data(6) & Data(7) 	
											 & Address(0) & Address(1) & Address(2) & Address(3) & Address(4) & Address(5) & Address(6) & ReadBit;
					Command_Out_Valid <= DISABLE_THE_WATCHDOG;
					
				when WAIT_PAGEMODE_G => 
					if Command_In_Valid = '1' then
						FSM_STATE <= WRITE_PAGEMODE_G;
						PAGEMODE <= Command_In_Data;
					elsif TimeOut_Counter >= TimeOut_Threshold then
						FSM_STATE <= INIT;
					else
						FSM_STATE <= WAIT_PAGEMODE_G;
					end if;
					TimeOut_Counter <= TimeOut_Counter + 1;
				
				when WRITE_PAGEMODE_G => -- Set Page to 0, write PAGEMODE(7 downto 1) & "0" to register 43 
					FSM_STATE <= READ_REG_TXBUFF_SIZE;
					ReadBit := '0';
					Data := PAGEMODE(7 downto 1) & "0";
					Address := std_logic_vector(to_unsigned(43,7));
					Command_Out_Data <= Data(0) & Data(1) & Data(2) & Data(3) & Data(4) & Data(5) & Data(6) & Data(7) 	
											 & Address(0) & Address(1) & Address(2) & Address(3) & Address(4) & Address(5) & Address(6) & ReadBit;
					Command_Out_Valid <= DISABLE_THE_WATCHDOG;

				when READ_REG_TXBUFF_SIZE => -- Read from register dec4
					FSM_STATE <= WAIT_REG_TXBUFF_SIZE;
					ReadBit := '1';
					Data := (others => '0');
					Address := std_logic_vector(to_unsigned(4,7));
					Command_Out_Data <= Data(0) & Data(1) & Data(2) & Data(3) & Data(4) & Data(5) & Data(6) & Data(7) 	
											 & Address(0) & Address(1) & Address(2) & Address(3) & Address(4) & Address(5) & Address(6) & ReadBit;
					Command_Out_Valid <= '1';	

				when WAIT_REG_TXBUFF_SIZE =>
					if Command_In_Valid = '1' then
						FSM_STATE <= INIT;
						ACTUAL_TXBufferSize_Buffer <= Command_In_Data;
					elsif TimeOut_Counter >= TimeOut_Threshold then
						FSM_STATE <= INIT;
					else
						FSM_STATE <= WAIT_REG_TXBUFF_SIZE;
					end if;
					TimeOut_Counter <= TimeOut_Counter + 1;

-- Start_GetTXBufferMaxLength

				when READ_PAGEMODE_H => -- Set Page to 0, read register 43
					FSM_STATE <= WAIT_PAGEMODE_H;
					ReadBit := '1';
					Data := (others => '0');
					Address := std_logic_vector(to_unsigned(43,7));
					Command_Out_Data <= Data(0) & Data(1) & Data(2) & Data(3) & Data(4) & Data(5) & Data(6) & Data(7) 	
											 & Address(0) & Address(1) & Address(2) & Address(3) & Address(4) & Address(5) & Address(6) & ReadBit;
					Command_Out_Valid <= DISABLE_THE_WATCHDOG;
					
				when WAIT_PAGEMODE_H => 
					if Command_In_Valid = '1' then
						FSM_STATE <= WRITE_PAGEMODE_H;
						PAGEMODE <= Command_In_Data;
					elsif TimeOut_Counter >= TimeOut_Threshold then
						FSM_STATE <= INIT;
					else
						FSM_STATE <= WAIT_PAGEMODE_H;
					end if;
					TimeOut_Counter <= TimeOut_Counter + 1;
				
				when WRITE_PAGEMODE_H => -- Set Page to 0, write PAGEMODE(7 downto 1) & "0" to register 43 
					FSM_STATE <= READ_REG_TXBUFF_MAXLENGTH;
					ReadBit := '0';
					Data := PAGEMODE(7 downto 1) & "0";
					Address := std_logic_vector(to_unsigned(43,7));
					Command_Out_Data <= Data(0) & Data(1) & Data(2) & Data(3) & Data(4) & Data(5) & Data(6) & Data(7) 	
											 & Address(0) & Address(1) & Address(2) & Address(3) & Address(4) & Address(5) & Address(6) & ReadBit;
					Command_Out_Valid <= DISABLE_THE_WATCHDOG;

				when READ_REG_TXBUFF_MAXLENGTH => -- Read from register dec5
					FSM_STATE <= WAIT_REG_TXBUFF_MAXLENGTH;
					ReadBit := '1';
					Data := (others => '0');
					Address := std_logic_vector(to_unsigned(5,7));
					Command_Out_Data <= Data(0) & Data(1) & Data(2) & Data(3) & Data(4) & Data(5) & Data(6) & Data(7) 	
											 & Address(0) & Address(1) & Address(2) & Address(3) & Address(4) & Address(5) & Address(6) & ReadBit;
					Command_Out_Valid <= '1';	

				when WAIT_REG_TXBUFF_MAXLENGTH =>
					if Command_In_Valid = '1' then
						FSM_STATE <= INIT;
						ACTUAL_TXBufferMaxLength_Buffer <= Command_In_Data;
					elsif TimeOut_Counter >= TimeOut_Threshold then
						FSM_STATE <= INIT;
					else
						FSM_STATE <= WAIT_REG_TXBUFF_MAXLENGTH;
					end if;
					TimeOut_Counter <= TimeOut_Counter + 1;
					
				when others =>
					FSM_STATE <= INIT;
					
			end case;
		
		end if;
	
	end process FSM_Process;

end architecture Behavioral;

