library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;

entity ExternalADCArbiter is
    Port (
			
			-- ADC Ports 
			DATA_IN						: in std_logic_vector(1 downto 0);
			NOT_DATAREADY				: in std_logic_vector(1 downto 0);
			Clk_OUT						: out std_logic_vector(1 downto 0);
			ADC_SYNC						: out std_logic_vector(1 downto 0);
			
			-- Other Info
			ADC_ChannelMask 			: in std_logic_vector(31 downto 0);
			ChannelMask_Count			: in STD_LOGIC_VECTOR(15 downto 0);
			ChannelMask_Ready			: in STD_LOGIC;
		
			-- Output Data (Network Side)
			FIFO_PlusFullPacket		: out STD_LOGIC;
			FIFO_wr_en 					: out STD_LOGIC;
			FIFO_din 					: out STD_LOGIC_VECTOR(16 DOWNTO 0);
			FIFO_FULL					: in  STD_LOGIC;
			
			-- Information about error states 
			FIFO_Error_Overrun  		: in  STD_LOGIC;
			FIFO_Error_Underrun 		: in  STD_LOGIC;	 
			
			-- Other information for the package 
			Timestamp 					: in STD_LOGIC_VECTOR (15 downto 0);
			TriggerChannelsIn			: in STD_LOGIC_VECTOR (15 downto 0);
			
			Enable 						: in STD_LOGIC;
			
			-- The usual ports 
			Reset 						: in  STD_LOGIC;			
			Clk 							: in  STD_LOGIC -- Asuming a 96 MHz Clock
		);			
end ExternalADCArbiter;

architecture Behavioral of ExternalADCArbiter is

	signal Clk_Greater14MHz : std_logic := '0';
	signal Clock_Counter : unsigned (5 downto 0) := (others => '0');
	signal OUTPUT_FIFO_WRITEENABLE_A : std_logic := '0'; 
	signal OUTPUT_FIFO_DATAOUT_A : std_logic_vector (16 downto 0) := (others => '0');
	signal OUTPUT_FIFO_WRITEENABLE_B : std_logic := '0';
	signal OUTPUT_FIFO_DATAOUT_B : std_logic_vector (16 downto 0) := (others => '0');
	
	signal FIFO_rd_en_A : std_logic := '0';
	signal FIFO_empty_A : std_logic := '0';
	signal FIFO_almost_empty_A : std_logic := '0';
	signal FIFO_valid_A : std_logic := '0';
	signal FIFO_dout_A : std_logic_vector (16 downto 0) := (others => '0');
	
	signal FIFO_rd_en_B : std_logic := '0';
	signal FIFO_empty_B : std_logic := '0';
	signal FIFO_almost_empty_B : std_logic := '0';
	signal FIFO_valid_B : std_logic := '0';
	signal FIFO_dout_B : std_logic_vector (16 downto 0) := (others => '0');

	type STATE_TYPE is (
		-- Reset State 
		STATE_RESET,
		-- Idle State 
		STATE_IDLE,

		STATE_PREPARE_HEADER,
		STATE_PREPARE_LENGTH,
		STATE_PREPARE_RESPONSEID,
		STATE_PREPARE_ID_ERROR,
		STATE_PREPARE_TIMESTAMP,
		STATE_PREPARE_TRIGGER,
		STATE_PREPARE_DATA_CHECK_A,
		STATE_PREPARE_DATA_A,
		STATE_PREPARE_DATA_A_WAIT,
		STATE_PREPARE_DATA_CHECK_B,
		STATE_PREPARE_DATA_B,
		STATE_PREPARE_DATA_B_WAIT,
		STATE_COOLDOWN,
		STATE_VALID,
		STATE_END
	);
	signal FSM_STATE : STATE_TYPE;

	constant NET_HEADER : STD_LOGIC_VECTOR(15 downto 0) := std_logic_vector(to_unsigned(21930,16));
	signal NET_LENGTH_OK : unsigned(15 downto 0) := (others => '0');
	-- 0x8003
	constant NET_RESPONSEID_OK : STD_LOGIC_VECTOR(15 downto 0) := std_logic_vector(to_unsigned(32771,16));
	signal FIFO_Reset : std_logic := '0';
	signal ADC_ChannelMask_Copy : std_logic_vector(31 downto 0);
	signal ADC_ChannelMask_Copy_SH : std_logic_vector(31 downto 0);
	signal SecondHalf : std_logic := '0';
	
	signal ENABLE_INTERN : std_logic := '0';
	
	constant CounterCorrect : unsigned (7 downto 0) := to_unsigned(64,8);
	
	signal Counter : unsigned (7 downto 0) := (others => '0');

	COMPONENT ADCSampleFIFO
	PORT (
		rst : IN STD_LOGIC;
		wr_clk : IN STD_LOGIC;
		rd_clk : IN STD_LOGIC;
		din : IN STD_LOGIC_VECTOR(16 DOWNTO 0);
		wr_en : IN STD_LOGIC;
		rd_en : IN STD_LOGIC;
		dout : OUT STD_LOGIC_VECTOR(16 DOWNTO 0);
		full : OUT STD_LOGIC;
		empty : OUT STD_LOGIC;
		almost_empty : OUT STD_LOGIC;
		valid : OUT STD_LOGIC
	);
	END COMPONENT;
	
begin
	
	FIFO_Reset <= RESET OR (NOT ENABLE_INTERN);
	
	ADC_SYNC <= ENABLE_INTERN & ENABLE_INTERN;
	
	FSM : process (Reset, Clk)
	begin
		if Reset = '1' then
			FSM_STATE <= STATE_RESET;
			NET_LENGTH_OK <= (others => '0');

			FIFO_PlusFullPacket <= '0';
			FIFO_wr_en <= '0';
			FIFO_din <= (others => '0');
			
			ADC_ChannelMask_Copy <= (others => '0');
			ADC_ChannelMask_Copy_SH <= (others => '0');
			SecondHalf <= '0';
			
			Counter <= (others => '0');
			FIFO_rd_en_A <= '0';
			FIFO_rd_en_B <= '0';
		elsif rising_edge(Clk) then

			FIFO_PlusFullPacket <= '0';
			FIFO_wr_en <= '0';
			FIFO_din <= (others => '0');
			
			NET_LENGTH_OK <= NET_LENGTH_OK;
			
			ADC_ChannelMask_Copy <= ADC_ChannelMask_Copy;
			ADC_ChannelMask_Copy_SH <= ADC_ChannelMask_Copy_SH;
			SecondHalf <= SecondHalf;
			
			Counter <= Counter;

			FIFO_rd_en_A <= '0';
			FIFO_rd_en_B <= '0';

			
			case FSM_STATE is
				when STATE_RESET =>
					FSM_STATE <= STATE_IDLE;
					Counter <= (others => '0');
					
				when STATE_IDLE => 
					if (ENABLE_INTERN = '1') AND (FIFO_valid_A = '1') AND (ChannelMask_Ready = '1') AND (FIFO_FULL = '0') then
						FSM_STATE <= STATE_PREPARE_HEADER;
					else 
						-- No => Wait
						FSM_STATE <= STATE_IDLE;					
					end if;
					ADC_ChannelMask_Copy <= ADC_ChannelMask;
					ADC_ChannelMask_Copy_SH <= ADC_ChannelMask;
					SecondHalf <= '0';
					ENABLE_INTERN <= Enable;
					
				when STATE_PREPARE_HEADER => 
					FIFO_wr_en <= '1';
					FIFO_din <= "0" & NET_HEADER;				
					FSM_STATE <= STATE_PREPARE_LENGTH;
					NET_LENGTH_OK <= to_unsigned(20,16) + unsigned(ChannelMask_Count) + unsigned(ChannelMask_Count)
											+ unsigned(ChannelMask_Count) + unsigned(ChannelMask_Count);

				when STATE_PREPARE_LENGTH =>
					FIFO_wr_en <= '1';
					FIFO_din <= "0" & std_logic_vector(NET_LENGTH_OK);
					FSM_STATE <= STATE_PREPARE_RESPONSEID;	

				when STATE_PREPARE_RESPONSEID => 
					FIFO_wr_en <= '1';
					FIFO_din <= "0" & NET_RESPONSEID_OK;
					FSM_STATE <= STATE_PREPARE_TIMESTAMP;					

				when STATE_PREPARE_TIMESTAMP =>
					FIFO_wr_en <= '1';
					FIFO_din <= "0" & Timestamp;
					FSM_STATE <= STATE_PREPARE_TRIGGER;

				when STATE_PREPARE_TRIGGER =>
					FIFO_wr_en <= '1';
					FIFO_din <= "0" & TriggerChannelsIn;				
					FSM_STATE <= STATE_PREPARE_DATA_CHECK_A;
					Counter <= (others => '0');

				when STATE_PREPARE_DATA_CHECK_A =>

					if ADC_ChannelMask_Copy(15 downto 0) = "0000000000000000" then
						Counter <= Counter + to_unsigned(32,8);
						FSM_STATE <= STATE_PREPARE_DATA_CHECK_B;
						ADC_ChannelMask_Copy <= "0000000000000000" & ADC_ChannelMask_Copy(31 downto 16);
						ADC_ChannelMask_Copy_SH <= "0000000000000000" & ADC_ChannelMask_Copy_SH(31 downto 16);
					else
						FSM_STATE <= STATE_PREPARE_DATA_A;
					end if;
				
				when STATE_PREPARE_DATA_A =>
					
					if FIFO_valid_A = '1' then
						
						if FIFO_dout_A(16) = '0' then
							FSM_STATE <= STATE_PREPARE_DATA_A_WAIT;
						else 
							FSM_STATE <= STATE_PREPARE_DATA_CHECK_B;
						end if;
						
						if SecondHalf = '0' then
							FIFO_wr_en <= ADC_ChannelMask_Copy(0);
						else
							FIFO_wr_en <= ADC_ChannelMask_Copy_SH(0);
						end if;
						
						FIFO_rd_en_A <= '1';
						
						if SecondHalf = '0' then
							ADC_ChannelMask_Copy <= "0" & ADC_ChannelMask_Copy(31 downto 1);
						else
							ADC_ChannelMask_Copy_SH <= "0" & ADC_ChannelMask_Copy_SH(31 downto 1);
						end if;
						SecondHalf <= not SecondHalf;
						Counter <= Counter + 1;
					else
						FIFO_wr_en <= '0';
						FSM_STATE <= STATE_PREPARE_DATA_A;
					end if;
					FIFO_din <= "0" & FIFO_dout_A(15 downto 0);
					
					
				when STATE_PREPARE_DATA_A_WAIT => 
					FSM_STATE <= STATE_PREPARE_DATA_A;
					
				when STATE_PREPARE_DATA_CHECK_B =>

					if ADC_ChannelMask_Copy(15 downto 0) = "0000000000000000" then
						Counter <= Counter + to_unsigned(32,8);
						FSM_STATE <= STATE_COOLDOWN;
					else
						FSM_STATE <= STATE_PREPARE_DATA_B;
					end if;
					
				when STATE_PREPARE_DATA_B =>
					
					if FIFO_valid_B = '1' then
						
						if FIFO_dout_B(16) = '0' then
							FSM_STATE <= STATE_PREPARE_DATA_B_WAIT;
						else 
							FSM_STATE <= STATE_COOLDOWN;
						end if;
						
						if SecondHalf = '0' then
							FIFO_wr_en <= ADC_ChannelMask_Copy(0);
						else 
							FIFO_wr_en <= ADC_ChannelMask_Copy_SH(0);
						end if;
						
						FIFO_rd_en_B <= '1';
						
						if SecondHalf = '0' then
							ADC_ChannelMask_Copy <= "0" & ADC_ChannelMask_Copy(31 downto 1);
						else
							ADC_ChannelMask_Copy_SH <= "0" & ADC_ChannelMask_Copy_SH(31 downto 1);
						end if;
						SecondHalf <= not SecondHalf;
						Counter <= Counter + 1;
					else
						FIFO_wr_en <= '0';
						FSM_STATE <= STATE_PREPARE_DATA_B;
					end if;
					FIFO_din <= "0" & FIFO_dout_B(15 downto 0);
					
				when STATE_PREPARE_DATA_B_WAIT =>
					FSM_STATE <= STATE_PREPARE_DATA_B;
				
				when STATE_COOLDOWN => 
					FSM_STATE <= STATE_VALID;
				
				when STATE_VALID => 
					FSM_STATE <= STATE_END;
					FIFO_wr_en <= '1';
					if Counter = CounterCorrect then
						FIFO_din <= "1" & std_logic_vector(to_unsigned(1,16));
					else
						FIFO_din <= "1" & std_logic_vector(to_unsigned(0,16));
					end if;
					
				when STATE_END =>
					FSM_STATE <= STATE_IDLE;
					FIFO_PlusFullPacket <= '1';
					
				when others =>
					FSM_STATE <= STATE_RESET;
			end case;					
		
		end if;
	end process FSM;
	
	-- The ADCs (14 MHz transmitted, 24 MHz Inside)
	MyClock14MHz : process (Clk, Reset)
	begin 

		if Reset = '1' then
			Clk_Greater14MHz <= '0';
			Clock_Counter <= (others => '0');
			
		elsif rising_edge(Clk) then
			
			Clk_Greater14MHz <= Clk_Greater14MHz;
			
			if Clock_Counter < to_unsigned(2,6) then
				Clock_Counter <= Clock_Counter + 1;
			else
				Clock_Counter <= (others => '0');
				Clk_Greater14MHz <= not Clk_Greater14MHz;
			end if;
			
		end if;
	end process MyClock14MHz;
	
	ADC_A: entity work.ADC_24Bit_Datacollector 
	PORT MAP(
		DATA_IN => DATA_IN(0),
		NOT_DATAREADY => NOT_DATAREADY(0),
		OUTPUT_FIFO_WRITEENABLE => OUTPUT_FIFO_WRITEENABLE_A,
		OUTPUT_FIFO_DATAOUT => OUTPUT_FIFO_DATAOUT_A,
		Clk_OUT => Clk_OUT(0),
		Reset => Reset,
		Clk => Clk_Greater14MHz
	);

	ADC_B: entity work.ADC_24Bit_Datacollector 
	PORT MAP(
		DATA_IN => DATA_IN(1),
		NOT_DATAREADY => NOT_DATAREADY(1),
		OUTPUT_FIFO_WRITEENABLE => OUTPUT_FIFO_WRITEENABLE_B,
		OUTPUT_FIFO_DATAOUT => OUTPUT_FIFO_DATAOUT_B,
		Clk_OUT => Clk_OUT(1),
		Reset => Reset,
		Clk => Clk_Greater14MHz
	);

	ADC_FIFO_A : ADCSampleFIFO
		PORT MAP (
			rst => FIFO_Reset,
			wr_clk => Clk_Greater14MHz,
			rd_clk => Clk,
			din => OUTPUT_FIFO_DATAOUT_A,
			wr_en => OUTPUT_FIFO_WRITEENABLE_A,
			rd_en => FIFO_rd_en_A,
			dout => FIFO_dout_A,
			full => open,
			empty => FIFO_empty_A,
			almost_empty => FIFO_almost_empty_A,
			valid => FIFO_valid_A
	);

	ADC_FIFO_B : ADCSampleFIFO
		PORT MAP (
			rst => FIFO_Reset,
			wr_clk => Clk_Greater14MHz,
			rd_clk => Clk,
			din => OUTPUT_FIFO_DATAOUT_B,
			wr_en => OUTPUT_FIFO_WRITEENABLE_B,
			rd_en => FIFO_rd_en_B,
			dout => FIFO_dout_B,
			full => open,
			empty => FIFO_empty_B,
			almost_empty => FIFO_almost_empty_B,
			valid => FIFO_valid_B
	);

end Behavioral;

