library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;

entity InputPacketInterpreter is
	 Generic (
		-- Number of listening entities... 
		LengthOfWait : integer := 1
	 );
    Port ( 
		-- FIFO IN
		FIFO_IN_Data 				: IN STD_LOGIC_VECTOR(15 DOWNTO 0);
		FIFO_IN_Valid 				: IN STD_LOGIC;
		FIFO_IN_rd_en 				: OUT STD_LOGIC;	 

		FIFO_OUT_Data 				: OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
		FIFO_OUT_Valid 			: OUT STD_LOGIC;
		
		Packet_Start				: OUT STD_LOGIC;
		Packet_End					: OUT STD_LOGIC;
		Packet_Valid				: OUT STD_LOGIC;
		Packet_Unknown				: OUT STD_LOGIC;
		
		WaitForListener			: IN STD_LOGIC_VECTOR ((LengthOfWait-1) DOWNTO 0);
		
		Reset 						: in  STD_LOGIC;	
		Clk 							: in  STD_LOGIC
		
	 );
end InputPacketInterpreter;


architecture Behavioral of InputPacketInterpreter is

	type STATE_TYPE is (
		-- Reset State 
		STATE_RESET,
		-- Idle State 
		STATE_IDLE,
		-- First Word => First word was okay... 0x55AA
		STATE_FIRST_WORD,
		-- Second Word => What type of packet? 
		STATE_SECOND_WORD,
		STATE_MORE_THAN_THE_ID,
		STATE_CRC_A,
		STATE_CRC_B,
		STATE_CRC_C,
		STATE_CRC_D,
		STATE_CRC_CHECK,
		STATE_UNKNOW_PACKETTYP,
		STATE_COOLDOWN_A,
		STATE_COOLDOWN_B
	);
	signal FSM_STATE : STATE_TYPE;

	-- CRC 
	signal CRC_00 : std_logic_vector (15 downto 0) := (others => '0');
	signal CRC_01 : std_logic_vector (15 downto 0) := (others => '0');
	signal CRC_10 : std_logic_vector (15 downto 0) := (others => '0');
	signal CRC_11 : std_logic_vector (15 downto 0) := (others => '0');

	signal CRC_00_Input : std_logic_vector (15 downto 0) := (others => '0');
	signal CRC_01_Input : std_logic_vector (15 downto 0) := (others => '0');
	signal CRC_10_Input : std_logic_vector (15 downto 0) := (others => '0');
	signal CRC_11_Input : std_logic_vector (15 downto 0) := (others => '0');


	signal CRC_Data : STD_LOGIC_VECTOR(15 DOWNTO 0) := (others => '0');
	signal CRC_Valid : STD_LOGIC := '0';
	signal CRC_Reset : STD_LOGIC := '0';
	signal CRC_Invalid : std_logic := '0';
	
	-- 0x55AA == 21930
	constant Header : std_logic_vector (15 downto 0) := std_logic_vector(to_unsigned(21930,16));
	-- The Command Constants 
	constant Command_Version_AND_Status_Request : std_logic_vector (15 downto 0) := std_logic_vector(to_unsigned(257,16));
	constant Command_Connect_RF : std_logic_vector (15 downto 0) := std_logic_vector(to_unsigned(258,16));
	constant Command_Close_RF : std_logic_vector (15 downto 0) := std_logic_vector(to_unsigned(259,16));	
	constant Command_Set_Channel_Mask : std_logic_vector (15 downto 0) := std_logic_vector(to_unsigned(611,16));	
	constant Command_Set_ADC_Resolution : std_logic_vector (15 downto 0) := std_logic_vector(to_unsigned(626,16));	
	constant Command_Set_Sample_Rate : std_logic_vector (15 downto 0) := std_logic_vector(to_unsigned(627,16));	
	constant Command_Set_Filter_Settings : std_logic_vector (15 downto 0) := std_logic_vector(to_unsigned(614,16));	
	constant Command_Start_Data_Acquisition : std_logic_vector (15 downto 0) := std_logic_vector(to_unsigned(769,16));	
	constant Command_End_Data_Acquisition : std_logic_vector (15 downto 0) := std_logic_vector(to_unsigned(770,16));	
	constant Command_Read_Zarlink_Register_Page0 : std_logic_vector (15 downto 0) := std_logic_vector(to_unsigned(1024,16));	
	constant Command_Read_Zarlink_Register_Page1 : std_logic_vector (15 downto 0) := std_logic_vector(to_unsigned(1025,16));	
	constant Command_Write_Zarlink_Register_Page0 : std_logic_vector (15 downto 0) := std_logic_vector(to_unsigned(1026,16));	
	constant Command_Write_Zarlink_Register_Page1 : std_logic_vector (15 downto 0) := std_logic_vector(to_unsigned(1027,16));	
	constant Command_Set_Digital_Channels : std_logic_vector (15 downto 0) := std_logic_vector(to_unsigned(1280,16));	
	constant Command_Write_Zarlink_Block : std_logic_vector (15 downto 0) := std_logic_vector(to_unsigned(1028,16));	
	constant Command_Read_Zarlink_Block : std_logic_vector (15 downto 0) := std_logic_vector(to_unsigned(1029,16));
	constant Command_Get_UsedChannels : std_logic_vector (15 downto 0) := std_logic_vector(to_unsigned(1282,16));
	constant Command_Get_TriggerIn : std_logic_vector (15 downto 0) := std_logic_vector(to_unsigned(1283,16));
	constant Command_Constants : std_logic_vector (15 downto 0) := std_logic_vector(to_unsigned(255,16));
	
	constant Command_Set_Amp : std_logic_vector (15 downto 0) := std_logic_vector(to_unsigned(633,16));
	constant Command_Start_Recoding : std_logic_vector (15 downto 0) := std_logic_vector(to_unsigned(771,16));
	constant Command_Stop_Recording : std_logic_vector (15 downto 0) := std_logic_vector(to_unsigned(772,16));
	constant Command_Set_Additional_Channel_Mask : std_logic_vector (15 downto 0) := std_logic_vector(to_unsigned(634,16));	
	
	-- How long is the Packet? 
	signal ThePacketLength : std_logic_vector (15 downto 0) := (others => '0');
	-- Where are we in the Packet? 
	signal CountTheWords : unsigned (15 downto 0) := (others => '0');
	-- For getting the next word
	signal FIFO_IN_rd_en_Internal : std_logic := '0';
	
begin

	-- Request the next word... (export)
	FIFO_IN_rd_en <= FIFO_IN_rd_en_Internal;
	
	FSM : process(Reset,Clk)
	begin
		if (Reset = '1') then
			-- We want to bring the FSM into the reset state
			FSM_STATE <= STATE_RESET;
			
			-- Default 
			FIFO_OUT_Data <= (others => '0');
			FIFO_OUT_Valid <= '0';
			CRC_Data <= (others => '0');
			CRC_Valid <= '0';
			CRC_Reset <= '0';
			Packet_Start <= '0';
			Packet_End <= '0';
			Packet_Valid <= '0';
			Packet_Unknown <= '0';
			FIFO_IN_rd_en_Internal <= '0';
			CRC_00_Input <= (others => '0');
			CRC_01_Input <= (others => '0');
			CRC_10_Input <= (others => '0');
			CRC_11_Input <= (others => '0');
			CRC_Invalid <= '0';
			
			ThePacketLength <= (others => '0');
			CountTheWords <= (others => '0');
		elsif rising_edge(Clk) then

			-- Default 
			FIFO_OUT_Data <= (others => '0');
			FIFO_OUT_Valid <= '0';
			CRC_Data <= (others => '0');
			CRC_Valid <= '0';		
			CRC_Reset <= '0';

			CRC_00_Input <= (others => '0');
			CRC_01_Input <= (others => '0');
			CRC_10_Input <= (others => '0');
			CRC_11_Input <= (others => '0');
			
			
			Packet_Start <= '0';
			Packet_End <= '0';
			Packet_Valid <= '0';
			Packet_Unknown <= '0';

			
			CRC_Invalid <= '0';
			
			FIFO_IN_rd_en_Internal <= '0';
			
			ThePacketLength <= (others => '0');
			CountTheWords <= (others => '0');
			
			-- If nothing speaks against it, lets request the next word...
			if (FIFO_IN_valid = '1') AND (WaitForListener = std_logic_vector(to_unsigned(0,LengthOfWait))) AND (FSM_STATE /= STATE_CRC_CHECK) AND
				(FSM_STATE /= STATE_COOLDOWN_A ) AND (FSM_STATE /= STATE_COOLDOWN_B ) AND (FSM_STATE /= STATE_CRC_D) then
				FIFO_IN_rd_en_Internal <= '1';
			else
				NULL;
			end if;
			
			-- The finite state machine 
			case FSM_STATE is 
				when STATE_RESET =>
				-- We are in the reset state. Now lets go to the idle state 
					FSM_STATE <= STATE_IDLE;
					CRC_Reset <= '1';

					
				-- Wait... 
				when STATE_IDLE =>

				-- We will leave the idle state if we detect a 0x55AA = 21930
					if (FIFO_IN_Data = Header) AND (FIFO_IN_Valid = '1') AND (FIFO_IN_rd_en_Internal = '1') then
						FSM_STATE <= STATE_FIRST_WORD;
						CRC_Data <= FIFO_IN_Data;
						CRC_Valid <= '1';
					-- This was not a valid header... Lets ignore it. 	
					else
						FSM_STATE <= STATE_IDLE;
						CRC_Reset <= '1';
					end if;
				
				-- The Packet has begun... (Packet-Length)
				when STATE_FIRST_WORD => 
					if (FIFO_IN_Valid = '1') AND (FIFO_IN_rd_en_Internal = '1') then
					
						if unsigned(FIFO_IN_Data) = to_unsigned(0,16) then
							FSM_STATE <= STATE_IDLE;
						else
							FSM_STATE <= STATE_SECOND_WORD;
							-- Remember the packet length
							ThePacketLength <= FIFO_IN_Data;
						end if;
						CRC_Data <= FIFO_IN_Data;
						CRC_Valid <= '1';
					else
						FSM_STATE <= STATE_FIRST_WORD;
					end if;
				
				-- The Packet's second word...
				when STATE_SECOND_WORD => 
					-- Keep alive: 
					ThePacketLength <= ThePacketLength;
					-- When we leave we will have used 3 words (6 bytes)...
					CountTheWords <= to_unsigned(6,16);
					
					if (FIFO_IN_Valid = '1') AND (FIFO_IN_rd_en_Internal = '1') then
						
						-- Local Default:
						-- The packet starts
						Packet_Start <= '1';
						-- Copy the ID
						FIFO_OUT_Data <= FIFO_IN_Data;
						-- Mark the output as valid
						FIFO_OUT_Valid <= '1';
					
						-- Standard: ThePacketLength is for internal use without CRC 
						-- Except on an Unknown Condition 
						ThePacketLength <= std_logic_vector(resize(unsigned(signed("0"&ThePacketLength)-10),16));
						
						-- Is the packet type known? 
						case FIFO_IN_Data is
							-- Version AND Status Request
							when Command_Version_AND_Status_Request =>
								FSM_STATE <= STATE_CRC_A;
							-- Connect RF
							when Command_Connect_RF =>
								FSM_STATE <= STATE_MORE_THAN_THE_ID;	
							-- Close RF
							when Command_Close_RF =>
								FSM_STATE <= STATE_CRC_A;
							-- Set Channel Mask
							when Command_Set_Channel_Mask =>
								FSM_STATE <= STATE_MORE_THAN_THE_ID;	
							-- Set ADC Resolution
							when Command_Set_ADC_Resolution =>
								FSM_STATE <= STATE_MORE_THAN_THE_ID;	
							-- Set Sample Rate 
							when Command_Set_Sample_Rate =>
								FSM_STATE <= STATE_MORE_THAN_THE_ID;	
							-- Set Filter-Settings 
							when Command_Set_Filter_Settings =>
								FSM_STATE <= STATE_MORE_THAN_THE_ID;	
							-- Set Additional Channel Mask (ADC of the Basestation)
							when Command_Set_Additional_Channel_Mask =>
								FSM_STATE <= STATE_MORE_THAN_THE_ID;	
							-- Start the data-acquisition
							when Command_Start_Data_Acquisition =>
								FSM_STATE <= STATE_CRC_A;
							-- End the data-acquisition
							when Command_End_Data_Acquisition =>
								FSM_STATE <= STATE_CRC_A;
							-- Read Zarlink Register (Page 0)
							when Command_Read_Zarlink_Register_Page0 =>
								FSM_STATE <= STATE_MORE_THAN_THE_ID;	
							-- Read Zarlink Register (Page 1)
							when Command_Read_Zarlink_Register_Page1 =>
								FSM_STATE <= STATE_MORE_THAN_THE_ID;	
							-- Write Zarlink Register (Page 0)
							when Command_Write_Zarlink_Register_Page0 =>
								FSM_STATE <= STATE_MORE_THAN_THE_ID;	
							-- Write Zarlink Register (Page 1)
							when Command_Write_Zarlink_Register_Page1 =>
								FSM_STATE <= STATE_MORE_THAN_THE_ID;	
							-- Set Digital Channel at the Basestation
							when Command_Set_Digital_Channels =>
								FSM_STATE <= STATE_MORE_THAN_THE_ID;	
							-- Write One 14 Byte Block to the Zarlink
							when Command_Write_Zarlink_Block =>	
								FSM_STATE <= STATE_MORE_THAN_THE_ID;	
							-- Read One 14 Byte Block from the Zarlink
							when Command_Read_Zarlink_Block =>
								FSM_STATE <= STATE_CRC_A;
							-- Set the AMP
							when Command_Set_Amp => 
								FSM_STATE <= STATE_MORE_THAN_THE_ID;		
							-- Setting Debug Constants 								
							when Command_Constants => 
								FSM_STATE <= STATE_MORE_THAN_THE_ID;			
							-- Get the used Channels (ASIC)
							when Command_Get_UsedChannels => 
								FSM_STATE <= STATE_CRC_A;
							-- Starts the recording of the external ADCs
							when Command_Start_Recoding =>
								FSM_STATE <= STATE_CRC_A;
							-- Stops the recording of the external ADCs
							when Command_Stop_Recording =>
								FSM_STATE <= STATE_CRC_A;
							when Command_Get_TriggerIn => 
								FSM_STATE <= STATE_CRC_A;
								
							when others => 
								-- This is not a known packet-type!!!
								Packet_Unknown <= '1';
								FSM_STATE <= STATE_UNKNOW_PACKETTYP;
								FIFO_OUT_Valid <= '0';
								ThePacketLength <= std_logic_vector(resize(unsigned(signed("0"&ThePacketLength)-2),16));
						end case;
						
						CRC_Data <= FIFO_IN_Data;
						CRC_Valid <= '1';

					else
						FSM_STATE <= STATE_SECOND_WORD;
					end if;
					
				when STATE_CRC_A => 
					if (FIFO_IN_Valid = '1') AND (FIFO_IN_rd_en_Internal = '1') then
						FSM_STATE <= STATE_CRC_B;
						CRC_00_Input <= FIFO_IN_Data;
					else
						FSM_STATE <= STATE_CRC_A;
					end if;
	
				when STATE_CRC_B => 
					CRC_00_Input <= CRC_00_Input;
					
					if (FIFO_IN_Valid = '1') AND (FIFO_IN_rd_en_Internal = '1') then
						FSM_STATE <= STATE_CRC_C;
						CRC_01_Input <= FIFO_IN_Data;
					else
						FSM_STATE <= STATE_CRC_B;
					end if;
				
				when STATE_CRC_C => 
					CRC_00_Input <= CRC_00_Input;
					CRC_01_Input <= CRC_01_Input;

					if (FIFO_IN_Valid = '1') AND (FIFO_IN_rd_en_Internal = '1') then
						FSM_STATE <= STATE_CRC_D;
						CRC_10_Input <= FIFO_IN_Data;
					else
						FSM_STATE <= STATE_CRC_C;
					end if;

				when STATE_CRC_D => 
					CRC_00_Input <= CRC_00_Input;
					CRC_01_Input <= CRC_01_Input;
					CRC_10_Input <= CRC_10_Input;					

					-- And here does the packet end... 
					if (FIFO_IN_Valid = '1') AND (FIFO_IN_rd_en_Internal = '1') then
						FSM_STATE <= STATE_CRC_CHECK;
						CRC_11_Input <= FIFO_IN_Data;
					else
						FSM_STATE <= STATE_CRC_D;
					end if;
					
				-- Let's check the CRC values 
				when STATE_CRC_CHECK => 
					Packet_End <= '1';
					if (CRC_00_Input = CRC_00) AND (CRC_01_Input = CRC_01) AND 
						(CRC_10_Input = CRC_10) AND (CRC_11_Input = CRC_11) then
						Packet_Valid <= '1';
					else
						Packet_Valid <= '0';
					end if;
					FSM_STATE <= STATE_COOLDOWN_A;
				
				--	Cooldown (the later modules need time to block us, while processing)
				when STATE_COOLDOWN_A =>
					FSM_STATE <= STATE_COOLDOWN_B;
					
				when STATE_COOLDOWN_B =>
					FSM_STATE <= STATE_IDLE;
					
				when STATE_MORE_THAN_THE_ID =>
					-- Keep alive
					ThePacketLength <= ThePacketLength;
					CountTheWords <= CountTheWords;
						
					if (FIFO_IN_Valid = '1') AND (FIFO_IN_rd_en_Internal = '1') then
						-- Count and Leave 
						CountTheWords <= CountTheWords + 2;
						if CountTheWords = unsigned(ThePacketLength) then
							FSM_STATE <= STATE_CRC_A;
						else
							FSM_STATE <= STATE_MORE_THAN_THE_ID;
						end if;
						
						-- Copy the data to the output
						FIFO_OUT_Data <= FIFO_IN_Data;
						-- Mark the output as valid
						FIFO_OUT_Valid <= '1';
						
						CRC_Data <= FIFO_IN_Data;
						CRC_Valid <= '1';	
					else
						FSM_STATE <= STATE_MORE_THAN_THE_ID;
					end if;
					
				when STATE_UNKNOW_PACKETTYP =>
					-- Keep alive
					ThePacketLength <= ThePacketLength;
					CountTheWords <= CountTheWords;

					if (FIFO_IN_Valid = '1') AND (FIFO_IN_rd_en_Internal = '1') then
						-- Count and Leave 
						CountTheWords <= CountTheWords + 2;
						if CountTheWords = unsigned(ThePacketLength) then
							FSM_STATE <= STATE_IDLE;
							Packet_End <= '1';
						else
							FSM_STATE <= STATE_UNKNOW_PACKETTYP;
						end if;					
					else
						FSM_STATE <= STATE_UNKNOW_PACKETTYP;
					end if;

				when others => 
					FSM_STATE <= STATE_RESET;
			end case;
			
		end if;
	end process FSM;
	
	MyCRC : entity work.CRC_Checker 
	Port MAP( 
			 Network_Data => CRC_Data,
			 Network_Valid => CRC_Valid,
          CRC_Reset => CRC_Reset,
          CRC_00 => CRC_00,
          CRC_01 => CRC_01,
          CRC_10 => CRC_10,
          CRC_11 => CRC_11,
          Reset => Reset,
          Clk => Clk
	);	
	
end architecture Behavioral;


