library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;

-- Priority Arbiter... (0 has the higest rank)
entity OutputPacketArbiter is
	Generic (
		-- Number Of Parties that can write... 
		NumberOfUsers : integer := 2
	);
    Port ( 
		RequestChannel 			: in STD_LOGIC_VECTOR (NumberOfUsers-1 downto 0);
		GrantChannel 				: out STD_LOGIC_VECTOR (NumberOfUsers-1 downto 0);
		InUse_External				: out STD_LOGIC;
		Reset 						: in  STD_LOGIC;
		Clk 							: in  STD_LOGIC
	);
end OutputPacketArbiter;

architecture Behavioral of OutputPacketArbiter is

	signal GrantChannel_Internal : STD_LOGIC_VECTOR (NumberOfUsers-1 downto 0) := (others => '0');
	signal RequestChannel_Copy 	: STD_LOGIC_VECTOR (NumberOfUsers-1 downto 0) := (others => '0');
	signal PositionCounter : unsigned (15 downto 0) := (others => '0');
	
	type STATE_TYPE is (
		-- Reset State 
		STATE_RESET,
		-- Idle State 
		STATE_IDLE,
		-- There has someone data...
		STATE_SELECT,
		-- Found the relevant state
		STATE_FOUND
		
	);
	signal FSM_STATE : STATE_TYPE;		
	
	signal InUse : std_logic := '0';
	
begin
	
	-- Export the internal Grant Channel Information
	GrantChannel <= GrantChannel_Internal;
	
	
	FSM : process(Reset,Clk)
	begin
	
		if (Reset = '1') then

			-- Default: 
			GrantChannel_Internal <= (others => '0');
			RequestChannel_Copy <= (others => '0');
			PositionCounter <= (others => '0');
			
			FSM_STATE <= STATE_RESET;
			InUse <= '0';
			InUse_External <= '0';
		elsif rising_edge(Clk) then

			-- Default: 
			GrantChannel_Internal <= (others => '0');

			RequestChannel_Copy <= RequestChannel_Copy;
			PositionCounter <= PositionCounter;
			InUse <= '0';
			InUse_External <= '0';
			
			-- The finite state machine 
			case FSM_STATE is 
				when STATE_RESET =>
					FSM_STATE <= STATE_IDLE;
					RequestChannel_Copy <= (others => '0');
					PositionCounter <= (others => '0');
					
				when STATE_IDLE =>		
					PositionCounter <= (others => '0');
					-- Who wants some?
					if RequestChannel = std_logic_vector(to_unsigned(0,NumberOfUsers)) then
						-- Nobody there... 
						FSM_STATE <= STATE_IDLE;				
						RequestChannel_Copy <= (others => '0');
					else
						-- Time to serve 
						FSM_STATE <= STATE_SELECT;
						RequestChannel_Copy <= RequestChannel;
					end if;

				when STATE_SELECT =>
					-- Is there something left to do? 
					if PositionCounter < to_unsigned(NumberOfUsers,16) then
						-- Yes
						if RequestChannel_Copy(0) = '1' then
							FSM_STATE <= STATE_FOUND;
						else
							-- The Active wasn't found yet... 
							FSM_STATE <= STATE_SELECT;
						
							-- Prepare the next test...
							PositionCounter <= PositionCounter + 1;
							RequestChannel_Copy <= "0" & RequestChannel_Copy(NumberOfUsers-1 downto 1) ;
						end if;
					else
						-- No , we failed
						FSM_STATE <= STATE_IDLE;
					end if;
				
				when STATE_FOUND => 
					-- Does the Requeste still need the output stream?
					if RequestChannel(to_integer(PositionCounter)) = '1' then
						-- Yes
						GrantChannel_Internal(to_integer(PositionCounter)) <= '1';
						FSM_STATE <= STATE_FOUND;
						InUse <= '1';
						
						-- Delay plus something more (faster ending)
						if InUse = '1' then
							InUse_External <= '1';
						end if;
						
					else
						-- No => Back to start 
						FSM_STATE <= STATE_IDLE;
					end if;
					
					
				when others =>
					FSM_STATE <= STATE_RESET;
					RequestChannel_Copy <= (others => '0');
					
			end case;
		end if;
	end process FSM;
	
end Behavioral;

