library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;

entity PacketFrontend is
    Port ( 
		-- FIFO IN
		FIFO_IN_Data 				: IN STD_LOGIC_VECTOR(15 DOWNTO 0);
		FIFO_IN_Valid 				: IN STD_LOGIC;
		FIFO_IN_rd_en 				: OUT STD_LOGIC;			
		
		
		-- Connection to the Zarlink 
		SPI_ZL_CS 					: out STD_LOGIC;
		SPI_SDI 						: out STD_LOGIC;
		SPI_SDO 						: in 	STD_LOGIC;
		SPI_CLK 						: out STD_LOGIC;		

		-- Programm the AMP 
		VGA_CS 						: OUT std_logic_vector(31 downto 0);
		VGA_DATA 					: OUT std_logic_vector(1 downto 0);
		VGA_CLK 						: OUT std_logic_vector(1 downto 0);	
		
		-- Connection to the Network
		Network_Data				: out STD_LOGIC_VECTOR(15 downto 0); 
		Network_WriteEn			: out STD_LOGIC; 
		Network_Wait				: in  STD_LOGIC;

		-- Digital Trigger Channels 


		TriggerOut_Address 		: out STD_LOGIC_VECTOR(4 downto 0);
		TriggerOut_Data			: out STD_LOGIC;
		TriggerOut_Clk				: out STD_LOGIC;
		
		TriggerIn_Address 		: out STD_LOGIC_VECTOR(4 downto 0);
		TriggerIn_Data 			: in STD_LOGIC;
		

		-- ADC Ports 
		ADC_DATA_IN						: in std_logic_vector(1 downto 0);
		ADC_NOT_DATAREADY				: in std_logic_vector(1 downto 0);
		ADC_Clk_OUT						: out std_logic_vector(1 downto 0);
		ADC_SYNC 						: out std_logic_vector(1 downto 0);
		Clk1024KHz						: out std_logic;
		
		Clk125MHz						: in STD_LOGIC;
		
		-- The usual ports 
		Reset 						: in  STD_LOGIC;			
		Clk 							: in  STD_LOGIC
	);
end PacketFrontend;

architecture Behavioral of PacketFrontend is

		signal Packet_Start : STD_LOGIC := '0'; 
		signal Packet_End : STD_LOGIC := '0'; 
		signal Packet_Valid : STD_LOGIC := '0'; 
		signal Packet_Unknown : STD_LOGIC := '0'; 

		signal Zarlink_RequestChannel : STD_LOGIC_VECTOR(19 downto 0) := (others => '0');
		signal Zarlink_GetChannel : STD_LOGIC_VECTOR(19 downto 0) := (others => '0');
		signal In_Wait	: STD_LOGIC_VECTOR(25 downto 0) := (others => '0');
		signal Network_RequestChannel : STD_LOGIC_VECTOR(26 downto 0) := (others => '0'); 
		signal Network_GetChannel : STD_LOGIC_VECTOR(26 downto 0) := (others => '0');
		
		signal CRC_Reset : STD_LOGIC := '0';
		signal CRC_00	: STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
		signal CRC_01	: STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
		signal CRC_10	: STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
		signal CRC_11	: STD_LOGIC_VECTOR(15 downto 0) := (others => '0');

		-- Connection to the first Input-Filter-Layer 
		signal In_Data	: STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
		signal In_Valid : STD_LOGIC := '0';

		signal Network_Data_Internal : STD_LOGIC_VECTOR(15 downto 0) := (others => '0'); 
		signal Network_WriteEn_Internal	: STD_LOGIC := '0'; 

		signal Zarlink_Data_Out	: STD_LOGIC_VECTOR(119 downto 0) := (others => '0'); 
		signal Zarlink_WriteEn	: STD_LOGIC := '0'; 
		signal Zarlink_Data_In : STD_LOGIC_VECTOR(111 downto 0) := (others => '0');
		signal Zarlink_Name : STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
		signal Zarlink_Valid_In : STD_LOGIC := '0';
		signal Zarlink_BlockMode	: STD_LOGIC := '0'; 
		signal Zarlink_Wait : STD_LOGIC := '0';
		signal Zarlink_Error	: STD_LOGIC := '0';
		
		-- Controlling the BufferCheck 
		signal Zarlink_EnableBlockBufferCheck  : STD_LOGIC := '1';
		signal Zarlink_EnableBlockBufferCheck_Testgen  : STD_LOGIC := '1';
		signal Zarlink_EnableBlockBufferCheck_Collector  : STD_LOGIC := '1';

		-- For starting and stopping the Polling Process 
		signal Stop_DAQ : STD_LOGIC := '0';
		signal Start_DAQ : STD_LOGIC := '0';
		signal PollingIsEnabled : STD_LOGIC := '0';

		signal Stop_ExternalDAQ : STD_LOGIC := '0';
		signal Start_ExternalDAQ : STD_LOGIC := '0';		
		signal ExternalADCIsEnabled : STD_LOGIC := '0';		
		
		-- For Controlling the Clock for the polling process 
		signal RestartClock : STD_LOGIC := '0';
		signal ClockEvent : STD_LOGIC := '0';
		
		-- TEST PACKET GENERATOR
		signal TestClockEvent : STD_LOGIC := '0';
		signal TestPacketGen_Data_Out	: STD_LOGIC_VECTOR(143 downto 0) := (others => '0');
		signal TestPacketGen_Valid_Out : STD_LOGIC := '0';
		signal TestPacketGen_ActivatedClock : STD_LOGIC := '0';
		signal MyTestPacketEmitter_FIFO_RdEn : STD_LOGIC := '0';
		signal MyTestPacketEmitter_FIFO_VALID : STD_LOGIC := '0';
		
		-- Outgoing Data Processing Chain
		-- Zarlink
		signal NDPI_FIFO_PlusFullPacket : STD_LOGIC := '0';
		signal NDPI_FIFO_wr_en : STD_LOGIC := '0';
		signal NDPI_FIFO_din : STD_LOGIC_VECTOR(16 DOWNTO 0) := (others => '0');
		signal NDPI_FIFO_FULL : STD_LOGIC := '0';
		signal NDPI_FIFO_Error_Overrun : STD_LOGIC := '0';
		signal NDPI_FIFO_Error_Underrun : STD_LOGIC := '0';
		
		-- External ADC
		signal EADC_FIFO_PlusFullPacket : STD_LOGIC := '0';
		signal EADC_FIFO_wr_en : STD_LOGIC := '0';
		signal EADC_FIFO_din : STD_LOGIC_VECTOR(16 DOWNTO 0) := (others => '0');
		signal EADC_FIFO_FULL : STD_LOGIC := '0';
		signal EADC_FIFO_Error_Overrun : STD_LOGIC := '0';
		signal EADC_FIFO_Error_Underrun : STD_LOGIC := '0';
		
		signal DataPacketRefurbish_FIFO_VALID : STD_LOGIC := '0';
		signal DataPacketRefurbish_ReadEn_In : STD_LOGIC := '0';
		signal The10KHzCounter : STD_LOGIC_VECTOR(15 downto 0) := (others => '0');

		signal SecondaryChannelMask : STD_LOGIC_VECTOR (31 downto 0) := (others => '0');
		
		-- Some Constants:
		signal Zarlink_DebugMode	: 	STD_LOGIC := '0';
		signal Zarlink_Timeout : STD_LOGIC_VECTOR (27 downto 0) := (others => '0');
		signal Zarlink_ConnectionTimeout : STD_LOGIC_VECTOR (47 downto 0) := (others => '0');
		signal Zarlink_WaitForReset : STD_LOGIC_VECTOR(27 downto 0) := (others => '0');
		signal Implant_DeviceID : STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
		signal Zarlink_CheckBuffer : STD_LOGIC_VECTOR(31 downto 0) := (others => '0');
		signal CleanUpCounter_Maximal  : std_logic_vector (18 downto 0)  := (others => '0');
		signal TestPacketgenerator : STD_LOGIC := '0';
		signal TestPacketZarlink_Redirect : STD_LOGIC := '0';
		signal Zarlink_Implant_Mode 	: STD_LOGIC := '0';
		constant Firmware_Major_Version  : STD_LOGIC_VECTOR(7 downto 0) := std_logic_vector(to_unsigned(1,8));
		constant Firmware_Minor_Version  : STD_LOGIC_VECTOR(7 downto 0) := std_logic_vector(to_unsigned(0,8));
		signal Zarlink_Retry 				: STD_LOGIC_vector(18 downto 0) := (others => '0');
		signal Debug_DumpMode : STD_LOGIC := '0';
		signal Zarlink_Connection_Mode : STD_LOGIC_VECTOR(3 downto 0) := (others => '0');
		signal Zarlink_LinkQuality_Report : STD_LOGIC := '0';
		signal Zarlink_Testemitter_Retry : STD_LOGIC_VECTOR (31 downto 0) := (others => '0');
		signal Zarlink_Testgenerator_The10thOf1MS : std_logic_vector (29 downto 0) := (others => '0');
		signal Zarlink_MaxBufferFill : std_logic_vector (7 downto 0) := (others => '0');
		signal Zarlink_Length_Of_SPIClk : STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
		signal Zarlink_SPISpeedFlags 	: STD_LOGIC_VECTOR(1 downto 0) := (others => '0');
		
		-- Values of the implant-settings.
		signal Implant_ChannelMask : std_logic_vector(127 downto 0) := (others => '0');
		signal Implant_ChannelMask_Resorted : std_logic_vector(127 downto 0) := (others => '0');
		signal Implant_ADCResolution : std_logic_vector(3 downto 0) := (others => '0');
		signal Implant_SampleRate : std_logic_vector(7 downto 0) := (others => '0');
		signal Implant_Filter : std_logic_vector(3 downto 0) := (others => '0');
		
		signal ChannelMask_Count : STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
		signal ChannelMask_Ready : STD_LOGIC := '0';

		signal ExternalMask_Count : STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
		signal ExternalMask_Ready : STD_LOGIC := '0';
		
		-- This comes out of the polling 
		signal Collector_Data_Out : STD_LOGIC_VECTOR(143 downto 0) := (others => '0');
		signal Collector_Valid_Out : STD_LOGIC := '0';

		-- This comes out of the polling fifo
		signal CollectorFIFO_Data_Out : STD_LOGIC_VECTOR(143 downto 0) := (others => '0');
		signal CollectorFIFO_Valid_Out : STD_LOGIC := '0';
		signal CollectorFIFO_rd_en : STD_LOGIC := '0';

		signal DecFIFO_Data_In	: STD_LOGIC_VECTOR(143 downto 0) := (others => '0');
		signal DecFIFO_Valid_In : STD_LOGIC := '0';

		signal NetworkInUse : std_logic := '0';
		signal ZarlinkInUse : std_logic := '0';
		signal Zarlink_WriteEn_NoZ : std_logic := '0';

		signal Cooldown_Counter_Max : std_logic_vector(7 downto 0) := (others => '0');
		
		signal The32KHzTimeStamp : std_logic_vector(15 downto 0) := (others => '0');

		-- 1.024 MHz Clock for ADC
		signal Clk120_8333MHz : std_logic;
		signal Clk1024KHz_Buffer : std_logic := '0';
		signal Clk1024KHz_Counter : unsigned(5 downto 0) := (others => '0');
		constant Clk1024KHz_Interval : unsigned(5 downto 0) := to_unsigned(58,6);
		
	COMPONENT PacketDecodingFIFO
		PORT (
			clk : IN STD_LOGIC;
			rst : IN STD_LOGIC;
			din : IN STD_LOGIC_VECTOR(143 DOWNTO 0);
			wr_en : IN STD_LOGIC;
			rd_en : IN STD_LOGIC;
			dout : OUT STD_LOGIC_VECTOR(143 DOWNTO 0);
			full : OUT STD_LOGIC;
			empty : OUT STD_LOGIC;
			valid : OUT STD_LOGIC
		);
	END COMPONENT;

		signal TriggerChannelsOut : STD_LOGIC_VECTOR (31 downto 0) := (others => '0');
		signal TriggerChannelsIn : STD_LOGIC_VECTOR (31 downto 0) := (others => '0');
		
		signal HalfCycle_TriggerOut : STD_LOGIC_VECTOR (16 downto 0) := (others => '0');
		signal Wait_TriggerIn : STD_LOGIC_VECTOR (16 downto 0) := (others => '0');
begin
		Network_Data <= Network_Data_Internal;
		Network_WriteEn <= Network_WriteEn_Internal when NetworkInUse = '1' else '0';

		-- Not used yet...
		In_Wait(1 downto 0) <= (others => '0');

		Zarlink_WriteEn_NoZ <= Zarlink_WriteEn when ZarlinkInUse = '1' else '0';

MyZarlinkCommunicationModule : entity work.ZarlinkEchoModule 
    Port MAP ( 
		-- Connection to the Zarlink 
		Zarlink_Data_Out => Zarlink_Data_Out,
		Zarlink_WriteEn => Zarlink_WriteEn_NoZ,
		Zarlink_Data_In => Zarlink_Data_In,
		Zarlink_Valid_In => Zarlink_Valid_In,
		Zarlink_BlockMode => Zarlink_BlockMode,
		Zarlink_Wait => Zarlink_Wait,
		Zarlink_Name => Zarlink_Name,
		
		Zarlink_Timeout => Zarlink_Timeout,
		Zarlink_DebugMode => Zarlink_DebugMode,
		EnableBlockBufferCheck => Zarlink_EnableBlockBufferCheck,
		Zarlink_Error => Zarlink_Error,
		
		Cooldown_Counter_Max => Cooldown_Counter_Max,
		
		-- Connection to the Network
		Network_Data => Network_Data_Internal,
		Network_WriteEn => Network_WriteEn_Internal,
		Network_Wait => Network_Wait,
		Network_RequestChannel => Network_RequestChannel(19),
		Network_GetChannel => Network_GetChannel(19),

		SPI_ZL_CS => SPI_ZL_CS,
		SPI_SDI => SPI_SDI,
		SPI_SDO => SPI_SDO,
		SPI_CLK => SPI_CLK,

		CRC_Reset => CRC_Reset,
		CRC_00 => CRC_00,
		CRC_01 => CRC_01,
		CRC_10 => CRC_10,
		CRC_11 => CRC_11,
		Length_Of_SPIClk => Zarlink_Length_Of_SPIClk,
		Zarlink_Retry => Zarlink_Retry,
		
		The10KHzCounter => The10KHzCounter,
		The32KHzCounter => The32KHzTimeStamp,
		
		-- The usual ports 
		Reset => Reset,
		Clk => Clk
	 );

	MyZarlinkDataCollector : entity work.ZarlinkDataCollector
		Generic MAP
		(
			Module_Name => std_logic_vector(to_unsigned(1,16))
		)
    Port MAP ( 
		
			PollingIsEnabled => PollingIsEnabled,
			RestartClock => RestartClock,
			ClockEvent => ClockEvent,
			CleanUpCounter_Maximal => CleanUpCounter_Maximal,
		
		-- Connection to the Zarlink 
			Zarlink_Data_Out => Zarlink_Data_Out,
			Zarlink_WriteEn => Zarlink_WriteEn,
			Zarlink_Data_In => Zarlink_Data_In,
			Zarlink_Valid_In => Zarlink_Valid_In,
			Zarlink_BlockMode => Zarlink_BlockMode,
			Zarlink_Wait => Zarlink_Wait,
			Zarlink_RequestChannel => Zarlink_RequestChannel(0),
			Zarlink_GetChannel => Zarlink_GetChannel(0),
			Zarlink_Name => Zarlink_Name,		
			Zarlink_EnableBlockBufferCheck => Zarlink_EnableBlockBufferCheck_Collector,
			
			Data_Out	=> Collector_Data_Out,
			Valid_Out => Collector_Valid_Out,
		
			TriggerChannelsIn => TriggerChannelsIn,
			The10KHzCounter => The32KHzTimeStamp,
			Zarlink_LinkQuality_Report => Zarlink_LinkQuality_Report,
		
		-- The usual ports 
			Reset => Reset,
			Clk => Clk
	 );

MyA10KHzCounter : entity work.A10KHzCounter 
    Port MAP (
		TheCounter => The10KHzCounter, 
		
		-- The usual ports 
		Reset => Reset,
		Clk => Clk 
	 );

-- The Counter runs freely until the external ADC are activated.
-- Then it used the NO_DATAREADY Signal to be precisely in sync
MyA32KHzCounter : entity work.A32KHzCounter 
    Port MAP (
		TheCounter => The32KHzTimeStamp, 
		
		ExternalADCIsEnabled => ExternalADCIsEnabled,
		ADC_NOT_DATAREADY => ADC_NOT_DATAREADY(0),
		-- The usual ports 
		Reset => Reset,
		Clk => Clk 
	 );

-- Drive the fifo via the collector or the Test Packet Generator 
DecFIFO_Data_In <= Collector_Data_Out when TestPacketgenerator = '0' else TestPacketGen_Data_Out;
DecFIFO_Valid_In <= Collector_Valid_Out when TestPacketgenerator = '0' else TestPacketGen_Valid_Out;

-- Who is allowed to read the FIFO
CollectorFIFO_rd_en <= DataPacketRefurbish_ReadEn_In when TestPacketZarlink_Redirect = '0' else MyTestPacketEmitter_FIFO_RdEn;

MyPacketDecodingFIFO : PacketDecodingFIFO
  PORT MAP (
    clk => Clk,
    rst => Reset,
    din => DecFIFO_Data_In,
    wr_en => DecFIFO_Valid_In,
    rd_en => CollectorFIFO_rd_en,
    dout => CollectorFIFO_Data_Out,
    full => open,
    empty => open,
    valid => CollectorFIFO_Valid_Out
  );

--MyNetworkDataProcessInterface_ExternalADC : entity work.NetworkDataProcessInterface 
--		Port MAP
--		(
--			-- Input Data 
--			FIFO_PlusFullPacket => EADC_FIFO_PlusFullPacket,
--			FIFO_wr_en => EADC_FIFO_wr_en,
--			FIFO_din => EADC_FIFO_din,
--			FIFO_FULL => EADC_FIFO_FULL,
--			
--			-- Information about error states 
--			FIFO_Error_Overrun => EADC_FIFO_Error_Overrun,
--			FIFO_Error_Underrun => EADC_FIFO_Error_Underrun,
--			
--		-- Connection to the Network
--			Network_Data => Network_Data_Internal,
--			Network_WriteEn => Network_WriteEn_Internal,
--			Network_Wait => Network_Wait,
--			Network_RequestChannel => Network_RequestChannel(0),
--			Network_GetChannel => Network_GetChannel(0),
--			CRC_Reset => CRC_Reset,
--			CRC_00 => CRC_00,
--			CRC_01 => CRC_01,
--			CRC_10 => CRC_10,
--			CRC_11 => CRC_11,
--
--		-- The usual ports 
--			Reset => Reset,
--			Clk => Clk
--		);
Network_RequestChannel(0) <= '0';

MyNetworkDataProcessInterface_Zarlink : entity work.NetworkDataProcessInterface 
		Port MAP
		(
			-- Input Data 
			FIFO_PlusFullPacket => NDPI_FIFO_PlusFullPacket,
			FIFO_wr_en => NDPI_FIFO_wr_en,
			FIFO_din => NDPI_FIFO_din,
			FIFO_FULL => NDPI_FIFO_FULL,
			
			-- Information about error states 
			FIFO_Error_Overrun => NDPI_FIFO_Error_Overrun,
			FIFO_Error_Underrun => NDPI_FIFO_Error_Underrun,
			
		-- Connection to the Network
			Network_Data => Network_Data_Internal,
			Network_WriteEn => Network_WriteEn_Internal,
			Network_Wait => Network_Wait,
			Network_RequestChannel => Network_RequestChannel(1),
			Network_GetChannel => Network_GetChannel(1),
			CRC_Reset => CRC_Reset,
			CRC_00 => CRC_00,
			CRC_01 => CRC_01,
			CRC_10 => CRC_10,
			CRC_11 => CRC_11,

		-- The usual ports 
			Reset => Reset,
			Clk => Clk
		);

-- Transmit the valid only if the test emitter core is NOT active...
DataPacketRefurbish_FIFO_VALID <= (CollectorFIFO_Valid_Out AND (NOT TestPacketZarlink_Redirect));

 MyDataPacketRefurbish : entity work.DataPacketRefurbish
    Port MAP (
	 
			-- Connection to the FIFO (Zarlink Side)
			Data_In => CollectorFIFO_Data_Out,
			Valid_In	=> DataPacketRefurbish_FIFO_VALID,
			ReadEn_In => DataPacketRefurbish_ReadEn_In,
			
			Implant_ChannelMask => Implant_ChannelMask_Resorted,
			Implant_ADCResolution => Implant_ADCResolution,
			Implant_SampleRate => Implant_SampleRate,
			Implant_Filter => Implant_Filter,
			Implant_DeviceID => Implant_DeviceID,
			
			-- Output Data (Network Side)
			FIFO_PlusFullPacket => NDPI_FIFO_PlusFullPacket,
			FIFO_wr_en => NDPI_FIFO_wr_en,
			FIFO_din => NDPI_FIFO_din,
			FIFO_FULL => NDPI_FIFO_FULL,
			
			ChannelMask_Count => ChannelMask_Count,
			ChannelMask_Ready => ChannelMask_Ready,
			
			-- Information about error states 
			FIFO_Error_Overrun => NDPI_FIFO_Error_Overrun,
			FIFO_Error_Underrun => NDPI_FIFO_Error_Underrun,
			
			Debug_DumpMode => Debug_DumpMode,
			
			-- The usual ports 
			Reset => Reset,
			Clk => Clk
		);

--MyExternalADCArbiter : entity work.ExternalADCArbiter 
--    Port Map(
--			
--			-- ADC Ports 
--			DATA_IN => ADC_DATA_IN,
--			NOT_DATAREADY => ADC_NOT_DATAREADY,
--			Clk_OUT => ADC_Clk_OUT,
--			ADC_SYNC => ADC_SYNC,
--			
--			-- Other Info
--			ADC_ChannelMask => SecondaryChannelMask,
--			ChannelMask_Count => ExternalMask_Count,
--			ChannelMask_Ready => ExternalMask_Ready,
--		
--			-- Output Data (Network Side)
--			FIFO_PlusFullPacket => EADC_FIFO_PlusFullPacket,
--			FIFO_wr_en => EADC_FIFO_wr_en, 
--			FIFO_din => EADC_FIFO_din,
--			FIFO_FULL => EADC_FIFO_FULL,
--			
--			-- Information about error states 
--			FIFO_Error_Overrun => EADC_FIFO_Error_Overrun,
--			FIFO_Error_Underrun => EADC_FIFO_Error_Underrun,	 
--			
--			-- Other information for the package 
--			Timestamp => The32KHzTimeStamp,
--			TriggerChannelsIn	=> TriggerChannelsIn,
--			
--			Enable => ExternalADCIsEnabled,
--			
--			-- The usual ports 
--			Reset => Reset,
--			Clk => Clk
--		);			

-- Transmit the valid only if the emitter core is active...
MyTestPacketEmitter_FIFO_VALID <= (CollectorFIFO_Valid_Out AND TestPacketZarlink_Redirect);

MyTestPacketEmitter : entity work.TestPacketEmitter 
		Generic MAP
		(
			Module_Name => std_logic_vector(to_unsigned(2,16))
		)
    Port MAP ( 

		-- Connection to the FIFO
			Data_In => CollectorFIFO_Data_Out(111 downto 0),
			Valid_In => MyTestPacketEmitter_FIFO_VALID,
			ReadEn_In => MyTestPacketEmitter_FIFO_RdEn,
	 
		-- Connection to the Zarlink 
			Zarlink_Data_Out => Zarlink_Data_Out,
			Zarlink_WriteEn => Zarlink_WriteEn,
			Zarlink_Data_In => Zarlink_Data_In,
			Zarlink_Valid_In => Zarlink_Valid_In,
			Zarlink_BlockMode => Zarlink_BlockMode,
			Zarlink_Wait => Zarlink_Wait,
			Zarlink_RequestChannel => Zarlink_RequestChannel(1),
			Zarlink_GetChannel => Zarlink_GetChannel(1),
			Zarlink_Name => Zarlink_Name,
			Zarlink_LinkQuality_Report	=> Zarlink_LinkQuality_Report,
			Zarlink_Testemitter_Retry => Zarlink_Testemitter_Retry,
			Zarlink_EnableBlockBufferCheck => Zarlink_EnableBlockBufferCheck_Testgen,	
			Zarlink_MaxBufferFill => Zarlink_MaxBufferFill,
		-- The usual ports 
			Reset => Reset,
			Clk => Clk
	 );

Zarlink_EnableBlockBufferCheck <= NOT((NOT Zarlink_EnableBlockBufferCheck_Collector) OR (NOT Zarlink_EnableBlockBufferCheck_Testgen));

MyInputFilter : entity work.InputPacketInterpreter 
	 Generic Map(
		-- Number of listening entities... 
		LengthOfWait => 26
	 )
    Port MAP ( 
		-- FIFO IN
		FIFO_IN_Data => FIFO_IN_Data,
		FIFO_IN_Valid => FIFO_IN_Valid,
		FIFO_IN_rd_en => FIFO_IN_rd_en,

		FIFO_OUT_Data => In_Data,
		FIFO_OUT_Valid => In_Valid,
		
		Packet_Start => Packet_Start,
		Packet_End => Packet_End,
		Packet_Valid => Packet_Valid,
		Packet_Unknown	=> Packet_Unknown,
		
		WaitForListener => In_Wait,
		
		Reset => Reset,
		Clk => Clk
	 );

Zarlink_Arbiter : entity work.OutputPacketArbiter 
	Generic Map (
		-- Number Of Parties that can write... 
		NumberOfUsers => 20
	)
   Port Map ( 
		RequestChannel => Zarlink_RequestChannel,
		GrantChannel => Zarlink_GetChannel,
		InUse_External => ZarlinkInUse,
		Reset => Reset,
		Clk => Clk
	);

Network_Arbiter : entity work.OutputPacketArbiter 
	Generic Map (
		-- Number Of Parties that can write... 
		NumberOfUsers => 27
	)
   Port Map ( 
		RequestChannel => Network_RequestChannel,
		GrantChannel => Network_GetChannel,
		InUse_External => NetworkInUse,
		Reset => Reset,
		Clk => Clk
	);

	MyCRC : entity work.CRC_Checker 
	Port MAP( 
			 Network_Data => Network_Data_Internal,
			 Network_Valid => Network_WriteEn_Internal,
          CRC_Reset => CRC_Reset,
          CRC_00 => CRC_00,
          CRC_01 => CRC_01,
          CRC_10 => CRC_10,
          CRC_11 => CRC_11,
          Reset => Reset,
          Clk => Clk
	);	 

--  U.) Unknown packets (L0)
Command_U : entity work.Command_Unknown
		Port Map
		(
		-- Connection to the first Input-Filter-Layer 
			In_Data => In_Data,
			In_Valid => In_Valid,
			In_Wait => In_Wait(19),
		
		-- More information about the packets 
			Packet_Start => Packet_Start,
			Packet_End	=> Packet_End,
			Packet_Valid => Packet_Valid,
			Packet_Unknown => Packet_Unknown,
			
		-- Connection to the Network
			Network_Data => Network_Data_Internal,
			Network_WriteEn => Network_WriteEn_Internal,
			Network_Wait => Network_Wait,
			Network_RequestChannel => Network_RequestChannel(20),
			Network_GetChannel => Network_GetChannel(20),
			CRC_Reset => CRC_Reset,
			CRC_00 => CRC_00,
			CRC_01 => CRC_01,
			CRC_10 => CRC_10,
			CRC_11 => CRC_11,

		-- The usual ports 
			Reset => Reset,
			Clk => Clk
		);

--  0.) 0x00FF Allows to set some constants (dec255) (L4)
Command_1 : entity work.Command_Constants
		Generic Map
		(
			Module_Name => std_logic_vector(to_unsigned(255,16)),
			Module_HasMemory => '1',
			Module_LengthInWord => 4,
			Module_UsesZarlink => '0'
		)
		Port Map
		(
		-- Connection to the first Input-Filter-Layer 
			In_Data => In_Data,
			In_Valid => In_Valid,
			In_Wait => In_Wait(20),
		
		-- More information about the packets 
			Packet_Start => Packet_Start,
			Packet_End	=> Packet_End,
			Packet_Valid => Packet_Valid,
			Packet_Unknown => Packet_Unknown,

		-- Connection to the Zarlink 
			Zarlink_Data_Out => Zarlink_Data_Out,
			Zarlink_WriteEn => Zarlink_WriteEn,
			Zarlink_Data_In => Zarlink_Data_In,
			Zarlink_Valid_In => Zarlink_Valid_In,
			Zarlink_BlockMode => Zarlink_BlockMode,
			Zarlink_Wait => Zarlink_Wait,
			Zarlink_RequestChannel => Zarlink_RequestChannel(19),
			Zarlink_GetChannel => Zarlink_GetChannel(19),
			Zarlink_Name => Zarlink_Name,
			
		-- Connection to the Network
			Network_Data => Network_Data_Internal,
			Network_WriteEn => Network_WriteEn_Internal,
			Network_Wait => Network_Wait,
			Network_RequestChannel => Network_RequestChannel(21),
			Network_GetChannel => Network_GetChannel(21),
			CRC_Reset => CRC_Reset,
			CRC_00 => CRC_00,
			CRC_01 => CRC_01,
			CRC_10 => CRC_10,
			CRC_11 => CRC_11,

		-- The constants
			Zarlink_DebugMode => Zarlink_DebugMode,
			Zarlink_Timeout => Zarlink_Timeout,
			Zarlink_ConnectionTimeout => Zarlink_ConnectionTimeout,
			Zarlink_WaitForReset => Zarlink_WaitForReset,
			Implant_DeviceID => Implant_DeviceID,
			Zarlink_CheckBuffer => Zarlink_CheckBuffer,
			CleanUpCounter_Maximal => CleanUpCounter_Maximal,
			TestPacketgenerator => TestPacketgenerator,
			TestPacketZarlink_Redirect => TestPacketZarlink_Redirect,
			Zarlink_Implant_Mode => Zarlink_Implant_Mode,
			Zarlink_Retry => Zarlink_Retry,
			Cooldown_Counter_Max => Cooldown_Counter_Max,
			Debug_DumpMode => Debug_DumpMode,
			Zarlink_Connection_Mode => Zarlink_Connection_Mode,
			Zarlink_LinkQuality_Report => Zarlink_LinkQuality_Report,
			Zarlink_Testemitter_Retry => Zarlink_Testemitter_Retry,
			Zarlink_Testgenerator_The10thOf1MS => Zarlink_Testgenerator_The10thOf1MS,
			Zarlink_MaxBufferFill => Zarlink_MaxBufferFill,
			HalfCycle_TriggerOut => HalfCycle_TriggerOut,
			Wait_TriggerIn => Wait_TriggerIn,
			Zarlink_Length_Of_SPIClk => Zarlink_Length_Of_SPIClk,
			Zarlink_SPISpeedFlags => Zarlink_SPISpeedFlags,
			
		-- The usual ports 
			Reset => Reset,
			Clk => Clk
		);


--  1.) 0x0101 Version- and Statusinformation (dec257) (L0)
Command_2 : entity work.Command_Info
		Generic Map
		(
			Module_Name => std_logic_vector(to_unsigned(257,16)),
			Module_HasMemory => '0',
			Module_LengthInWord => 1,
			Module_UsesZarlink => '0'
		)
		Port Map
		(
		-- Connection to the first Input-Filter-Layer 
			In_Data => In_Data,
			In_Valid => In_Valid,
			In_Wait => In_Wait(2),
		
		-- More information about the packets 
			Packet_Start => Packet_Start,
			Packet_End	=> Packet_End,
			Packet_Valid => Packet_Valid,
			Packet_Unknown => Packet_Unknown,
		
		-- Connection to the Zarlink 
			Zarlink_Data_Out => Zarlink_Data_Out,
			Zarlink_WriteEn => Zarlink_WriteEn,
			Zarlink_Data_In => Zarlink_Data_In,
			Zarlink_Valid_In => Zarlink_Valid_In,
			Zarlink_BlockMode => Zarlink_BlockMode,
			Zarlink_Wait => Zarlink_Wait,
			Zarlink_RequestChannel => Zarlink_RequestChannel(2),
			Zarlink_GetChannel => Zarlink_GetChannel(2),
			Zarlink_Name => Zarlink_Name,
			
		-- Connection to the Network
			Network_Data => Network_Data_Internal,
			Network_WriteEn => Network_WriteEn_Internal,
			Network_Wait => Network_Wait,
			Network_RequestChannel => Network_RequestChannel(2),
			Network_GetChannel => Network_GetChannel(2),
			CRC_Reset => CRC_Reset,
			CRC_00 => CRC_00,
			CRC_01 => CRC_01,
			CRC_10 => CRC_10,
			CRC_11 => CRC_11,
			
			Firmware_Major_Version => Firmware_Major_Version,
			Firmware_Minor_Version => Firmware_Minor_Version,
			
		-- The usual ports 
			Reset => Reset,
			Clk => Clk
		);

--  2.) 0x0102 Open RF (dec258) (L2)
Command_3 : entity work.Command_ZarlinkConnect
		Generic Map
		(
			Module_Name => std_logic_vector(to_unsigned(258,16)),
			Module_HasMemory => '1',
			Module_LengthInWord => 2,
			Module_UsesZarlink => '1'
		)
		Port Map
		(
		-- Connection to the first Input-Filter-Layer 
			In_Data => In_Data,
			In_Valid => In_Valid,
			In_Wait => In_Wait(3),
		
		-- More information about the packets 
			Packet_Start => Packet_Start,
			Packet_End	=> Packet_End,
			Packet_Valid => Packet_Valid,
			Packet_Unknown => Packet_Unknown,
		
		-- Connection to the Zarlink 
			Zarlink_Data_Out => Zarlink_Data_Out,
			Zarlink_WriteEn => Zarlink_WriteEn,
			Zarlink_Data_In => Zarlink_Data_In,
			Zarlink_Valid_In => Zarlink_Valid_In,
			Zarlink_BlockMode => Zarlink_BlockMode,
			Zarlink_Wait => Zarlink_Wait,
			Zarlink_RequestChannel => Zarlink_RequestChannel(3),
			Zarlink_GetChannel => Zarlink_GetChannel(3),
			Zarlink_Name => Zarlink_Name,
			Zarlink_Error => Zarlink_Error,
			
		-- Connection to the Network
			Network_Data => Network_Data_Internal,
			Network_WriteEn => Network_WriteEn_Internal,
			Network_Wait => Network_Wait,
			Network_RequestChannel => Network_RequestChannel(3),
			Network_GetChannel => Network_GetChannel(3),
			CRC_Reset => CRC_Reset,
			CRC_00 => CRC_00,
			CRC_01 => CRC_01,
			CRC_10 => CRC_10,
			CRC_11 => CRC_11,
			
			Zarlink_Implant_Mode => Zarlink_Implant_Mode,
			Zarlink_Connection_Mode => Zarlink_Connection_Mode,
			Zarlink_SPISpeedFlags => Zarlink_SPISpeedFlags,
			Zarlink_Retry => Zarlink_Retry,
			
		-- Constants 
			Zarlink_WaitForReset => Zarlink_WaitForReset,
			Zarlink_ConnectionTimeout => Zarlink_ConnectionTimeout,
			
		-- The usual ports 
			Reset => Reset,
			Clk => Clk
		);

--  3.) 0x0103 Close RF (dec259) (L0)
Command_4 : entity work.Command_RFClose
		Generic Map
		(
			Module_Name => std_logic_vector(to_unsigned(259,16)),
			Module_HasMemory => '0',
			Module_LengthInWord => 1,
			Module_UsesZarlink => '1'
		)
		Port Map
		(
		-- Connection to the first Input-Filter-Layer 
			In_Data => In_Data,
			In_Valid => In_Valid,
			In_Wait => In_Wait(4),
		
		-- More information about the packets 
			Packet_Start => Packet_Start,
			Packet_End	=> Packet_End,
			Packet_Valid => Packet_Valid,
			Packet_Unknown => Packet_Unknown,
		
		-- Connection to the Zarlink 
			Zarlink_Data_Out => Zarlink_Data_Out,
			Zarlink_WriteEn => Zarlink_WriteEn,
			Zarlink_Data_In => Zarlink_Data_In,
			Zarlink_Valid_In => Zarlink_Valid_In,
			Zarlink_BlockMode => Zarlink_BlockMode,
			Zarlink_Wait => Zarlink_Wait,
			Zarlink_RequestChannel => Zarlink_RequestChannel(4),
			Zarlink_GetChannel => Zarlink_GetChannel(4),
			Zarlink_Name => Zarlink_Name,
		
		-- Connection to the Network
			Network_Data => Network_Data_Internal,
			Network_WriteEn => Network_WriteEn_Internal,
			Network_Wait => Network_Wait,
			Network_RequestChannel => Network_RequestChannel(4),
			Network_GetChannel => Network_GetChannel(4),
			CRC_Reset => CRC_Reset,
			CRC_00 => CRC_00,
			CRC_01 => CRC_01,
			CRC_10 => CRC_10,
			CRC_11 => CRC_11,

		-- The usual ports 
			Reset => Reset,
			Clk => Clk
		);

--  4.) 0x0263 Set Channel Mask (dec611) (L8)
Command_5 : entity work.Command_ChannelMask
		Generic Map
		(
			Module_Name => std_logic_vector(to_unsigned(611,16)),
			Module_HasMemory => '1',
			Module_LengthInWord => 8,
			Module_UsesZarlink => '1'
		)
		Port Map
		(
		-- Connection to the first Input-Filter-Layer 
			In_Data => In_Data,
			In_Valid => In_Valid,
			In_Wait => In_Wait(5),
		
		-- More information about the packets 
			Packet_Start => Packet_Start,
			Packet_End	=> Packet_End,
			Packet_Valid => Packet_Valid,
			Packet_Unknown => Packet_Unknown,
		
		-- Connection to the Zarlink 
			Zarlink_Data_Out => Zarlink_Data_Out,
			Zarlink_WriteEn => Zarlink_WriteEn,
			Zarlink_Data_In => Zarlink_Data_In,
			Zarlink_Valid_In => Zarlink_Valid_In,
			Zarlink_BlockMode => Zarlink_BlockMode,
			Zarlink_Wait => Zarlink_Wait,
			Zarlink_RequestChannel => Zarlink_RequestChannel(5),
			Zarlink_GetChannel => Zarlink_GetChannel(5),
			Zarlink_Name => Zarlink_Name,
		
		-- Connection to the Network
			Network_Data => Network_Data_Internal,
			Network_WriteEn => Network_WriteEn_Internal,
			Network_Wait => Network_Wait,
			Network_RequestChannel => Network_RequestChannel(5),
			Network_GetChannel => Network_GetChannel(5),
			CRC_Reset => CRC_Reset,
			CRC_00 => CRC_00,
			CRC_01 => CRC_01,
			CRC_10 => CRC_10,
			CRC_11 => CRC_11,
			
			Implant_ChannelMask => Implant_ChannelMask,
			Implant_DeviceID => Implant_DeviceID,
			
		-- The usual ports 
			Reset => Reset,
			Clk => Clk
		);

--  5.) 0x0272 Set AD Resolution (dec626) (L1)
Command_6 : entity work.Command_ADCResolution
		Generic Map
		(
			Module_Name => std_logic_vector(to_unsigned(626,16)),
			Module_HasMemory => '1',
			Module_LengthInWord => 1,
			Module_UsesZarlink => '1'
		)
		Port Map
		(
		-- Connection to the first Input-Filter-Layer 
			In_Data => In_Data,
			In_Valid => In_Valid,
			In_Wait => In_Wait(6),
		
		-- More information about the packets 
			Packet_Start => Packet_Start,
			Packet_End	=> Packet_End,
			Packet_Valid => Packet_Valid,
			Packet_Unknown => Packet_Unknown,
		
		-- Connection to the Zarlink 
			Zarlink_Data_Out => Zarlink_Data_Out,
			Zarlink_WriteEn => Zarlink_WriteEn,
			Zarlink_Data_In => Zarlink_Data_In,
			Zarlink_Valid_In => Zarlink_Valid_In,
			Zarlink_BlockMode => Zarlink_BlockMode,
			Zarlink_Wait => Zarlink_Wait,
			Zarlink_RequestChannel => Zarlink_RequestChannel(6),
			Zarlink_GetChannel => Zarlink_GetChannel(6),
			Zarlink_Name => Zarlink_Name,
		
		-- Connection to the Network
			Network_Data => Network_Data_Internal,
			Network_WriteEn => Network_WriteEn_Internal,
			Network_Wait => Network_Wait,
			Network_RequestChannel => Network_RequestChannel(6),
			Network_GetChannel => Network_GetChannel(6),
			CRC_Reset => CRC_Reset,
			CRC_00 => CRC_00,
			CRC_01 => CRC_01,
			CRC_10 => CRC_10,
			CRC_11 => CRC_11,

			Implant_ADCResolution => Implant_ADCResolution,
			Implant_DeviceID => Implant_DeviceID,
		-- The usual ports 
			Reset => Reset,
			Clk => Clk
		);

--  6.) 0x0273 Set Samplerate (dec627) (L1)
Command_7 : entity work.Command_SampleRate
		Generic Map
		(
			Module_Name => std_logic_vector(to_unsigned(627,16)),
			Module_HasMemory => '1',
			Module_LengthInWord => 1,
			Module_UsesZarlink => '1'
		)
		Port Map
		(
		-- Connection to the first Input-Filter-Layer 
			In_Data => In_Data,
			In_Valid => In_Valid,
			In_Wait => In_Wait(7),
		
		-- More information about the packets 
			Packet_Start => Packet_Start,
			Packet_End	=> Packet_End,
			Packet_Valid => Packet_Valid,
			Packet_Unknown => Packet_Unknown,
		
		-- Connection to the Zarlink 
			Zarlink_Data_Out => Zarlink_Data_Out,
			Zarlink_WriteEn => Zarlink_WriteEn,
			Zarlink_Data_In => Zarlink_Data_In,
			Zarlink_Valid_In => Zarlink_Valid_In,
			Zarlink_BlockMode => Zarlink_BlockMode,
			Zarlink_Wait => Zarlink_Wait,
			Zarlink_RequestChannel => Zarlink_RequestChannel(7),
			Zarlink_GetChannel => Zarlink_GetChannel(7),
			Zarlink_Name => Zarlink_Name,
		
		-- Connection to the Network
			Network_Data => Network_Data_Internal,
			Network_WriteEn => Network_WriteEn_Internal,
			Network_Wait => Network_Wait,
			Network_RequestChannel => Network_RequestChannel(7),
			Network_GetChannel => Network_GetChannel(7),
			CRC_Reset => CRC_Reset,
			CRC_00 => CRC_00,
			CRC_01 => CRC_01,
			CRC_10 => CRC_10,
			CRC_11 => CRC_11,
			
			Implant_SampleRate => Implant_SampleRate,
			Implant_DeviceID => Implant_DeviceID,
		-- The usual ports 
			Reset => Reset,
			Clk => Clk
		);

--  7.) 0x0266 Set Filter (dec614) (L1)
Command_8 : entity work.Command_Filter
		Generic Map
		(
			Module_Name => std_logic_vector(to_unsigned(614,16)),
			Module_HasMemory => '1',
			Module_LengthInWord => 1,
			Module_UsesZarlink => '1'
		)
		Port Map
		(
		-- Connection to the first Input-Filter-Layer 
			In_Data => In_Data,
			In_Valid => In_Valid,
			In_Wait => In_Wait(8),
		
		-- More information about the packets 
			Packet_Start => Packet_Start,
			Packet_End	=> Packet_End,
			Packet_Valid => Packet_Valid,
			Packet_Unknown => Packet_Unknown,
		
		-- Connection to the Zarlink 
			Zarlink_Data_Out => Zarlink_Data_Out,
			Zarlink_WriteEn => Zarlink_WriteEn,
			Zarlink_Data_In => Zarlink_Data_In,
			Zarlink_Valid_In => Zarlink_Valid_In,
			Zarlink_BlockMode => Zarlink_BlockMode,
			Zarlink_Wait => Zarlink_Wait,
			Zarlink_RequestChannel => Zarlink_RequestChannel(8),
			Zarlink_GetChannel => Zarlink_GetChannel(8),
			Zarlink_Name => Zarlink_Name,
		
		-- Connection to the Network
			Network_Data => Network_Data_Internal,
			Network_WriteEn => Network_WriteEn_Internal,
			Network_Wait => Network_Wait,
			Network_RequestChannel => Network_RequestChannel(8),
			Network_GetChannel => Network_GetChannel(8),
			CRC_Reset => CRC_Reset,
			CRC_00 => CRC_00,
			CRC_01 => CRC_01,
			CRC_10 => CRC_10,
			CRC_11 => CRC_11,
			
			Implant_Filter => Implant_Filter,
			Implant_DeviceID => Implant_DeviceID,
		-- The usual ports 
			Reset => Reset,
			Clk => Clk
		);

----  8.) 0x027A Channel Mask Basestation (dec634) (L3)
--Command_9 : entity work.Command_SetSecondaryChannelMask
--		Generic Map
--		(
--			Module_Name => std_logic_vector(to_unsigned(634,16)),
--			Module_HasMemory => '1',
--			Module_LengthInWord => 2,
--			Module_UsesZarlink => '0'
--		)
--		Port Map
--		(
--		-- Connection to the first Input-Filter-Layer 
--			In_Data => In_Data,
--			In_Valid => In_Valid,
--			In_Wait => In_Wait(9),
--		
--		-- More information about the packets 
--			Packet_Start => Packet_Start,
--			Packet_End	=> Packet_End,
--			Packet_Valid => Packet_Valid,
--			Packet_Unknown => Packet_Unknown,
--		
--		-- Connection to the Zarlink 
--			Zarlink_Data_Out => Zarlink_Data_Out,
--			Zarlink_WriteEn => Zarlink_WriteEn,
--			Zarlink_Data_In => Zarlink_Data_In,
--			Zarlink_Valid_In => Zarlink_Valid_In,
--			Zarlink_BlockMode => Zarlink_BlockMode,
--			Zarlink_Wait => Zarlink_Wait,
--			Zarlink_RequestChannel => Zarlink_RequestChannel(9),
--			Zarlink_GetChannel => Zarlink_GetChannel(9),
--			Zarlink_Name => Zarlink_Name,
--		
--		-- Connection to the Network
--			Network_Data => Network_Data_Internal,
--			Network_WriteEn => Network_WriteEn_Internal,
--			Network_Wait => Network_Wait,
--			Network_RequestChannel => Network_RequestChannel(9),
--			Network_GetChannel => Network_GetChannel(9),
--			CRC_Reset => CRC_Reset,
--			CRC_00 => CRC_00,
--			CRC_01 => CRC_01,
--			CRC_10 => CRC_10,
--			CRC_11 => CRC_11,
--
--		-- Speical Ports
--		SecondaryChannelMask	=> SecondaryChannelMask,
--
--		-- The usual ports 
--			Reset => Reset,
--			Clk => Clk
--		);
Zarlink_RequestChannel(9) <= '0';
Network_RequestChannel(9) <= '0';

--  9.) 0x0301 Start DAC (dec769) (L0)
Command_10 : entity work.Command_DAQ_Start
		Generic Map
		(
			Module_Name => std_logic_vector(to_unsigned(769,16)),
			Module_HasMemory => '0',
			Module_LengthInWord => 1,
			Module_UsesZarlink => '1'
		)
		Port Map
		(
		-- Connection to the first Input-Filter-Layer 
			In_Data => In_Data,
			In_Valid => In_Valid,
			In_Wait => In_Wait(10),
		
		-- More information about the packets 
			Packet_Start => Packet_Start,
			Packet_End	=> Packet_End,
			Packet_Valid => Packet_Valid,
			Packet_Unknown => Packet_Unknown,
		
			Controll_DAQ => Start_DAQ,
			
		-- Connection to the Zarlink 
			Zarlink_Data_Out => Zarlink_Data_Out,
			Zarlink_WriteEn => Zarlink_WriteEn,
			Zarlink_Data_In => Zarlink_Data_In,
			Zarlink_Valid_In => Zarlink_Valid_In,
			Zarlink_BlockMode => Zarlink_BlockMode,
			Zarlink_Wait => Zarlink_Wait,
			Zarlink_RequestChannel => Zarlink_RequestChannel(10),
			Zarlink_GetChannel => Zarlink_GetChannel(10),
			Zarlink_Name => Zarlink_Name,
		
		-- Connection to the Network
			Network_Data => Network_Data_Internal,
			Network_WriteEn => Network_WriteEn_Internal,
			Network_Wait => Network_Wait,
			Network_RequestChannel => Network_RequestChannel(10),
			Network_GetChannel => Network_GetChannel(10),
			CRC_Reset => CRC_Reset,
			CRC_00 => CRC_00,
			CRC_01 => CRC_01,
			CRC_10 => CRC_10,
			CRC_11 => CRC_11,

			Implant_DeviceID => Implant_DeviceID,
		-- The usual ports 
			Reset => Reset,
			Clk => Clk
		);

-- 10.) 0x0302 Ende DAC (dec770) (L0)
Command_11 : entity work.Command_DAQ_Stop
		Generic Map
		(
			Module_Name => std_logic_vector(to_unsigned(770,16)),
			Module_HasMemory => '0',
			Module_LengthInWord => 1,
			Module_UsesZarlink => '1'
		)
		Port Map
		(
		-- Connection to the first Input-Filter-Layer 
			In_Data => In_Data,
			In_Valid => In_Valid,
			In_Wait => In_Wait(11),
		
		-- More information about the packets 
			Packet_Start => Packet_Start,
			Packet_End	=> Packet_End,
			Packet_Valid => Packet_Valid,
			Packet_Unknown => Packet_Unknown,
		
			Controll_DAQ => Stop_DAQ,
			
		-- Connection to the Zarlink 
			Zarlink_Data_Out => Zarlink_Data_Out,
			Zarlink_WriteEn => Zarlink_WriteEn,
			Zarlink_Data_In => Zarlink_Data_In,
			Zarlink_Valid_In => Zarlink_Valid_In,
			Zarlink_BlockMode => Zarlink_BlockMode,
			Zarlink_Wait => Zarlink_Wait,
			Zarlink_RequestChannel => Zarlink_RequestChannel(11),
			Zarlink_GetChannel => Zarlink_GetChannel(11),
			Zarlink_Name => Zarlink_Name,
		
		-- Connection to the Network
			Network_Data => Network_Data_Internal,
			Network_WriteEn => Network_WriteEn_Internal,
			Network_Wait => Network_Wait,
			Network_RequestChannel => Network_RequestChannel(11),
			Network_GetChannel => Network_GetChannel(11),
			CRC_Reset => CRC_Reset,
			CRC_00 => CRC_00,
			CRC_01 => CRC_01,
			CRC_10 => CRC_10,
			CRC_11 => CRC_11,

			Implant_DeviceID => Implant_DeviceID,
		-- The usual ports 
			Reset => Reset,
			Clk => Clk
		);

-- 11.) 0x0400 Read Zarlink Register 0(dec1024) (L1)
Command_12 : entity work.Command_SingleRead0
		Generic Map
		(
			Module_Name => std_logic_vector(to_unsigned(1024,16)),
			Module_HasMemory => '1',
			Module_LengthInWord => 1,
			Module_UsesZarlink => '1'
		)
		Port Map
		(
		-- Connection to the first Input-Filter-Layer 
			In_Data => In_Data,
			In_Valid => In_Valid,
			In_Wait => In_Wait(12),
		
		-- More information about the packets 
			Packet_Start => Packet_Start,
			Packet_End	=> Packet_End,
			Packet_Valid => Packet_Valid,
			Packet_Unknown => Packet_Unknown,
		
		-- Connection to the Zarlink 
			Zarlink_Data_Out => Zarlink_Data_Out,
			Zarlink_WriteEn => Zarlink_WriteEn,
			Zarlink_Data_In => Zarlink_Data_In,
			Zarlink_Valid_In => Zarlink_Valid_In,
			Zarlink_BlockMode => Zarlink_BlockMode,
			Zarlink_Wait => Zarlink_Wait,
			Zarlink_RequestChannel => Zarlink_RequestChannel(12),
			Zarlink_GetChannel => Zarlink_GetChannel(12),
			Zarlink_Name => Zarlink_Name,
		
		-- Connection to the Network
			Network_Data => Network_Data_Internal,
			Network_WriteEn => Network_WriteEn_Internal,
			Network_Wait => Network_Wait,
			Network_RequestChannel => Network_RequestChannel(12),
			Network_GetChannel => Network_GetChannel(12),
			CRC_Reset => CRC_Reset,
			CRC_00 => CRC_00,
			CRC_01 => CRC_01,
			CRC_10 => CRC_10,
			CRC_11 => CRC_11,

		-- The usual ports 
			Reset => Reset,
			Clk => Clk
		);

-- 12.) 0x0401 Read Zarlink Register 1(dec1025) (L1)
Command_13 : entity work.Command_SingleRead1
		Generic Map
		(
			Module_Name => std_logic_vector(to_unsigned(1025,16)),
			Module_HasMemory => '1',
			Module_LengthInWord => 1,
			Module_UsesZarlink => '1'
		)
		Port Map
		(
		-- Connection to the first Input-Filter-Layer 
			In_Data => In_Data,
			In_Valid => In_Valid,
			In_Wait => In_Wait(13),
		
		-- More information about the packets 
			Packet_Start => Packet_Start,
			Packet_End	=> Packet_End,
			Packet_Valid => Packet_Valid,
			Packet_Unknown => Packet_Unknown,
		
		-- Connection to the Zarlink 
			Zarlink_Data_Out => Zarlink_Data_Out,
			Zarlink_WriteEn => Zarlink_WriteEn,
			Zarlink_Data_In => Zarlink_Data_In,
			Zarlink_Valid_In => Zarlink_Valid_In,
			Zarlink_BlockMode => Zarlink_BlockMode,
			Zarlink_Wait => Zarlink_Wait,
			Zarlink_RequestChannel => Zarlink_RequestChannel(13),
			Zarlink_GetChannel => Zarlink_GetChannel(13),
			Zarlink_Name => Zarlink_Name,
		
		-- Connection to the Network
			Network_Data => Network_Data_Internal,
			Network_WriteEn => Network_WriteEn_Internal,
			Network_Wait => Network_Wait,
			Network_RequestChannel => Network_RequestChannel(13),
			Network_GetChannel => Network_GetChannel(13),
			CRC_Reset => CRC_Reset,
			CRC_00 => CRC_00,
			CRC_01 => CRC_01,
			CRC_10 => CRC_10,
			CRC_11 => CRC_11,

		-- The usual ports 
			Reset => Reset,
			Clk => Clk
		);

-- 13.) 0x0402 Write Zarlink Register 0(dec1026) (L1)
Command_14 : entity work.Command_SingleWrite0
		Generic Map
		(
			Module_Name => std_logic_vector(to_unsigned(1026,16)),
			Module_HasMemory => '1',
			Module_LengthInWord => 1,
			Module_UsesZarlink => '1'
		)
		Port Map
		(
		-- Connection to the first Input-Filter-Layer 
			In_Data => In_Data,
			In_Valid => In_Valid,
			In_Wait => In_Wait(14),
		
		-- More information about the packets 
			Packet_Start => Packet_Start,
			Packet_End	=> Packet_End,
			Packet_Valid => Packet_Valid,
			Packet_Unknown => Packet_Unknown,
		
		-- Connection to the Zarlink 
			Zarlink_Data_Out => Zarlink_Data_Out,
			Zarlink_WriteEn => Zarlink_WriteEn,
			Zarlink_Data_In => Zarlink_Data_In,
			Zarlink_Valid_In => Zarlink_Valid_In,
			Zarlink_BlockMode => Zarlink_BlockMode,
			Zarlink_Wait => Zarlink_Wait,
			Zarlink_RequestChannel => Zarlink_RequestChannel(14),
			Zarlink_GetChannel => Zarlink_GetChannel(14),
			Zarlink_Name => Zarlink_Name,
		
		-- Connection to the Network
			Network_Data => Network_Data_Internal,
			Network_WriteEn => Network_WriteEn_Internal,
			Network_Wait => Network_Wait,
			Network_RequestChannel => Network_RequestChannel(14),
			Network_GetChannel => Network_GetChannel(14),
			CRC_Reset => CRC_Reset,
			CRC_00 => CRC_00,
			CRC_01 => CRC_01,
			CRC_10 => CRC_10,
			CRC_11 => CRC_11,

		-- The usual ports 
			Reset => Reset,
			Clk => Clk
		);

-- 14.) 0x0403 Write Zarlink Register 1(dec1027) (L1)
Command_15 : entity work.Command_SingleWrite1
		Generic Map
		(
			Module_Name => std_logic_vector(to_unsigned(1027,16)),
			Module_HasMemory => '1',
			Module_LengthInWord => 1,
			Module_UsesZarlink => '1'
		)
		Port Map
		(
		-- Connection to the first Input-Filter-Layer 
			In_Data => In_Data,
			In_Valid => In_Valid,
			In_Wait => In_Wait(15),
		
		-- More information about the packets 
			Packet_Start => Packet_Start,
			Packet_End	=> Packet_End,
			Packet_Valid => Packet_Valid,
			Packet_Unknown => Packet_Unknown,
		
		-- Connection to the Zarlink 
			Zarlink_Data_Out => Zarlink_Data_Out,
			Zarlink_WriteEn => Zarlink_WriteEn,
			Zarlink_Data_In => Zarlink_Data_In,
			Zarlink_Valid_In => Zarlink_Valid_In,
			Zarlink_BlockMode => Zarlink_BlockMode,
			Zarlink_Wait => Zarlink_Wait,
			Zarlink_RequestChannel => Zarlink_RequestChannel(15),
			Zarlink_GetChannel => Zarlink_GetChannel(15),
			Zarlink_Name => Zarlink_Name,
		
		-- Connection to the Network
			Network_Data => Network_Data_Internal,
			Network_WriteEn => Network_WriteEn_Internal,
			Network_Wait => Network_Wait,
			Network_RequestChannel => Network_RequestChannel(15),
			Network_GetChannel => Network_GetChannel(15),
			CRC_Reset => CRC_Reset,
			CRC_00 => CRC_00,
			CRC_01 => CRC_01,
			CRC_10 => CRC_10,
			CRC_11 => CRC_11,

		-- The usual ports 
			Reset => Reset,
			Clk => Clk
		);

-- 15.) 0x0404 Write Zarlink Block(dec1028) (L7)
Command_16 : entity work.Command_BlockWrite
		Generic Map
		(
			Module_Name => std_logic_vector(to_unsigned(1028,16)),
			Module_HasMemory => '1',
			Module_LengthInWord => 7,
			Module_UsesZarlink => '1'
		)
		Port Map
		(
		-- Connection to the first Input-Filter-Layer 
			In_Data => In_Data,
			In_Valid => In_Valid,
			In_Wait => In_Wait(16),
		
		-- More information about the packets 
			Packet_Start => Packet_Start,
			Packet_End	=> Packet_End,
			Packet_Valid => Packet_Valid,
			Packet_Unknown => Packet_Unknown,
		
		-- Connection to the Zarlink 
			Zarlink_Data_Out => Zarlink_Data_Out,
			Zarlink_WriteEn => Zarlink_WriteEn,
			Zarlink_Data_In => Zarlink_Data_In,
			Zarlink_Valid_In => Zarlink_Valid_In,
			Zarlink_BlockMode => Zarlink_BlockMode,
			Zarlink_Wait => Zarlink_Wait,
			Zarlink_RequestChannel => Zarlink_RequestChannel(16),
			Zarlink_GetChannel => Zarlink_GetChannel(16),
			Zarlink_Name => Zarlink_Name,
		
		-- Connection to the Network
			Network_Data => Network_Data_Internal,
			Network_WriteEn => Network_WriteEn_Internal,
			Network_Wait => Network_Wait,
			Network_RequestChannel => Network_RequestChannel(16),
			Network_GetChannel => Network_GetChannel(16),
			CRC_Reset => CRC_Reset,
			CRC_00 => CRC_00,
			CRC_01 => CRC_01,
			CRC_10 => CRC_10,
			CRC_11 => CRC_11,

		-- The usual ports 
			Reset => Reset,
			Clk => Clk
		);

-- 16.) 0x0405 Read Zarlink Block(dec1029) (L0)
Command_17 : entity work.Command_BlockRead
		Generic Map
		(
			Module_Name => std_logic_vector(to_unsigned(1029,16)),
			Module_HasMemory => '0',
			Module_LengthInWord => 7,
			Module_UsesZarlink => '1'
		)
		Port Map
		(
		-- Connection to the first Input-Filter-Layer 
			In_Data => In_Data,
			In_Valid => In_Valid,
			In_Wait => In_Wait(17),
		
		-- More information about the packets 
			Packet_Start => Packet_Start,
			Packet_End	=> Packet_End,
			Packet_Valid => Packet_Valid,
			Packet_Unknown => Packet_Unknown,
		
		-- Connection to the Zarlink 
			Zarlink_Data_Out => Zarlink_Data_Out,
			Zarlink_WriteEn => Zarlink_WriteEn,
			Zarlink_Data_In => Zarlink_Data_In,
			Zarlink_Valid_In => Zarlink_Valid_In,
			Zarlink_BlockMode => Zarlink_BlockMode,
			Zarlink_Wait => Zarlink_Wait,
			Zarlink_RequestChannel => Zarlink_RequestChannel(17),
			Zarlink_GetChannel => Zarlink_GetChannel(17),
			Zarlink_Name => Zarlink_Name,
			Zarlink_Error => Zarlink_Error,
			
		-- Connection to the Network
			Network_Data => Network_Data_Internal,
			Network_WriteEn => Network_WriteEn_Internal,
			Network_Wait => Network_Wait,
			Network_RequestChannel => Network_RequestChannel(17),
			Network_GetChannel => Network_GetChannel(17),
			CRC_Reset => CRC_Reset,
			CRC_00 => CRC_00,
			CRC_01 => CRC_01,
			CRC_10 => CRC_10,
			CRC_11 => CRC_11,

		-- The usual ports 
			Reset => Reset,
			Clk => Clk
		);

-- 17.) 0x0500 Set Trigger Channel Basestation(dec1280) (L1)
Command_18 : entity work.Command_SetTriggerChannels
		Generic Map
		(
			Module_Name => std_logic_vector(to_unsigned(1280,16)),
			Module_HasMemory => '1',
			Module_LengthInWord => 2,
			Module_UsesZarlink => '0'
		)
		Port Map
		(
		-- Connection to the first Input-Filter-Layer 
			In_Data => In_Data,
			In_Valid => In_Valid,
			In_Wait => In_Wait(18),
		
		-- More information about the packets 
			Packet_Start => Packet_Start,
			Packet_End	=> Packet_End,
			Packet_Valid => Packet_Valid,
			Packet_Unknown => Packet_Unknown,
		
		-- Connection to the Zarlink 
			Zarlink_Data_Out => Zarlink_Data_Out,
			Zarlink_WriteEn => Zarlink_WriteEn,
			Zarlink_Data_In => Zarlink_Data_In,
			Zarlink_Valid_In => Zarlink_Valid_In,
			Zarlink_BlockMode => Zarlink_BlockMode,
			Zarlink_Wait => Zarlink_Wait,
			Zarlink_RequestChannel => Zarlink_RequestChannel(18),
			Zarlink_GetChannel => Zarlink_GetChannel(18),
			Zarlink_Name => Zarlink_Name,
		
		-- Connection to the Network
			Network_Data => Network_Data_Internal,
			Network_WriteEn => Network_WriteEn_Internal,
			Network_Wait => Network_Wait,
			Network_RequestChannel => Network_RequestChannel(18),
			Network_GetChannel => Network_GetChannel(18),
			CRC_Reset => CRC_Reset,
			CRC_00 => CRC_00,
			CRC_01 => CRC_01,
			CRC_10 => CRC_10,
			CRC_11 => CRC_11,

		-- Special Ports 
			TriggerChannelsOut => TriggerChannelsOut,

		-- The usual ports 
			Reset => Reset,
			Clk => Clk
		);

---- 18.) 0x0279 Set Amplification in the Basestation(dec633) (L2)
--Command_19 : entity work.Command_SetAmplifier
--		Generic Map
--		(
--			Module_Name => std_logic_vector(to_unsigned(633,16)),
--			Module_HasMemory => '1',
--			Module_LengthInWord => 2
--		)
--		Port Map
--		(
--		-- Connection to the first Input-Filter-Layer 
--			In_Data => In_Data,
--			In_Valid => In_Valid,
--			In_Wait => In_Wait(21),
--		
--		-- More information about the packets 
--			Packet_Start => Packet_Start,
--			Packet_End	=> Packet_End,
--			Packet_Valid => Packet_Valid,
--			Packet_Unknown => Packet_Unknown,
--		
--		-- External VGA Ports
--			VGA_CS => VGA_CS,
--			VGA_DATA => VGA_DATA,
--			VGA_CLK => VGA_CLK,
--		
--		-- Connection to the Network
--			Network_Data => Network_Data_Internal,
--			Network_WriteEn => Network_WriteEn_Internal,
--			Network_Wait => Network_Wait,
--			Network_RequestChannel => Network_RequestChannel(22),
--			Network_GetChannel => Network_GetChannel(22),
--			CRC_Reset => CRC_Reset,
--			CRC_00 => CRC_00,
--			CRC_01 => CRC_01,
--			CRC_10 => CRC_10,
--			CRC_11 => CRC_11,
--
--
--		-- The usual ports 
--			Reset => Reset,
--			Clk => Clk
--		);
Network_RequestChannel(22) <= '0';

-- 20.) 0x0502 Get Used Channels and their order(dec1282) (L0)
Command_20 : entity work.Command_GetUsedChannels
		Generic Map
		(
			Module_Name => std_logic_vector(to_unsigned(1282,16)),
			Module_HasMemory => '0',
			Module_LengthInWord => 1
		)
		Port Map
		(
		-- Connection to the first Input-Filter-Layer 
			In_Data => In_Data,
			In_Valid => In_Valid,
			In_Wait => In_Wait(22),
		
		-- More information about the packets 
			Packet_Start => Packet_Start,
			Packet_End	=> Packet_End,
			Packet_Valid => Packet_Valid,
			Packet_Unknown => Packet_Unknown,

		-- Connection to the Network
			Network_Data => Network_Data_Internal,
			Network_WriteEn => Network_WriteEn_Internal,
			Network_Wait => Network_Wait,
			Network_RequestChannel => Network_RequestChannel(23),
			Network_GetChannel => Network_GetChannel(23),
			CRC_Reset => CRC_Reset,
			CRC_00 => CRC_00,
			CRC_01 => CRC_01,
			CRC_10 => CRC_10,
			CRC_11 => CRC_11,

			Implant_ChannelMask_Resorted => Implant_ChannelMask_Resorted,
			ChannelMask_Count => ChannelMask_Count,
			ChannelMask_Ready => ChannelMask_Ready,
			
		-- The usual ports 
			Reset => Reset,
			Clk => Clk
		);

--  21.) 0x0303 Start DAC (dec769) (L0)
--Command_21 : entity work.Command_ExternalDAQ_Start
--		Generic Map
--		(
--			Module_Name => std_logic_vector(to_unsigned(771,16)),
--			Module_HasMemory => '0',
--			Module_LengthInWord => 1
--		)
--		Port Map
--		(
--		-- Connection to the first Input-Filter-Layer 
--			In_Data => In_Data,
--			In_Valid => In_Valid,
--			In_Wait => In_Wait(23),
--		
--		-- More information about the packets 
--			Packet_Start => Packet_Start,
--			Packet_End	=> Packet_End,
--			Packet_Valid => Packet_Valid,
--			Packet_Unknown => Packet_Unknown,
--		
--			Controll_DAQ => Start_ExternalDAQ,
--			
--		-- Connection to the Network
--			Network_Data => Network_Data_Internal,
--			Network_WriteEn => Network_WriteEn_Internal,
--			Network_Wait => Network_Wait,
--			Network_RequestChannel => Network_RequestChannel(24),
--			Network_GetChannel => Network_GetChannel(24),
--			CRC_Reset => CRC_Reset,
--			CRC_00 => CRC_00,
--			CRC_01 => CRC_01,
--			CRC_10 => CRC_10,
--			CRC_11 => CRC_11,
--
--		-- The usual ports 
--			Reset => Reset,
--			Clk => Clk
--		);
Network_RequestChannel(24) <= '0';
-- 22.) 0x0304 Ende DAC (dec772) (L0)
--Command_22 : entity work.Command_ExternalDAQ_Start
--		Generic Map
--		(
--			Module_Name => std_logic_vector(to_unsigned(772,16)),
--			Module_HasMemory => '0',
--			Module_LengthInWord => 1
--		)
--		Port Map
--		(
--		-- Connection to the first Input-Filter-Layer 
--			In_Data => In_Data,
--			In_Valid => In_Valid,
--			In_Wait => In_Wait(24),
--		
--		-- More information about the packets 
--			Packet_Start => Packet_Start,
--			Packet_End	=> Packet_End,
--			Packet_Valid => Packet_Valid,
--			Packet_Unknown => Packet_Unknown,
--		
--			Controll_DAQ => Stop_ExternalDAQ,
--	
--		-- Connection to the Network
--			Network_Data => Network_Data_Internal,
--			Network_WriteEn => Network_WriteEn_Internal,
--			Network_Wait => Network_Wait,
--			Network_RequestChannel => Network_RequestChannel(25),
--			Network_GetChannel => Network_GetChannel(25),
--			CRC_Reset => CRC_Reset,
--			CRC_00 => CRC_00,
--			CRC_01 => CRC_01,
--			CRC_10 => CRC_10,
--			CRC_11 => CRC_11,
--
--		-- The usual ports 
--			Reset => Reset,
--			Clk => Clk
--		);
--
Network_RequestChannel(25) <= '0';

-- 23.) 0x0503 Get the input trigger channels(dec1283) (L0)
Command_23 : entity work.Command_GetTriggerIn
		Generic Map
		(
			Module_Name => std_logic_vector(to_unsigned(1283,16)),
			Module_HasMemory => '0',
			Module_LengthInWord => 1
		)
		Port Map
		(
		-- Connection to the first Input-Filter-Layer 
			In_Data => In_Data,
			In_Valid => In_Valid,
			In_Wait => In_Wait(25),
		
		-- More information about the packets 
			Packet_Start => Packet_Start,
			Packet_End	=> Packet_End,
			Packet_Valid => Packet_Valid,
			Packet_Unknown => Packet_Unknown,

		-- Connection to the Network
			Network_Data => Network_Data_Internal,
			Network_WriteEn => Network_WriteEn_Internal,
			Network_Wait => Network_Wait,
			Network_RequestChannel => Network_RequestChannel(26),
			Network_GetChannel => Network_GetChannel(26),
			CRC_Reset => CRC_Reset,
			CRC_00 => CRC_00,
			CRC_01 => CRC_01,
			CRC_10 => CRC_10,
			CRC_11 => CRC_11,
		
			TriggerChannelsIn => TriggerChannelsIn,
		
		-- The usual ports 
			Reset => Reset,
			Clk => Clk
		);


PollingSwitch : entity work.BiStableSwitchPolling
    Port MAP ( 		
	 
		-- Enable / Disable
		Enable => Start_DAQ,
		Disable => Stop_DAQ,
		
		-- Result 
		PollingIsEnabled => PollingIsEnabled,
		
		-- The usual ports 
		Reset => Reset,
		Clk => Clk
	);

PollingSwitchExternal : entity work.BiStableSwitchPolling
    Port MAP ( 		
	 
		-- Enable / Disable
		Enable => Start_ExternalDAQ,
		Disable => Stop_ExternalDAQ,
		
		-- Result 
		PollingIsEnabled => ExternalADCIsEnabled,
		
		-- The usual ports 
		Reset => Reset,
		Clk => Clk
	);


PollingClock : entity work.ClockForPolling
    Port MAP( 
		-- Constant with maximum counter value
		Zarlink_CheckBuffer => Zarlink_CheckBuffer,

		-- Start Counting Process
		RestartClock => RestartClock,
		
		-- The results of the counting process 
		ClockEvent => ClockEvent,
		
		-- The usual ports 
		Reset => Reset,
		Clk => Clk
	);

-- TEST PACKET GENERATOR -- 

MyTestClock : entity work.TestClock
    Port MAP ( 
		-- Event
		EventPort => TestClockEvent,
		
		-- The Sample Rate 
		Implant_SampleRate => Implant_SampleRate,
		
		The10thOf1MS => Zarlink_Testgenerator_The10thOf1MS,
		
		-- Usual Ports 
		Reset => Reset,
		Clk => Clk
	 );

-- Only transmit the Test Clock Events when the Test Packetgernator is active 
TestPacketGen_ActivatedClock <= (TestClockEvent AND TestPacketgenerator);

MyTestPacketGenerator : entity work.TestPacketGenerator 
    Port MAP ( 

		ClockEvent => TestPacketGen_ActivatedClock,
		
		Implant_ChannelMask => Implant_ChannelMask_Resorted,
		Implant_ADCResolution => Implant_ADCResolution,
		Implant_SampleRate => Implant_SampleRate,
		Implant_Filter => Implant_Filter,
		Implant_DeviceID => Implant_DeviceID,
		
		Data_Out => TestPacketGen_Data_Out,
		
		Valid_Out => TestPacketGen_Valid_Out,
		
		TriggerChannelsIn => TriggerChannelsIn,
		The10KHzCounter => The10KHzCounter,

		Clk => Clk,
		Reset => Reset
	);

-- This module counts the number of active channels in the channel mask...
MyChannelMaskCount : entity work.ChannelMaskCount 
	 Generic MAP
	 (
		MaskLength => 127,
		CountLength => 7
	 )
    Port MAP ( 
	 
		Count	=> ChannelMask_Count,
		Ready	=> ChannelMask_Ready,
		
		Implant_ChannelMask => Implant_ChannelMask,
				
		Clk => Clk,
		Reset => Reset
	 );

-- This module counts the number of active channels in the external adc mask...
MyExternalMaskCount : entity work.ChannelMaskCount 
	 Generic MAP
	 (
		MaskLength => 31,
		CountLength => 15
	 )
    Port MAP ( 
	 
		Count	=> ExternalMask_Count,
		Ready	=> ExternalMask_Ready,
		
		Implant_ChannelMask => SecondaryChannelMask,
				
		Clk => Clk,
		Reset => Reset
	 );

-- FROM 125 MHz to 120.833 MHz
	MyClockSlowDownFrom125MHzTo120_833MHz: entity work.ClockSlowDownFrom125MHzTo120UB
   port map ( 
			 CLKIN_IN        => Clk125MHz, 
          RST_IN          => Reset, 
          CLKFX_OUT       => Clk120_8333MHz, 
          CLKIN_IBUFG_OUT => open, 
          CLK0_OUT        => open, 
          LOCKED_OUT      => open
	);
	
-- FROM 120.833 MHz to 1.024 MHz 	
	My_1024KHz_Clock : process (Clk120_8333MHz,Reset)
	begin
		if Reset = '1' then
			Clk1024KHz_Buffer <= '0';
			Clk1024KHz_Counter <= (others=>'0');
			
		elsif rising_edge(Clk120_8333MHz) then
			if Clk1024KHz_Counter < Clk1024KHz_Interval then
				Clk1024KHz_Buffer <= Clk1024KHz_Buffer;
				Clk1024KHz_Counter <= Clk1024KHz_Counter + 1;
			else 
				Clk1024KHz_Buffer <= NOT Clk1024KHz_Buffer;
				Clk1024KHz_Counter <= (others => '0');
			end if;
			
		end if;
	
	end process My_1024KHz_Clock;
Clk1024KHz <= Clk1024KHz_Buffer;


	MyTiggerOut_MuxControl : entity work.TiggerOut_MuxControl
    Port MAP( 
	 
				TriggerOut_Address => TriggerOut_Address,
				TriggerOut_Data => TriggerOut_Data,
				TriggerOut_Clk	=> TriggerOut_Clk,
				
				TriggerChannelsOut => TriggerChannelsOut,

				HalfCycle => HalfCycle_TriggerOut,
				
				Clk => Clk,
				Reset => Reset
		);

  MyTiggerIn_MuxControl : entity work.TiggerIn_MuxControl
    Port MAP ( 

				TriggerIn_Address => TriggerIn_Address,
				TriggerIn_Data => TriggerIn_Data,
	 
				Wait_TriggerIn => Wait_TriggerIn,
				TriggerChannelsIn => TriggerChannelsIn,
				Clk => Clk,
				Reset => Reset
	);



Implant_ChannelMask_Resorted <= 
	  Implant_ChannelMask(127) & Implant_ChannelMask(111) & Implant_ChannelMask(95) & Implant_ChannelMask(79)
	& Implant_ChannelMask(63) & Implant_ChannelMask(47) & Implant_ChannelMask(31) & Implant_ChannelMask(15)
	& Implant_ChannelMask(126) & Implant_ChannelMask(110) & Implant_ChannelMask(94) & Implant_ChannelMask(78)
	& Implant_ChannelMask(62) & Implant_ChannelMask(46) & Implant_ChannelMask(30) & Implant_ChannelMask(14)
	& Implant_ChannelMask(125) & Implant_ChannelMask(109) & Implant_ChannelMask(93) & Implant_ChannelMask(77)
	& Implant_ChannelMask(61) & Implant_ChannelMask(45) & Implant_ChannelMask(29) & Implant_ChannelMask(13)
	& Implant_ChannelMask(124) & Implant_ChannelMask(108) & Implant_ChannelMask(92) & Implant_ChannelMask(76)
	& Implant_ChannelMask(60) & Implant_ChannelMask(44) & Implant_ChannelMask(28) & Implant_ChannelMask(12)
	& Implant_ChannelMask(123) & Implant_ChannelMask(107) & Implant_ChannelMask(91) & Implant_ChannelMask(75)
	& Implant_ChannelMask(59) & Implant_ChannelMask(43) & Implant_ChannelMask(27) & Implant_ChannelMask(11)
	& Implant_ChannelMask(122) & Implant_ChannelMask(106) & Implant_ChannelMask(90) & Implant_ChannelMask(74)
	& Implant_ChannelMask(58) & Implant_ChannelMask(42) & Implant_ChannelMask(26) & Implant_ChannelMask(10)
	& Implant_ChannelMask(121) & Implant_ChannelMask(105) & Implant_ChannelMask(89) & Implant_ChannelMask(73)
	& Implant_ChannelMask(57) & Implant_ChannelMask(41) & Implant_ChannelMask(25) & Implant_ChannelMask(9)
	& Implant_ChannelMask(120) & Implant_ChannelMask(104) & Implant_ChannelMask(88) & Implant_ChannelMask(72)
	& Implant_ChannelMask(56) & Implant_ChannelMask(40) & Implant_ChannelMask(24) & Implant_ChannelMask(8)
	& Implant_ChannelMask(119) & Implant_ChannelMask(103) & Implant_ChannelMask(87) & Implant_ChannelMask(71)
	& Implant_ChannelMask(55) & Implant_ChannelMask(39) & Implant_ChannelMask(23) & Implant_ChannelMask(7)
	& Implant_ChannelMask(118) & Implant_ChannelMask(102) & Implant_ChannelMask(86) & Implant_ChannelMask(70)
	& Implant_ChannelMask(54) & Implant_ChannelMask(38) & Implant_ChannelMask(22) & Implant_ChannelMask(6)
	& Implant_ChannelMask(117) & Implant_ChannelMask(101) & Implant_ChannelMask(85) & Implant_ChannelMask(69)
	& Implant_ChannelMask(53) & Implant_ChannelMask(37) & Implant_ChannelMask(21) & Implant_ChannelMask(5)
	& Implant_ChannelMask(116) & Implant_ChannelMask(100) & Implant_ChannelMask(84) & Implant_ChannelMask(68)
	& Implant_ChannelMask(52) & Implant_ChannelMask(36) & Implant_ChannelMask(20) & Implant_ChannelMask(4)
	& Implant_ChannelMask(115) & Implant_ChannelMask(99) & Implant_ChannelMask(83) & Implant_ChannelMask(67)
	& Implant_ChannelMask(51) & Implant_ChannelMask(35) & Implant_ChannelMask(19) & Implant_ChannelMask(3)
	& Implant_ChannelMask(114) & Implant_ChannelMask(98) & Implant_ChannelMask(82) & Implant_ChannelMask(66)
	& Implant_ChannelMask(50) & Implant_ChannelMask(34) & Implant_ChannelMask(18) & Implant_ChannelMask(2)
	& Implant_ChannelMask(113) & Implant_ChannelMask(97) & Implant_ChannelMask(81) & Implant_ChannelMask(65)
	& Implant_ChannelMask(49) & Implant_ChannelMask(33) & Implant_ChannelMask(17) & Implant_ChannelMask(1)
	& Implant_ChannelMask(112) & Implant_ChannelMask(96) & Implant_ChannelMask(80) & Implant_ChannelMask(64)
	& Implant_ChannelMask(48) & Implant_ChannelMask(32) & Implant_ChannelMask(16) & Implant_ChannelMask(0);

end Behavioral;

