library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;

-- Core for creating a SPI interface to a Zarlink chip ZL70102 
-- 7 Bit Addresses ; 1 Byte and 1 Block transfer is implemented. 

-- Note: Adding 8 bit addresses is simple and prepared because the third byte is already included.
-- 		Two new transfer types are required and NumberOfBits has to be set to 24.  

entity SimpleSPICore is
    Port ( 
		-- ZARLINK SPI - PORTS
		SPI_CS_B 					: out STD_LOGIC; 										-- SPI Enable
		SPI_SDI 						: out STD_LOGIC; 										-- SPI Data input into the Zarlink 
		SPI_SDO						: in STD_LOGIC; 										-- SPI Data from the Zarlink
		SPI_CLK						: out STD_LOGIC; 										-- SPI Clock (4MHz)

		
		-- Input
		INPUT_FIFO_DATAIN			: in STD_LOGIC_VECTOR(29 downto 0);				-- Data from SPI Input Fifo
		INPUT_FIFO_DATAIN_VALID	: in STD_LOGIC;										-- Data from SPI Input Fifo is valid (=1)
		INPUT_FIFO_READENABLE 	: out STD_LOGIC;										-- SPI Input Fifo Read-Enable
		INPUT_FIFO_EMPTY 			: in STD_LOGIC;										-- SPI Input Fifo is empty (=1)

		-- Output 
		OUTPUT_FIFO_WRITEENABLE	: out STD_LOGIC;										-- Data from SPI Interface Core to Output Fifo is valid (=1)
		OUTPUT_FIFO_DATAOUT		: out STD_LOGIC_VECTOR(15 downto 0);			-- Data from SPI Interface Core to Output Fifo
		OUTPUT_FIFO_1000BLOCKTOFULLL : in STD_LOGIC;									-- Check if LESS then 1000 16bit - entries are possible (=1)

		Reset							: in STD_LOGIC;										-- 8 MHz Clock
		CoreClock 					: in STD_LOGIC											-- 125 MHz (or >> 8MHz) for the external fifo
	 );
end entity SimpleSPICore;

architecture Behavioral of SimpleSPICore is

	-- Some Notes about the Transfer Types: 
	-- Calculation of the width: 
	-- 6bit(Typ) + 24(max cycle length in 8Bit-Adressmode)
	-- DatenTypen: 
	-- 7bit-adress action only
	
	-- "000000" => Write (16 cycles; on & Adress & Data & off)
	-- or (depending on the first bit)
	-- "000000" => Read (16 cycles ; on & Adress & Data & off)
	
	-- "000001" => Write Block Data - Start (16 cyles ; on & Adress & Data)
	-- or (depending on the first bit)
	-- "000001" => Read Block Data - Start (16 cyles ; on & Adress & Data)
	
	-- "000010" => Write Block Data - Intermediate (8 cyles ; Data )
	-- or (depending on the first bit)
	-- "000010" => Read Block Data - Intermediate (8 cyles ; Data)
	
	-- "000011" => Write Block Data - End (8 cyles ; Data & off)
	-- or (depending on the first bit)
	-- "000011" => Read Block Data - End (8 cyles ; Data & off)
	
	-- The states of the SPI state machine
	type STATE_Type is (
		-- State after the reset 
		SPI_INIT,
		-- State where we wait for new data 
		SPI_IDLE,
		-- Start the SPI transfer
--		SPI_START (second&third start state for holding the CS down for a bit longer before begining with the rite),
		SPI_START_A,
		SPI_START_B,
		SPI_START_C,
		-- Handling the SPI Bit in the SPI_CLK = 0 phase
		SPI_BITHANLDE_1,
		-- Handling the SPI Bit in the SPI_CLK = 1 phase
		SPI_BITHANLDE_2,
		-- Ending the SPI transfer
		SPI_END,
		-- Cool down states for holding SPI_CS_B = 1 for > 250 ns (2x 125 = 375ns) 
		-- If neccessary, one of the cooldown states can (possibly) be removed 
		SPI_COOLDOWN_1,
		SPI_COOLDOWN_2,
		SPI_COOLDOWN_3
	);
	signal SPIMACHINE_STATE : STATE_TYPE;
	
	-- Bits for the SPI exchange 
	-- -> SPI
	signal BitToExport : std_logic := '0';
	
	-- Will the SPI process produce something for the output buffer? 
	signal SPI_WillProduceOutput : STD_LOGIC := '0';
	-- What Type of Transmission is this? 
	signal SPI_TransmitType : STD_LOGIC_VECTOR(5 downto 0) := (others => '0');
	-- Will the SPI process end directly by the SPI_BITHANLDE_1 -> SPI_END state (=1)
	-- or is this a block transfer (=0)?
	signal SPI_WillEnd : STD_LOGIC := '0';
	-- How many bits do we have to transfer?
	signal NumberOfBits : unsigned(5 downto 0) := (others => '0');
	-- Here we collect the Output byte from SPO. 
	signal CollectedOutput : std_logic_vector(7 downto 0) := (others => '0');
	-- Here we store the bit stream that will be send to SPI. 
	signal DataStringToSend : std_logic_vector(23 downto 0) := (others => '0');	
	-- Buffer for the INPUT_FIFO_READENABLE port
	signal INPUT_FIFO_READENABLE_Buffer : STD_LOGIC := '0';
	-- We need a copy of the 8th bit of the address (rd/wr transfert) for the block mode
	signal ReadMode : std_logic := '0';
	
	signal SPI_SDO_Buffer : std_logic := '0';
	signal SPI_CS_B_Buffer : std_logic := '0';
	signal SPI_SDI_Buffer : std_logic := '0';
	signal SPI_CLK_Buffer : std_logic := '0';
	
begin

SPI_SDO_Buffer <= SPI_SDO;
SPI_CS_B <= SPI_CS_B_Buffer;
SPI_SDI <= SPI_SDI_Buffer;
SPI_CLK <= SPI_CLK_Buffer;


-- We need a copy of the READENABLE bit thus a buffered copy
INPUT_FIFO_READENABLE <= INPUT_FIFO_READENABLE_Buffer;

-- Obviously the state machine. 
-- It is debatable if this is a good StateMachine design, 
-- because the logic and clocked components are mixed. 
-- Nevertheless it works... 
-- Problems due to using rising_edge and falling_edge were dodged by using a 8MHz
-- instead of a 4MHz clock.   
SPIStateMachineStates : process (CoreClock,Reset)
	begin
		if (Reset = '1') then
			ReadMode <= '0';
			SPIMACHINE_STATE <= SPI_INIT;
			SPI_WillProduceOutput <= '0';
			SPI_TransmitType <= (others => '0');
			SPI_WillEnd <= '0';
			NumberOfBits <= (others => '0');
			CollectedOutput <= (others => '0');
			DataStringToSend <= (others => '0');	
			INPUT_FIFO_READENABLE_Buffer <= '0';
			OUTPUT_FIFO_WRITEENABLE <= '0';
			OUTPUT_FIFO_DATAOUT <= (others => '0');
			BitToExport <= '0';
			SPI_CS_B_Buffer <= '1';
			SPI_CLK_Buffer <= '0';
			SPI_SDI_Buffer <= '0';

		elsif rising_edge(CoreClock) then
 			
			case SPIMACHINE_STATE is
				-- Right after the reset... 
				when SPI_INIT =>
					ReadMode <= '0';
					SPIMACHINE_STATE <= SPI_IDLE;
					SPI_WillProduceOutput <= '0';
					SPI_TransmitType <= (others => '0');
					SPI_WillEnd <= '0';
					NumberOfBits <= (others => '0');
					CollectedOutput <= (others => '0');
					DataStringToSend <= (others => '0');	
					INPUT_FIFO_READENABLE_Buffer <= '0';
					OUTPUT_FIFO_WRITEENABLE <= '0';
					OUTPUT_FIFO_DATAOUT <= (others => '0');
					BitToExport <= '0';
					SPI_CS_B_Buffer <= '1';
					SPI_CLK_Buffer <= '0';
					SPI_SDI_Buffer <= '0';					

				-- Wait for new data... 
				-- If there is something new, then copy and request new data
				-- Copy means:
				-- 0-5: TransmitType
				-- 6-13 Bit1 (Address+ReadWrite Flag) (in wrong order)
				-- 14-21 Bit2 (in wrong order)
				-- 22-29 Bit3 (in wrong order)
				when SPI_IDLE =>
					SPI_CS_B_Buffer <= '1';
					SPI_CLK_Buffer <= '0';
					SPI_SDI_Buffer <= '0';	
					-- if we find here Types of "000010" or "000011" something went wrong. 
					-- Because: A block transfer is not allowed to start in the middle!
					-- We need to filter this junk out. Because the SPI_CS_B was set to '1'
					--	which made this data worthless... 
					if (INPUT_FIFO_DATAIN_VALID = '1') AND (INPUT_FIFO_EMPTY = '0') AND (INPUT_FIFO_DATAIN(13) = '1') AND (OUTPUT_FIFO_1000BLOCKTOFULLL = '0') then
						ReadMode <= '1';
						--if there is new data (export and import + enought space for the results)
						
						SPI_WillProduceOutput <= '1';
						case INPUT_FIFO_DATAIN(5 downto 0) is
						 -- Write 8 & Read 8 & End
							when "000000" | "000100" | "001000" | "001001" | "001010" =>
								SPI_WillEnd <= '1';
								NumberOfBits <= to_unsigned(16,6);
								SPIMACHINE_STATE <= SPI_START_A;--SPI_START;
								SPI_TransmitType <= INPUT_FIFO_DATAIN(5 downto 0);							
								DataStringToSend(23 downto 8) <= (others => '0');	
								DataStringToSend(0) <= INPUT_FIFO_DATAIN(13);
								DataStringToSend(1) <= INPUT_FIFO_DATAIN(12);
								DataStringToSend(2) <= INPUT_FIFO_DATAIN(11);
								DataStringToSend(3) <= INPUT_FIFO_DATAIN(10);
								DataStringToSend(4) <= INPUT_FIFO_DATAIN(9);
								DataStringToSend(5) <= INPUT_FIFO_DATAIN(8);
								DataStringToSend(6) <= INPUT_FIFO_DATAIN(7);
								DataStringToSend(7) <= INPUT_FIFO_DATAIN(6);
								-- Write 8 & Read 8 & No End
							when "000001" | "000101" =>
								SPI_WillEnd <= '0';
								NumberOfBits <= to_unsigned(16,6);
								SPIMACHINE_STATE <= SPI_START_A;--SPI_START;
								SPI_TransmitType <= INPUT_FIFO_DATAIN(5 downto 0);							
								DataStringToSend(23 downto 8) <= (others => '0');	
								DataStringToSend(0) <= INPUT_FIFO_DATAIN(13);
								DataStringToSend(1) <= INPUT_FIFO_DATAIN(12);
								DataStringToSend(2) <= INPUT_FIFO_DATAIN(11);
								DataStringToSend(3) <= INPUT_FIFO_DATAIN(10);
								DataStringToSend(4) <= INPUT_FIFO_DATAIN(9);
								DataStringToSend(5) <= INPUT_FIFO_DATAIN(8);
								DataStringToSend(6) <= INPUT_FIFO_DATAIN(7);
								DataStringToSend(7) <= INPUT_FIFO_DATAIN(6);
							-- if there is no known datatype
							when others =>
								SPIMACHINE_STATE <= SPI_IDLE;
								SPI_TransmitType <= (others => '0');
								DataStringToSend <= (others => '0');
								SPI_WillProduceOutput <= '0';
								SPI_WillEnd <= '0';
								NumberOfBits <= (others => '0');
						end case;
						INPUT_FIFO_READENABLE_Buffer <= '1';
					elsif (INPUT_FIFO_DATAIN_VALID = '1') AND (INPUT_FIFO_EMPTY = '0') AND (INPUT_FIFO_DATAIN(13) = '0') then
						ReadMode <= '0';
						--if there is new data (export only) 
						SPI_WillProduceOutput <= '0';
						case INPUT_FIFO_DATAIN(5 downto 0) is
						 -- Write 16 & Read 0 & End
							when "000000" | "000100" | "001000" | "001001" | "001010" =>
								SPI_WillEnd <= '1';
								NumberOfBits <= to_unsigned(16,6);
								SPIMACHINE_STATE <= SPI_START_A;--SPI_START;
								SPI_TransmitType <= INPUT_FIFO_DATAIN(5 downto 0);							
								DataStringToSend(23 downto 15) <= (others => '0');	
								DataStringToSend(0) <= INPUT_FIFO_DATAIN(13);
								DataStringToSend(1) <= INPUT_FIFO_DATAIN(12);
								DataStringToSend(2) <= INPUT_FIFO_DATAIN(11);
								DataStringToSend(3) <= INPUT_FIFO_DATAIN(10);
								DataStringToSend(4) <= INPUT_FIFO_DATAIN(9);
								DataStringToSend(5) <= INPUT_FIFO_DATAIN(8);
								DataStringToSend(6) <= INPUT_FIFO_DATAIN(7);
								DataStringToSend(7) <= INPUT_FIFO_DATAIN(6);

								DataStringToSend(8) <= INPUT_FIFO_DATAIN(21);
								DataStringToSend(9) <= INPUT_FIFO_DATAIN(20);
								DataStringToSend(10) <= INPUT_FIFO_DATAIN(19);
								DataStringToSend(11) <= INPUT_FIFO_DATAIN(18);
								DataStringToSend(12) <= INPUT_FIFO_DATAIN(17);
								DataStringToSend(13) <= INPUT_FIFO_DATAIN(16);
								DataStringToSend(14) <= INPUT_FIFO_DATAIN(15);
								DataStringToSend(15) <= INPUT_FIFO_DATAIN(14);
								
						 -- Write 16 & Read 0 & No End
							when "000001" | "000101" =>
								SPI_WillEnd <= '0';
								NumberOfBits <= to_unsigned(16,6);
								SPIMACHINE_STATE <= SPI_START_A;--SPI_START;
								SPI_TransmitType <= INPUT_FIFO_DATAIN(5 downto 0);							
								DataStringToSend(23 downto 15) <= (others => '0');	
								DataStringToSend(0) <= INPUT_FIFO_DATAIN(13);
								DataStringToSend(1) <= INPUT_FIFO_DATAIN(12);
								DataStringToSend(2) <= INPUT_FIFO_DATAIN(11);
								DataStringToSend(3) <= INPUT_FIFO_DATAIN(10);
								DataStringToSend(4) <= INPUT_FIFO_DATAIN(9);
								DataStringToSend(5) <= INPUT_FIFO_DATAIN(8);
								DataStringToSend(6) <= INPUT_FIFO_DATAIN(7);
								DataStringToSend(7) <= INPUT_FIFO_DATAIN(6);

								DataStringToSend(8) <= INPUT_FIFO_DATAIN(21);
								DataStringToSend(9) <= INPUT_FIFO_DATAIN(20);
								DataStringToSend(10) <= INPUT_FIFO_DATAIN(19);
								DataStringToSend(11) <= INPUT_FIFO_DATAIN(18);
								DataStringToSend(12) <= INPUT_FIFO_DATAIN(17);
								DataStringToSend(13) <= INPUT_FIFO_DATAIN(16);
								DataStringToSend(14) <= INPUT_FIFO_DATAIN(15);
								DataStringToSend(15) <= INPUT_FIFO_DATAIN(14);
							-- if there is no known datatype
							when others =>
								SPIMACHINE_STATE <= SPI_IDLE;
								SPI_TransmitType <= (others => '0');
								DataStringToSend <= (others => '0');
								SPI_WillProduceOutput <= '0';
								SPI_WillEnd <= '0';
								NumberOfBits <= (others => '0');
						end case;
						INPUT_FIFO_READENABLE_Buffer <= '1';
					else
						-- if there is no new data
						ReadMode <= '0';
						SPIMACHINE_STATE <= SPI_IDLE;
						SPI_TransmitType <= (others => '0');
						DataStringToSend <= (others => '0');
						SPI_WillProduceOutput <= '0';
						SPI_WillEnd <= '0';
						NumberOfBits <= (others => '0');
--						if (INPUT_FIFO_DATAIN_VALID = '0') AND (INPUT_FIFO_READENABLE_Buffer = '0') then
--							INPUT_FIFO_READENABLE_Buffer <= '1';
--						else
							INPUT_FIFO_READENABLE_Buffer <= '0';
--						end if;
					end if;
					
					CollectedOutput <= (others => '0');
						
					OUTPUT_FIFO_WRITEENABLE <= '0';
					OUTPUT_FIFO_DATAOUT <= (others => '0');
					BitToExport <= '0';
					
				-- And here we go...
				when SPI_START_A =>

					SPIMACHINE_STATE <= SPI_START_B; --SPI_BITHANLDE_1;
					SPI_CS_B_Buffer <= '0';
					SPI_CLK_Buffer <= '0';
					SPI_SDI_Buffer <= '0';
					SPI_WillProduceOutput <= SPI_WillProduceOutput;
					SPI_TransmitType <= SPI_TransmitType;
					SPI_WillEnd <= SPI_WillEnd;
					NumberOfBits <= NumberOfBits;
					CollectedOutput <= (others => '0');
					DataStringToSend <= DataStringToSend;	
					INPUT_FIFO_READENABLE_Buffer <= '0';
					OUTPUT_FIFO_WRITEENABLE <= '0';
					OUTPUT_FIFO_DATAOUT <= (others => '0');
					BitToExport <= '0';
					ReadMode <= ReadMode;
				when SPI_START_B =>

					SPIMACHINE_STATE <= SPI_START_C;
					SPI_CS_B_Buffer <= '0';
					SPI_CLK_Buffer <= '0';
					SPI_SDI_Buffer <= '0';
					SPI_WillProduceOutput <= SPI_WillProduceOutput;
					SPI_TransmitType <= SPI_TransmitType;
					SPI_WillEnd <= SPI_WillEnd;
					NumberOfBits <= NumberOfBits;
					CollectedOutput <= (others => '0');
					DataStringToSend <= DataStringToSend;	
					INPUT_FIFO_READENABLE_Buffer <= '0';
					OUTPUT_FIFO_WRITEENABLE <= '0';
					OUTPUT_FIFO_DATAOUT <= (others => '0');
					BitToExport <= '0';
					ReadMode <= ReadMode;

				when SPI_START_C =>

					SPIMACHINE_STATE <= SPI_BITHANLDE_1;
					SPI_CS_B_Buffer <= '0';
					SPI_CLK_Buffer <= '0';
					SPI_SDI_Buffer <= '0';
					SPI_WillProduceOutput <= SPI_WillProduceOutput;
					SPI_TransmitType <= SPI_TransmitType;
					SPI_WillEnd <= SPI_WillEnd;
					NumberOfBits <= NumberOfBits;
					CollectedOutput <= (others => '0');
					DataStringToSend <= DataStringToSend;	
					INPUT_FIFO_READENABLE_Buffer <= '0';
					OUTPUT_FIFO_WRITEENABLE <= '0';
					OUTPUT_FIFO_DATAOUT <= (others => '0');
					BitToExport <= '0';
					ReadMode <= ReadMode;
				-- SPI Phase 0: Set the values
				when SPI_BITHANLDE_1 =>
					ReadMode <= ReadMode;
					SPI_CS_B_Buffer <= '0';
					SPI_CLK_Buffer <= '0';
					if NumberOfBits = to_unsigned(0,6) then
						SPIMACHINE_STATE <= SPI_END;
					else
						SPIMACHINE_STATE <= SPI_BITHANLDE_2;
					end if;
				
					-- Get next BitToExport from the 30 bit vector by shifting
					DataStringToSend <= "0" & DataStringToSend(23 downto 1);	
					SPI_SDI_Buffer <= DataStringToSend(0);
					BitToExport <= DataStringToSend(0);
					SPI_WillProduceOutput <= SPI_WillProduceOutput;
					SPI_TransmitType <= SPI_TransmitType;
					SPI_WillEnd <= SPI_WillEnd;
					NumberOfBits <= NumberOfBits;
					CollectedOutput <= CollectedOutput;
					
					INPUT_FIFO_READENABLE_Buffer <= '0';
					OUTPUT_FIFO_WRITEENABLE <= '0';
					OUTPUT_FIFO_DATAOUT <= (others => '0');

				
				-- SPI Phase 1: Get the value
				when SPI_BITHANLDE_2 =>
					ReadMode <= ReadMode;
					SPI_CS_B_Buffer <= '0';
					SPI_CLK_Buffer <= '1';
					SPI_SDI_Buffer <= BitToExport;				
					
--					if (NumberOfBits = to_unsigned(1,6)) AND (SPI_WillProduceOutput = '1') then
					if (NumberOfBits = to_unsigned(1,6)) then
						-- if lastbit AND something to write: Export to fifo and reset 16 bit output vector
						-- Append Bit to 16 bit output value & export plus reset
						CollectedOutput <= (others => '0');
						OUTPUT_FIFO_WRITEENABLE <= '1';

						if (SPI_WillProduceOutput = '1') then
							OUTPUT_FIFO_DATAOUT <= CollectedOutput(6 downto 0) & SPI_SDO & SPI_TransmitType & "00";
						else
							OUTPUT_FIFO_DATAOUT <= CollectedOutput(6 downto 0) & "1" & SPI_TransmitType & "00";
						end if;
					else
						-- Append Bit to 16 bit output value
						if (SPI_WillProduceOutput = '1') then
							CollectedOutput <= CollectedOutput(6 downto 0) & SPI_SDO;
						else
							CollectedOutput <= CollectedOutput(6 downto 0) & "1";
						end if;
						
						OUTPUT_FIFO_WRITEENABLE <= '0';
						OUTPUT_FIFO_DATAOUT <= (others => '0');
					end if;
					
					BitToExport <= BitToExport;
					
					-- This part deals with block transfers. To be precise the 
					-- middle and the end part of a block transfer. 
					-- Everthing else is a problem and has to be purified.
					-- Means: Kill'em and go back to the INIT-state (link a reset)
					if (NumberOfBits = to_unsigned(1,6)) AND (SPI_WillEnd = '0') then
						-- if ("000001" | "000101" | "000010") AND lastbit load next 30bit vector
						if (INPUT_FIFO_DATAIN_VALID = '1') AND (ReadMode = '1') then
							SPI_WillProduceOutput <= '1';
							case INPUT_FIFO_DATAIN(5 downto 0) is
								-- Write 0 & Read 8 & No End
								when "000010" | "000110"=>
									SPI_WillEnd <= '0';
									NumberOfBits <= to_unsigned(8,6);
									SPIMACHINE_STATE <= SPI_BITHANLDE_1;
									SPI_TransmitType <= INPUT_FIFO_DATAIN(5 downto 0);							
									DataStringToSend(23 downto 0) <= (others => '0');	
									
								-- Write 0 & Read 8 & End
								when "000011" | "000111"=>
									SPI_WillEnd <= '1';
									NumberOfBits <= to_unsigned(8,6);
									SPIMACHINE_STATE <= SPI_BITHANLDE_1;
									SPI_TransmitType <= INPUT_FIFO_DATAIN(5 downto 0);							
									DataStringToSend(23 downto 0) <= (others => '0');	
								-- if there is no known datatype
								when others =>
									SPIMACHINE_STATE <= SPI_INIT;
									SPI_TransmitType <= (others => '0');
									DataStringToSend <= (others => '0');
									SPI_WillProduceOutput <= '0';
									SPI_WillEnd <= '0';
									NumberOfBits <= (others => '0');
								end case;
								INPUT_FIFO_READENABLE_Buffer <= '1';

							elsif (INPUT_FIFO_DATAIN_VALID = '1') AND (ReadMode = '0') then
								SPI_WillProduceOutput <= '0';
								case INPUT_FIFO_DATAIN(5 downto 0) is							
									-- Write 8 & Read 0 & No End
									when "000010" | "000110" =>
										SPI_WillEnd <= '0';
										NumberOfBits <= to_unsigned(8,6);
										SPIMACHINE_STATE <= SPI_BITHANLDE_1;
										SPI_TransmitType <= INPUT_FIFO_DATAIN(5 downto 0);							
										DataStringToSend(23 downto 8) <= (others => '0');	
										DataStringToSend(0) <= INPUT_FIFO_DATAIN(13);
										DataStringToSend(1) <= INPUT_FIFO_DATAIN(12);
										DataStringToSend(2) <= INPUT_FIFO_DATAIN(11);
										DataStringToSend(3) <= INPUT_FIFO_DATAIN(10);
										DataStringToSend(4) <= INPUT_FIFO_DATAIN(9);
										DataStringToSend(5) <= INPUT_FIFO_DATAIN(8);
										DataStringToSend(6) <= INPUT_FIFO_DATAIN(7);
										DataStringToSend(7) <= INPUT_FIFO_DATAIN(6);

									-- Write 8 & Read 0 & End
									when "000011" | "000111"=>
										SPI_WillEnd <= '1';
										NumberOfBits <= to_unsigned(8,6);
										SPIMACHINE_STATE <= SPI_BITHANLDE_1;
										SPI_TransmitType <= INPUT_FIFO_DATAIN(5 downto 0);							
										DataStringToSend(23 downto 8) <= (others => '0');	
										DataStringToSend(0) <= INPUT_FIFO_DATAIN(13);
										DataStringToSend(1) <= INPUT_FIFO_DATAIN(12);
										DataStringToSend(2) <= INPUT_FIFO_DATAIN(11);
										DataStringToSend(3) <= INPUT_FIFO_DATAIN(10);
										DataStringToSend(4) <= INPUT_FIFO_DATAIN(9);
										DataStringToSend(5) <= INPUT_FIFO_DATAIN(8);
										DataStringToSend(6) <= INPUT_FIFO_DATAIN(7);
										DataStringToSend(7) <= INPUT_FIFO_DATAIN(6);

									-- if there is no known datatype
									when others =>
										SPIMACHINE_STATE <= SPI_INIT;
										SPI_TransmitType <= (others => '0');
										DataStringToSend <= (others => '0');
										SPI_WillProduceOutput <= '0';
										SPI_WillEnd <= '0';
										NumberOfBits <= (others => '0');
									end case;
									INPUT_FIFO_READENABLE_Buffer <= '1';
									
								else
									-- if there is no new data
									SPIMACHINE_STATE <= SPI_INIT;
									SPI_TransmitType <= (others => '0');
									DataStringToSend <= (others => '0');
									SPI_WillProduceOutput <= '0';
									SPI_WillEnd <= '0';
									NumberOfBits <= (others => '0');
									INPUT_FIFO_READENABLE_Buffer <= '0';

								end if;								
					else

						if NumberOfBits > to_unsigned(0,6) then
							NumberOfBits <= NumberOfBits -1;
						else
							NumberOfBits <= NumberOfBits;
						end if;
						INPUT_FIFO_READENABLE_Buffer <= '0';
						SPIMACHINE_STATE <= SPI_BITHANLDE_1;
						SPI_WillProduceOutput <= SPI_WillProduceOutput;
						SPI_TransmitType <= SPI_TransmitType;
						SPI_WillEnd <= SPI_WillEnd;
						DataStringToSend <= DataStringToSend;	
					end if;
					
				-- Lets end this... 
				when SPI_END =>
					ReadMode <= ReadMode;
					SPI_CS_B_Buffer <= '0';
					SPI_CLK_Buffer <= '0';
					SPI_SDI_Buffer <= '0';				
					SPIMACHINE_STATE <= SPI_COOLDOWN_1;
					SPI_WillProduceOutput <= '0';
					SPI_TransmitType <= (others => '0');
					SPI_WillEnd <= '0';
					NumberOfBits <= (others => '0');
					CollectedOutput <= (others => '0');
					DataStringToSend <= (others => '0');	
					INPUT_FIFO_READENABLE_Buffer <= '0';
					OUTPUT_FIFO_WRITEENABLE <= '0';
					OUTPUT_FIFO_DATAOUT <= (others => '0');
					BitToExport <= '0';
					
				-- Cool down... 125 ns
				when SPI_COOLDOWN_1 =>
					ReadMode <= ReadMode;
					SPI_CS_B_Buffer <= '1';
					SPI_CLK_Buffer <= '0';
					SPI_SDI_Buffer <= '0';				
					SPIMACHINE_STATE <= SPI_COOLDOWN_2;
					SPI_WillProduceOutput <= '0';
					SPI_TransmitType <= (others => '0');
					SPI_WillEnd <= '0';
					NumberOfBits <= (others => '0');
					CollectedOutput <= (others => '0');
					DataStringToSend <= (others => '0');	
					INPUT_FIFO_READENABLE_Buffer <= '0';
					OUTPUT_FIFO_WRITEENABLE <= '0';
					OUTPUT_FIFO_DATAOUT <= (others => '0');
					BitToExport <= '0';
				
				-- Cool down... +125 ns
				when SPI_COOLDOWN_2 =>
					ReadMode <= ReadMode;
					SPI_CS_B_Buffer <= '1';
					SPI_CLK_Buffer <= '0';
					SPI_SDI_Buffer <= '0';
					SPIMACHINE_STATE <= SPI_COOLDOWN_3;
					SPI_WillProduceOutput <= '0';
					SPI_TransmitType <= (others => '0');
					SPI_WillEnd <= '0';
					NumberOfBits <= (others => '0');
					CollectedOutput <= (others => '0');
					DataStringToSend <= (others => '0');	
					INPUT_FIFO_READENABLE_Buffer <= '0';
					OUTPUT_FIFO_WRITEENABLE <= '0';
					OUTPUT_FIFO_DATAOUT <= (others => '0');
					BitToExport <= '0';
					
				-- Cool down... +125ns
				when SPI_COOLDOWN_3 =>
					ReadMode <= ReadMode;
					SPI_CS_B_Buffer <= '1';
					SPI_CLK_Buffer <= '0';
					SPI_SDI_Buffer <= '0';				
					SPIMACHINE_STATE <= SPI_IDLE;
					SPI_WillProduceOutput <= '0';
					SPI_TransmitType <= (others => '0');
					SPI_WillEnd <= '0';
					NumberOfBits <= (others => '0');
					CollectedOutput <= (others => '0');
					DataStringToSend <= (others => '0');	
					INPUT_FIFO_READENABLE_Buffer <= '0';
					OUTPUT_FIFO_WRITEENABLE <= '0';
					OUTPUT_FIFO_DATAOUT <= (others => '0');
					BitToExport <= '0';
					
				-- Something is totally wrong... Back to start and make a clean restart... 
				when others =>
					ReadMode <= '0';
					SPI_CS_B_Buffer <= '1';
					SPI_CLK_Buffer <= '0';
					SPI_SDI_Buffer <= '0';				
					SPIMACHINE_STATE <= SPI_IDLE;
					SPI_WillProduceOutput <= '0';
					SPI_TransmitType <= (others => '0');
					SPI_WillEnd <= '0';
					NumberOfBits <= (others => '0');
					CollectedOutput <= (others => '0');
					DataStringToSend <= (others => '0');	
					INPUT_FIFO_READENABLE_Buffer <= '0';
					OUTPUT_FIFO_WRITEENABLE <= '0';
					OUTPUT_FIFO_DATAOUT <= (others => '0');
					BitToExport <= '0';
				end case;
		end if;
	end process SPIStateMachineStates;

end architecture Behavioral;

