library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;
entity StreamDecomposer is
    Port ( 

		Resolution 			: in  STD_LOGIC_VECTOR(3 downto 0);
		
		-- This comes in... 
		Input_Data 			: in  STD_LOGIC_VECTOR(110 downto 0);
		Input_IsHeader		: in 	STD_LOGIC;
		Input_HeaderLong	: in	STD_LOGIC;
		Input_Valid			: in  STD_LOGIC;
		Input_RequestNext	: out	STD_LOGIC;
		
		-- This comes out...
		Output_Data			: out STD_LOGIC_VECTOR(15 downto 0);
		Output_Valid		: out STD_LOGIC;
		
		-- New Packet
		NewPacket			: in  STD_LOGIC;

		-- Usual Ports 
		Clk : in  STD_LOGIC;
      Reset : in  STD_LOGIC
	 );
end StreamDecomposer;

architecture Behavioral of StreamDecomposer is
	
	signal Buffer_Data : STD_LOGIC_VECTOR(110 downto 0);
	signal Buffer_Counter : unsigned (7 downto 0);
	
	signal Unused_Data : STD_LOGIC_VECTOR(15 downto 0);
	signal Unused_Counter : unsigned (7 downto 0);

	signal Output_Buffer_Rest : STD_LOGIC_VECTOR(15 downto 0);
	signal Output_Buffer_Counter : unsigned(7 downto 0);

	type STATE_TYPE is (
		-- Reset State 
		STATE_RESET,
		STATE_WAIT_FOR_DATA,
		STATE_PROCESS_DATA_WITHOUT_REST,
		STATE_PROCESS_DATA_WITH_REST,
		STATE_PROCESS_DATA_WITH_REST_B,
		STATE_PROCESS_DATA_WITH_REST_C
		);
	signal FSM_STATE : STATE_TYPE;
		
begin
	FSM : process(Reset,Clk)
	begin
		if (Reset = '1') then
			-- Default:
				Input_RequestNext <= '0';
				Buffer_Data <= (others => '0');
				Buffer_Counter <= (others => '0');
				Unused_Data <= (others => '0');
				Unused_Counter <= (others => '0');
				
				Output_Data <= (others => '0');
				Output_Valid <= '0';
				
				FSM_STATE <= STATE_RESET;
				
				Output_Buffer_Rest <= (others => '0');
				Output_Buffer_Counter <= (others => '0');
		elsif rising_edge(Clk) then
				-- Default:
				Input_RequestNext <= '0';
				Buffer_Data <= Buffer_Data;
				Buffer_Counter <= Buffer_Counter;
				Unused_Data <= Unused_Data;
				Unused_Counter <= Unused_Counter;

				Output_Data <= (others => '0');
				Output_Valid <= '0';
			
				Output_Buffer_Rest <= (others => '0');
				Output_Buffer_Counter <= (others => '0');
				
				case FSM_STATE is 
					when STATE_RESET =>
						FSM_STATE <= STATE_WAIT_FOR_DATA;
						Buffer_Data <= (others => '0');
						Buffer_Counter <= (others => '0');
						Unused_Data <= (others => '0');
						Unused_Counter <= (others => '0');						
					-- Wait for new data... 
					when STATE_WAIT_FOR_DATA =>
	
						-- There is a new packet 
						if NewPacket = '1' then
							Buffer_Data <= (others => '0');
							Buffer_Counter <= (others => '0');
							Unused_Data <= (others => '0');
							Unused_Counter <= (others => '0');
							FSM_STATE <= STATE_WAIT_FOR_DATA;
				
						-- Normal packet (contains no header components)
						elsif (Input_Valid = '1') AND (Input_IsHeader = '0') then
							Buffer_Data <= Input_Data;
							Buffer_Counter <= to_unsigned(111,8);
							Input_RequestNext <= '1';
							if Unused_Counter = "00000000" then
								FSM_STATE <= STATE_PROCESS_DATA_WITHOUT_REST;
							else
								FSM_STATE <= STATE_PROCESS_DATA_WITH_REST;
							end if;
			
						-- Packet with short header
						-- Ignore Type (4 bit), Device ID (8bit), Timestamp (16 bit) and 2xControll Bit = 30 BIT
						-- 111 - 30 = 81
						elsif (Input_Valid = '1') AND (Input_IsHeader = '1') AND (Input_HeaderLong = '0') then
							Buffer_Data <= (others => '0');
							Buffer_Data(80 downto 0) <= Input_Data(110 downto 30);
							Buffer_Counter <= to_unsigned(81,8);
							Input_RequestNext <= '1';
							Unused_Data <= (others => '0');
							Unused_Counter <= (others => '0');	
							FSM_STATE <= STATE_PROCESS_DATA_WITHOUT_REST;	
							
						-- Packet with long header
						-- Ignore full first packet 
						-- Ignore 63 bit of the second packet 
						-- Ignore Type (4 bit), Device ID (8bit), Channel Mask (128 bit), rha_filter (4Bit),
						-- timestamp (16Bit), Samplerate (8 Bit), resolution (4Bit) and 
						-- 2x Controllbit
						-- 4+8+128+4+16+8+4+1 = 174
						-- 174 - 111 = 63 Bit (Ignore first packet) 
						-- 111 - 63 = 48 Bit
						elsif (Input_Valid = '1') AND (Input_IsHeader = '1') AND (Input_HeaderLong = '1') then
							Buffer_Data <= (others => '0');
							Buffer_Data(47 downto 0) <= Input_Data(110 downto 63);
							Buffer_Counter <= to_unsigned(48,8);
							Input_RequestNext <= '1';
							Unused_Data <= (others => '0');
							Unused_Counter <= (others => '0');			
							FSM_STATE <= STATE_PROCESS_DATA_WITHOUT_REST;
							
						else
							Buffer_Data <= (others => '0');
							Buffer_Counter <= (others => '0');
							FSM_STATE <= STATE_WAIT_FOR_DATA;
							Input_RequestNext <= '1';
						end if;

					when STATE_PROCESS_DATA_WITHOUT_REST =>
						if NewPacket = '1' then
							Buffer_Data <= (others => '0');
							Buffer_Counter <= (others => '0');
							Unused_Data <= (others => '0');
							Unused_Counter <= (others => '0');
							FSM_STATE <= STATE_WAIT_FOR_DATA;
						else
							FSM_STATE <= STATE_PROCESS_DATA_WITHOUT_REST;
							case Resolution is
								when "0000" =>
									if Buffer_Counter >= to_unsigned(1,8) then
										Output_Data(0) <= Buffer_Data(0);
										Output_Valid <= '1';
										Buffer_Data <= "0" & Buffer_Data(110 downto 1);
										Buffer_Counter <= Buffer_Counter - 1;
									else
										Unused_Data <= Buffer_Data(15 downto 0);
										Unused_Counter <= Buffer_Counter;
										FSM_STATE <= STATE_WAIT_FOR_DATA;
									end if;

								when "0001" =>
									if Buffer_Counter >= to_unsigned(2,8) then
										Output_Data(1 downto 0) <= Buffer_Data(1 downto 0);
										Output_Valid <= '1';
										Buffer_Data <= "00" & Buffer_Data(110 downto 2);
										Buffer_Counter <= Buffer_Counter - 2;
									else
										Unused_Data <= Buffer_Data(15 downto 0);
										Unused_Counter <= Buffer_Counter;
										FSM_STATE <= STATE_WAIT_FOR_DATA;
									end if;
									
								when "0010" =>
									if Buffer_Counter >= to_unsigned(3,8) then
										Output_Data(2 downto 0) <= Buffer_Data(2 downto 0);
										Output_Valid <= '1';
										Buffer_Data <= "000" & Buffer_Data(110 downto 3);
										Buffer_Counter <= Buffer_Counter - 3;
									else
										Unused_Data <= Buffer_Data(15 downto 0);
										Unused_Counter <= Buffer_Counter;
										FSM_STATE <= STATE_WAIT_FOR_DATA;
									end if;

								when "0011" =>
									if Buffer_Counter >= to_unsigned(4,8) then
										Output_Data(3 downto 0) <= Buffer_Data(3 downto 0);
										Output_Valid <= '1';
										Buffer_Data <= "0000" & Buffer_Data(110 downto 4);
										Buffer_Counter <= Buffer_Counter - 4;
									else
										Unused_Data <= Buffer_Data(15 downto 0);
										Unused_Counter <= Buffer_Counter;
										FSM_STATE <= STATE_WAIT_FOR_DATA;
									end if;

								when "0100" =>
									if Buffer_Counter >= to_unsigned(5,8) then
										Output_Data(4 downto 0) <= Buffer_Data(4 downto 0);
										Output_Valid <= '1';
										Buffer_Data <= "00000" & Buffer_Data(110 downto 5);
										Buffer_Counter <= Buffer_Counter - 5;
									else
										Unused_Data <= Buffer_Data(15 downto 0);
										Unused_Counter <= Buffer_Counter;
										FSM_STATE <= STATE_WAIT_FOR_DATA;
									end if;

								when "0101" =>
									if Buffer_Counter >= to_unsigned(6,8) then
										Output_Data(5 downto 0) <= Buffer_Data(5 downto 0);
										Output_Valid <= '1';
										Buffer_Data <= "000000" & Buffer_Data(110 downto 6);
										Buffer_Counter <= Buffer_Counter - 6;
									else
										Unused_Data <= Buffer_Data(15 downto 0);
										Unused_Counter <= Buffer_Counter;
										FSM_STATE <= STATE_WAIT_FOR_DATA;
									end if;

								when "0110" =>
									if Buffer_Counter >= to_unsigned(7,8) then
										Output_Data(6 downto 0) <= Buffer_Data(6 downto 0);
										Output_Valid <= '1';
										Buffer_Data <= "0000000" & Buffer_Data(110 downto 7);
										Buffer_Counter <= Buffer_Counter - 7;
									else
										Unused_Data <= Buffer_Data(15 downto 0);
										Unused_Counter <= Buffer_Counter;
										FSM_STATE <= STATE_WAIT_FOR_DATA;
									end if;


								when "0111" =>
									if Buffer_Counter >= to_unsigned(8,8) then
										Output_Data(7 downto 0) <= Buffer_Data(7 downto 0);
										Output_Valid <= '1';
										Buffer_Data <= "00000000" & Buffer_Data(110 downto 8);
										Buffer_Counter <= Buffer_Counter - 8;
									else
										Unused_Data <= Buffer_Data(15 downto 0);
										Unused_Counter <= Buffer_Counter;
										FSM_STATE <= STATE_WAIT_FOR_DATA;
									end if;

								when "1000" =>
									if Buffer_Counter >= to_unsigned(9,8) then
										Output_Data(8 downto 0) <= Buffer_Data(8 downto 0);
										Output_Valid <= '1';
										Buffer_Data <= "000000000" & Buffer_Data(110 downto 9);
										Buffer_Counter <= Buffer_Counter - 9;
									else
										Unused_Data <= Buffer_Data(15 downto 0);
										Unused_Counter <= Buffer_Counter;
										FSM_STATE <= STATE_WAIT_FOR_DATA;
									end if;

								when "1001" =>
									if Buffer_Counter >= to_unsigned(10,8) then
										Output_Data(9 downto 0) <= Buffer_Data(9 downto 0);
										Output_Valid <= '1';
										Buffer_Data <= "0000000000" & Buffer_Data(110 downto 10);
										Buffer_Counter <= Buffer_Counter - 10;
									else
										Unused_Data <= Buffer_Data(15 downto 0);
										Unused_Counter <= Buffer_Counter;
										FSM_STATE <= STATE_WAIT_FOR_DATA;
									end if;

								when "1010" =>
									if Buffer_Counter >= to_unsigned(11,8) then
										Output_Data(10 downto 0) <= Buffer_Data(10 downto 0);
										Output_Valid <= '1';
										Buffer_Data <= "00000000000" & Buffer_Data(110 downto 11);
										Buffer_Counter <= Buffer_Counter - 11;
									else
										Unused_Data <= Buffer_Data(15 downto 0);
										Unused_Counter <= Buffer_Counter;
										FSM_STATE <= STATE_WAIT_FOR_DATA;
									end if;

								when "1011" =>
									if Buffer_Counter >= to_unsigned(12,8) then
										Output_Data(11 downto 0) <= Buffer_Data(11 downto 0);
										Output_Valid <= '1';
										Buffer_Data <= "000000000000" & Buffer_Data(110 downto 12);
										Buffer_Counter <= Buffer_Counter - 12;
									else
										Unused_Data <= Buffer_Data(15 downto 0);
										Unused_Counter <= Buffer_Counter;
										FSM_STATE <= STATE_WAIT_FOR_DATA;
									end if;

								when "1100" =>
									if Buffer_Counter >= to_unsigned(13,8) then
										Output_Data(12 downto 0) <= Buffer_Data(12 downto 0);
										Output_Valid <= '1';
										Buffer_Data <= "0000000000000" & Buffer_Data(110 downto 13);
										Buffer_Counter <= Buffer_Counter - 13;
									else
										Unused_Data <= Buffer_Data(15 downto 0);
										Unused_Counter <= Buffer_Counter;
										FSM_STATE <= STATE_WAIT_FOR_DATA;
									end if;

								when "1101" =>
									if Buffer_Counter >= to_unsigned(14,8) then
										Output_Data(13 downto 0) <= Buffer_Data(13 downto 0);
										Output_Valid <= '1';
										Buffer_Data <= "00000000000000" & Buffer_Data(110 downto 14);
										Buffer_Counter <= Buffer_Counter - 14;
									else
										Unused_Data <= Buffer_Data(15 downto 0);
										Unused_Counter <= Buffer_Counter;
										FSM_STATE <= STATE_WAIT_FOR_DATA;
									end if;

								when "1110" =>
									if Buffer_Counter >= to_unsigned(15,8) then
										Output_Data(14 downto 0) <= Buffer_Data(14 downto 0);
										Output_Valid <= '1';
										Buffer_Data <= "000000000000000" & Buffer_Data(110 downto 15);
										Buffer_Counter <= Buffer_Counter - 15;
									else
										Unused_Data <= Buffer_Data(15 downto 0);
										Unused_Counter <= Buffer_Counter;
										FSM_STATE <= STATE_WAIT_FOR_DATA;
									end if;

								when "1111" =>			
									if Buffer_Counter >= to_unsigned(16,8) then
										Output_Data(15 downto 0) <= Buffer_Data(15 downto 0);
										Output_Valid <= '1';
										Buffer_Data <= "0000000000000000" & Buffer_Data(110 downto 16);
										Buffer_Counter <= Buffer_Counter - 16;
									else
										Unused_Data <= Buffer_Data(15 downto 0);
										Unused_Counter <= Buffer_Counter;
										FSM_STATE <= STATE_WAIT_FOR_DATA;
									end if;
								
								when others => 
									FSM_STATE <= STATE_RESET;
							end case;
						end if;
	
					when STATE_PROCESS_DATA_WITH_REST =>
						-- Copy the old data 
						FSM_STATE <= STATE_PROCESS_DATA_WITH_REST_B;
						Output_Buffer_Rest <= (others => '0');
						if NewPacket = '1' then
							Buffer_Data <= (others => '0');
							Buffer_Counter <= (others => '0');
							Unused_Data <= (others => '0');
							Unused_Counter <= (others => '0');
							FSM_STATE <= STATE_WAIT_FOR_DATA;
						else								
							case Unused_Counter is
								when to_unsigned(1,8) =>
									Output_Buffer_Rest(15) <= Unused_Data(0);
								when to_unsigned(2,8) => 
									Output_Buffer_Rest(15 downto 14) <= Unused_Data(1 downto 0);
								when to_unsigned(3,8) => 
									Output_Buffer_Rest(15 downto 13) <= Unused_Data(2 downto 0);
								when to_unsigned(4,8) => 
									Output_Buffer_Rest(15 downto 12) <= Unused_Data(3 downto 0);							
								when to_unsigned(5,8) => 
									Output_Buffer_Rest(15 downto 11) <= Unused_Data(4 downto 0);														
								when to_unsigned(6,8) => 
									Output_Buffer_Rest(15 downto 10) <= Unused_Data(5 downto 0);														
								when to_unsigned(7,8) => 
									Output_Buffer_Rest(15 downto 9) <= Unused_Data(6 downto 0);														
								when to_unsigned(8,8) => 
									Output_Buffer_Rest(15 downto 8) <= Unused_Data(7 downto 0);														
								when to_unsigned(9,8) => 
									Output_Buffer_Rest(15 downto 7) <= Unused_Data(8 downto 0);														
								when to_unsigned(10,8) => 
									Output_Buffer_Rest(15 downto 6) <= Unused_Data(9 downto 0);														
								when to_unsigned(11,8) => 
									Output_Buffer_Rest(15 downto 5) <= Unused_Data(10 downto 0);							
								when to_unsigned(12,8) => 
									Output_Buffer_Rest(15 downto 4) <= Unused_Data(11 downto 0);														
								when to_unsigned(13,8) => 
									Output_Buffer_Rest(15 downto 3) <= Unused_Data(12 downto 0);														
								when to_unsigned(14,8) => 
									Output_Buffer_Rest(15 downto 2) <= Unused_Data(13 downto 0);														
								when to_unsigned(15,8) => 
									Output_Buffer_Rest(15 downto 1) <= Unused_Data(14 downto 0);							
								when others =>
									FSM_STATE <= STATE_RESET;
							end case;
							Output_Buffer_Counter <= unsigned("0000" & Resolution) + to_unsigned(1,8) - Unused_Counter;	
					end if;
					
					when STATE_PROCESS_DATA_WITH_REST_B => 
						-- Add the missing bits from the new data 
						FSM_STATE <= STATE_PROCESS_DATA_WITH_REST_C;
						if NewPacket = '1' then
							Buffer_Data <= (others => '0');
							Buffer_Counter <= (others => '0');
							Unused_Data <= (others => '0');
							Unused_Counter <= (others => '0');
							FSM_STATE <= STATE_WAIT_FOR_DATA;
						else						
							case Output_Buffer_Counter is
								when to_unsigned(1,8) =>
									Output_Buffer_Rest <= Buffer_Data(0) & Output_Buffer_Rest(15 downto 1);
									Buffer_Data <= "0" & Buffer_Data(110 downto 1);
									Buffer_Counter <= Buffer_Counter - 1;
								when to_unsigned(2,8) =>
									Output_Buffer_Rest <= Buffer_Data(1 downto 0) & Output_Buffer_Rest(15 downto 2);
									Buffer_Data <= "00" & Buffer_Data(110 downto 2);
									Buffer_Counter <= Buffer_Counter - 2;							
								when to_unsigned(3,8) =>
									Output_Buffer_Rest <= Buffer_Data(2 downto 0) & Output_Buffer_Rest(15 downto 3);
									Buffer_Data <= "000" & Buffer_Data(110 downto 3);
									Buffer_Counter <= Buffer_Counter - 3;							
								when to_unsigned(4,8) =>
									Output_Buffer_Rest <= Buffer_Data(3 downto 0) & Output_Buffer_Rest(15 downto 4);
									Buffer_Data <= "0000" & Buffer_Data(110 downto 4);
									Buffer_Counter <= Buffer_Counter - 4;
								when to_unsigned(5,8) =>
									Output_Buffer_Rest <= Buffer_Data(4 downto 0) & Output_Buffer_Rest(15 downto 5);
									Buffer_Data <= "00000" & Buffer_Data(110 downto 5);
									Buffer_Counter <= Buffer_Counter - 5;
								when to_unsigned(6,8) =>
									Output_Buffer_Rest <= Buffer_Data(5 downto 0) & Output_Buffer_Rest(15 downto 6);
									Buffer_Data <= "000000" & Buffer_Data(110 downto 6);
									Buffer_Counter <= Buffer_Counter - 6;
								when to_unsigned(7,8) =>
									Output_Buffer_Rest <= Buffer_Data(6 downto 0) & Output_Buffer_Rest(15 downto 7);
									Buffer_Data <= "0000000" & Buffer_Data(110 downto 7);
									Buffer_Counter <= Buffer_Counter - 7;
								when to_unsigned(8,8) =>
									Output_Buffer_Rest <= Buffer_Data(7 downto 0) & Output_Buffer_Rest(15 downto 8);
									Buffer_Data <= "00000000" & Buffer_Data(110 downto 8);
									Buffer_Counter <= Buffer_Counter - 8;
								when to_unsigned(9,8) =>
									Output_Buffer_Rest <= Buffer_Data(8 downto 0) & Output_Buffer_Rest(15 downto 9);
									Buffer_Data <= "000000000" & Buffer_Data(110 downto 9);
									Buffer_Counter <= Buffer_Counter - 9;
								when to_unsigned(10,8) =>
									Output_Buffer_Rest <= Buffer_Data(9 downto 0) & Output_Buffer_Rest(15 downto 10);
									Buffer_Data <= "0000000000" & Buffer_Data(110 downto 10);
									Buffer_Counter <= Buffer_Counter - 10;
								when to_unsigned(11,8) =>
									Output_Buffer_Rest <= Buffer_Data(10 downto 0) & Output_Buffer_Rest(15 downto 11);
									Buffer_Data <= "00000000000" & Buffer_Data(110 downto 11);
									Buffer_Counter <= Buffer_Counter - 11;
								when to_unsigned(12,8) =>
									Output_Buffer_Rest <= Buffer_Data(11 downto 0) & Output_Buffer_Rest(15 downto 12);
									Buffer_Data <= "000000000000" & Buffer_Data(110 downto 12);
									Buffer_Counter <= Buffer_Counter - 12;
								when to_unsigned(13,8) =>
									Output_Buffer_Rest <= Buffer_Data(12 downto 0) & Output_Buffer_Rest(15 downto 13);
									Buffer_Data <= "0000000000000" & Buffer_Data(110 downto 13);
									Buffer_Counter <= Buffer_Counter - 13;
								when to_unsigned(14,8) =>
									Output_Buffer_Rest <= Buffer_Data(13 downto 0) & Output_Buffer_Rest(15 downto 14);
									Buffer_Data <= "00000000000000" & Buffer_Data(110 downto 14);
									Buffer_Counter <= Buffer_Counter - 14;
								when to_unsigned(15,8) =>
									Output_Buffer_Rest <= Buffer_Data(14 downto 0) & Output_Buffer_Rest(15 downto 15);
									Buffer_Data <= "000000000000000" & Buffer_Data(110 downto 15);
									Buffer_Counter <= Buffer_Counter - 15;
								when others =>
									FSM_STATE <= STATE_RESET;
							end case;
						end if;
						
					when STATE_PROCESS_DATA_WITH_REST_C => 
						-- Fill up the other bits with 0
						FSM_STATE <= STATE_PROCESS_DATA_WITHOUT_REST;
						Output_Valid <= '1';
						Output_Data <= (others => '0');
						if NewPacket = '1' then
							Buffer_Data <= (others => '0');
							Buffer_Counter <= (others => '0');
							Unused_Data <= (others => '0');
							Unused_Counter <= (others => '0');
							FSM_STATE <= STATE_WAIT_FOR_DATA;
						else
							case Resolution is
								
								when "0001" => 
									Output_Data(1 downto 0) <= Output_Buffer_Rest(15 downto 14);
								when "0010" => 
									Output_Data(2 downto 0) <= Output_Buffer_Rest(15 downto 13);
								when "0011" => 
									Output_Data(3 downto 0) <= Output_Buffer_Rest(15 downto 12);							
								when "0100" => 
									Output_Data(4 downto 0) <= Output_Buffer_Rest(15 downto 11);							
								when "0101" => 
									Output_Data(5 downto 0) <= Output_Buffer_Rest(15 downto 10);							
								when "0110" => 
									Output_Data(6 downto 0) <= Output_Buffer_Rest(15 downto 9);							
								when "0111" => 
									Output_Data(7 downto 0) <= Output_Buffer_Rest(15 downto 8);							
								when "1000" => 
									Output_Data(8 downto 0) <= Output_Buffer_Rest(15 downto 7);							
								when "1001" => 
									Output_Data(9 downto 0) <= Output_Buffer_Rest(15 downto 6);							
								when "1010" => 
									Output_Data(10 downto 0) <= Output_Buffer_Rest(15 downto 5);							
								when "1011" => 
									Output_Data(11 downto 0) <= Output_Buffer_Rest(15 downto 4);							
								when "1100" => 
									Output_Data(12 downto 0) <= Output_Buffer_Rest(15 downto 3);							
								when "1101" => 
									Output_Data(13 downto 0) <= Output_Buffer_Rest(15 downto 2);							
								when "1110" => 
									Output_Data(14 downto 0) <= Output_Buffer_Rest(15 downto 1);							
								when "1111" => 
									Output_Data(15 downto 0) <= Output_Buffer_Rest(15 downto 0);
								when others => 
									FSM_STATE <= STATE_RESET;
									Output_Valid <= '1';
							end case;
						end if;
				end case;
		end if;
	end process FSM;
end architecture Behavioral;

