library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;
entity StreamDecomposer is
    Port ( 

		Resolution 			: in  STD_LOGIC_VECTOR(3 downto 0);
		
		-- This comes in... 
		Input_Data 			: in  STD_LOGIC_VECTOR(110 downto 0);
		Input_IsHeader		: in 	STD_LOGIC;
		Input_HeaderLong	: in	STD_LOGIC;
		Input_Valid			: in  STD_LOGIC;
		Input_RequestNext	: out	STD_LOGIC;
		
		-- This comes out...
		Output_Data			: out STD_LOGIC_VECTOR(15 downto 0);
		Output_Valid		: out STD_LOGIC;
		
		-- New Packet
		NewPacket			: in  STD_LOGIC;

		-- Usual Ports 
		Clk : in  STD_LOGIC;
      Reset : in  STD_LOGIC
	 );
end StreamDecomposer;

architecture Behavioral of StreamDecomposer is
	
	signal Buffer_Data : STD_LOGIC_VECTOR(110 downto 0) := (others => '0');
	signal Buffer_Counter : unsigned (7 downto 0) := (others => '0');
	
	type STATE_TYPE is (
		-- Reset State 
		STATE_RESET,
		STATE_WAIT_FOR_DATA,
		STATE_PROCESS_DATA,
		STATE_AFTERSEND
		);
	signal FSM_STATE : STATE_TYPE;
	
	signal Resolution_Plus1 : unsigned (7 downto 0) := (others => '0');
	signal Output_Data_Intern	: STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
	signal Output_Data_Counter	: unsigned(7 downto 0) := (others => '0');
	
begin
	FSM : process(Reset,Clk)
	begin
		if (Reset = '1') then
			-- Default:
				Input_RequestNext <= '0';
				Buffer_Data <= (others => '0');
				Buffer_Counter <= (others => '0');
				
				Output_Data_Intern <= (others => '0');
				Output_Data_Counter <= (others => '0');
				Output_Valid <= '0';
				
				FSM_STATE <= STATE_RESET;

				Resolution_Plus1 <= (others => '0');
				
		elsif rising_edge(Clk) then
				-- Default:
				Input_RequestNext <= '0';
				Buffer_Data <= Buffer_Data;
				Buffer_Counter <= Buffer_Counter;

				Output_Data_Intern <= Output_Data_Intern;
				Output_Data_Counter <= Output_Data_Counter;
				Output_Valid <= '0';
			
				Resolution_Plus1 <= Resolution_Plus1;
				
				case FSM_STATE is 
					when STATE_RESET =>
						FSM_STATE <= STATE_WAIT_FOR_DATA;
						Buffer_Data <= (others => '0');
						Buffer_Counter <= (others => '0');
						Output_Data_Intern <= (others => '0');
						Output_Data_Counter <= (others => '0');
						
						
					-- Wait for new data... 
					when STATE_WAIT_FOR_DATA =>
						
						Resolution_Plus1 <= unsigned("0000" & Resolution) + to_unsigned(1,8);
						
						-- There is a new packet 
						if NewPacket = '1' then
							Buffer_Data <= (others => '0');
							Buffer_Counter <= (others => '0');
							FSM_STATE <= STATE_WAIT_FOR_DATA;
							Output_Data_Intern <= (others => '0');
							Output_Data_Counter <= (others => '0');
							
						-- Normal packet (contains no header components)
						elsif (Input_Valid = '1') AND (Input_IsHeader = '0') then
							Buffer_Data <= Input_Data;
							Buffer_Counter <= to_unsigned(111,8);
							Input_RequestNext <= '1';
							FSM_STATE <= STATE_PROCESS_DATA;
			
						-- Packet with short header
						-- Ignore Type (4 bit), Device ID (8bit), Timestamp (16 bit) and 2xControll Bit = 30 BIT
						-- 111 - 30 = 81
						elsif (Input_Valid = '1') AND (Input_IsHeader = '1') AND (Input_HeaderLong = '0') then
							Buffer_Data <= (others => '0');
							Buffer_Data(80 downto 0) <= Input_Data(110 downto 30);
							Buffer_Counter <= to_unsigned(81,8);
							Input_RequestNext <= '1';
							FSM_STATE <= STATE_PROCESS_DATA;	
							
						-- Packet with long header
						-- Ignore full first packet 
						-- Ignore 63 bit of the second packet 
						-- Ignore Type (4 bit), Device ID (8bit), Channel Mask (128 bit), rha_filter (4Bit),
						-- timestamp (16Bit), Samplerate (8 Bit), resolution (4Bit) and 
						-- 2x Controllbit
						-- 4+8+128+4+16+8+4+1 = 174
						-- 174 - 111 = 63 Bit (Ignore first packet) 
						-- 111 - 63 = 48 Bit
						elsif (Input_Valid = '1') AND (Input_IsHeader = '1') AND (Input_HeaderLong = '1') then
							Buffer_Data <= (others => '0');
							Buffer_Data(47 downto 0) <= Input_Data(110 downto 63);
							Buffer_Counter <= to_unsigned(48,8);
							Input_RequestNext <= '1';
							FSM_STATE <= STATE_PROCESS_DATA;
							
						else
							Buffer_Data <= (others => '0');
							Buffer_Counter <= (others => '0');
							FSM_STATE <= STATE_WAIT_FOR_DATA;
							Input_RequestNext <= '1';
						end if;
					
					when STATE_PROCESS_DATA =>
						if NewPacket = '1' then
							Buffer_Data <= (others => '0');
							Buffer_Counter <= (others => '0');
							Output_Data_Intern <= (others => '0');
							Output_Data_Counter <= (others => '0');							
							FSM_STATE <= STATE_WAIT_FOR_DATA;
						else
						
							if Output_Data_Counter >= Resolution_Plus1 then
								FSM_STATE <= STATE_AFTERSEND;
								Output_Valid <= '1';
								
							elsif Buffer_Counter = to_unsigned(0,8) then
								FSM_STATE <= STATE_WAIT_FOR_DATA;
							
							else 
								Output_Data_Intern <= Buffer_Data(0) & Output_Data_Intern(15 downto 1);
								Buffer_Data <= "0" & Buffer_Data (110 downto 1);
								Buffer_Counter <= Buffer_Counter - 1;
								Output_Data_Counter <= Output_Data_Counter + 1;
								FSM_STATE <= STATE_PROCESS_DATA;
							end if;
						
						end if;

					when STATE_AFTERSEND =>
						if NewPacket = '1' then
							Buffer_Data <= (others => '0');
							Buffer_Counter <= (others => '0');
							Output_Data_Intern <= (others => '0');
							Output_Data_Counter <= (others => '0');							
							FSM_STATE <= STATE_WAIT_FOR_DATA;
						else					
							Output_Data_Intern <= (others => '0');
							Output_Data_Counter <= (others => '0');
							FSM_STATE <= STATE_PROCESS_DATA;
						end if;
						
					when others => 
						FSM_STATE <= STATE_RESET;
				end case;
		end if;
	end process FSM;
	
	AddZeros : process (Resolution, Output_Data_Intern)
	begin 
		case Resolution is
			when "0000" =>
				Output_Data <= "000000000000000" & Output_Data_Intern(15);
			when "0001" =>
				Output_Data <= "00000000000000" & Output_Data_Intern(15 downto 14);
			when "0010" =>
				Output_Data <= "0000000000000" & Output_Data_Intern(15 downto 13);
			when "0011" =>
				Output_Data <= "000000000000" & Output_Data_Intern(15 downto 12);
			when "0100" =>
				Output_Data <= "00000000000" & Output_Data_Intern(15 downto 11);
			when "0101" =>
				Output_Data <= "0000000000" & Output_Data_Intern(15 downto 10);
			when "0110" =>
				Output_Data <= "000000000" & Output_Data_Intern(15 downto 9);
			when "0111" =>
				Output_Data <= "00000000" & Output_Data_Intern(15 downto 8);			
			when "1000" =>
				Output_Data <= "0000000" & Output_Data_Intern(15 downto 7);						
			when "1001" =>
				Output_Data <= "000000" & Output_Data_Intern(15 downto 6);						
			when "1010" =>
				Output_Data <= "00000" & Output_Data_Intern(15 downto 5);						
			when "1011" =>
				Output_Data <= "0000" & Output_Data_Intern(15 downto 4);						
			when "1100" =>
				Output_Data <= "000" & Output_Data_Intern(15 downto 3);						
			when "1101" =>
				Output_Data <= "00" & Output_Data_Intern(15 downto 2);						
			when "1110" =>
				Output_Data <= "0" & Output_Data_Intern(15 downto 1);						
			when "1111" =>
				Output_Data <= Output_Data_Intern(15 downto 0);	
			when others =>
				Output_Data <= (others => '0');
		end case;	
	end process AddZeros;

end architecture Behavioral;

