library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;
-- Errors:
-- 0: No Error
-- 1: Input Data too long
-- 2: Input Data too short
-- 3: Strange state related to the input data 
-- 4: CRC Error
-- 5: Zarlink Timeout Error

entity ZarlinkDataCollector is
	Generic(
		Module_Name : STD_LOGIC_VECTOR (15 downto 0) := (others => '0')
		);
    Port ( 
		
		PollingIsEnabled			: in STD_LOGIC;
		RestartClock				: out STD_LOGIC;
		ClockEvent					: in STD_LOGIC;
		CleanUpCounter_Maximal  : in std_logic_vector (18 downto 0);
		
		-- Connection to the Zarlink 
		Zarlink_Data_Out			: out STD_LOGIC_VECTOR(119 downto 0); -- Tri-State
		Zarlink_WriteEn			: out STD_LOGIC; -- Tri-State
		Zarlink_Data_In			: in 	STD_LOGIC_VECTOR(111 downto 0);
		Zarlink_Valid_In			: in 	STD_LOGIC;
		Zarlink_BlockMode			: out	STD_LOGIC; -- Tri-State 
		Zarlink_Wait				: in  STD_LOGIC;
		Zarlink_RequestChannel  : out STD_LOGIC;
		Zarlink_GetChannel		: in  STD_LOGIC;
		Zarlink_Name				: out STD_LOGIC_VECTOR (15 downto 0); -- Tri-State
		
		
		Zarlink_EnableBlockBufferCheck : out STD_LOGIC;
		
		Data_Out						: out STD_LOGIC_VECTOR(143 downto 0);
		Valid_Out					: out STD_LOGIC;

		TriggerChannelsIn 		: in STD_LOGIC_VECTOR (31 downto 0);
		The10KHzCounter 			: in STD_LOGIC_VECTOR (15 downto 0);
		
		Zarlink_LinkQuality_Report : in STD_LOGIC;
		
		-- The usual ports 
		Reset 						: in  STD_LOGIC;			
		Clk 							: in  STD_LOGIC
	 );
end ZarlinkDataCollector;

architecture Behavioral of ZarlinkDataCollector is

-- Handle the Tri-States 	
	-- Zarlink Outputs
	signal Zarlink_Data_Out_Internal	: STD_LOGIC_VECTOR(119 downto 0) := (others => '0');
	signal Zarlink_WriteEn_Internal : STD_LOGIC := '0';
	signal Zarlink_BlockMode_Internal : STD_LOGIC := '0';
	signal PollingIsEnabled_Copy : STD_LOGIC := '0';
	signal ClockEvent_Copy : STD_LOGIC := '0';
	
	signal Zarlink_Data_In_Copy : STD_LOGIC_VECTOR(111 downto 0) := (others => '0');
	signal Zarlink_Valid_In_Copy : STD_LOGIC := '0';
	
	type STATE_TYPE is (
		-- Reset State 
		STATE_RESET,
		-- Idle State 
		STATE_IDLE,
		-- Get the Zarlink Link 
		STATE_ZARLINK_PREPARE,
		-- Recording ongoing (Wait for reset of the polling-clock)
		STATE_RECORDING_PRE,
		STATE_RECORDING_PRE2,
		-- Recording ongoing
		STATE_RECORDING,

		-- Debug stuff
		STATE_DEBUG_START,
		STATE_DEBUG_ECC,
		STATE_DEBUG_CRC,
		STATE_DEBUG_CNT1,
		STATE_DEBUG_CNT2,
		STATE_DEBUG_STOP,
		
		-- Lets if there is something in the buffer 
		STATE_CHECKBUFFER,
		STATE_CHECKBUFFER_WAIT,

		-- Get the data from the buffer 
		STATE_DOWNLOAD_DATA,
		STATE_DOWNLOAD_DATA_WAIT,
		STATE_DOWNLOAD_DATA_CONTINUE,
		
		-- Wait until we clean up the mess 
		STATE_CLEANUP
	);
	signal FSM_STATE : STATE_TYPE;

	signal ZDOR	: STD_LOGIC_VECTOR(111 downto 0) := (others => '0');
	signal Z_RD : STD_LOGIC := '0';
	signal Z_Address : STD_LOGIC_VECTOR(6 downto 0) := (others => '0');
	signal Z_BM : STD_LOGIC := '0';
	signal Z_WE : STD_LOGIC := '0';
	signal Z_Done : STD_LOGIC := '0';
	
	signal BufferStatus : STD_LOGIC_VECTOR (7 downto 0) := (others => '0');
	signal BufferStatus_Counter : unsigned (7 downto 0) := (others => '0');
	
	signal CleanUpCounter : unsigned (19 downto 0) := (others => '0');
	
	signal CleaningMode : STD_LOGIC := '0';
	
begin
-- Resort the Zarlink_Data_Out_Internal in Zarlink-Fashion
Zarlink_Data_Out_Internal <= 
		 ZDOR(104)&ZDOR(105)&ZDOR(106)&ZDOR(107)&ZDOR(108)&ZDOR(109)&ZDOR(110)&ZDOR(111)
		&ZDOR(96)&ZDOR(97)&ZDOR(98)&ZDOR(99)&ZDOR(100)&ZDOR(101)&ZDOR(102)&ZDOR(103)
		&ZDOR(88)&ZDOR(89)&ZDOR(90)&ZDOR(91)&ZDOR(92)&ZDOR(93)&ZDOR(94)&ZDOR(95)
		&ZDOR(80)&ZDOR(81)&ZDOR(82)&ZDOR(83)&ZDOR(84)&ZDOR(85)&ZDOR(86)&ZDOR(87)
		&ZDOR(72)&ZDOR(73)&ZDOR(74)&ZDOR(75)&ZDOR(76)&ZDOR(77)&ZDOR(78)&ZDOR(79)
		&ZDOR(64)&ZDOR(65)&ZDOR(66)&ZDOR(67)&ZDOR(68)&ZDOR(69)&ZDOR(70)&ZDOR(71)
		&ZDOR(56)&ZDOR(57)&ZDOR(58)&ZDOR(59)&ZDOR(60)&ZDOR(61)&ZDOR(62)&ZDOR(63)
		&ZDOR(48)&ZDOR(49)&ZDOR(50)&ZDOR(51)&ZDOR(52)&ZDOR(53)&ZDOR(54)&ZDOR(55)
		&ZDOR(40)&ZDOR(41)&ZDOR(42)&ZDOR(43)&ZDOR(44)&ZDOR(45)&ZDOR(46)&ZDOR(47)
		&ZDOR(32)&ZDOR(33)&ZDOR(34)&ZDOR(35)&ZDOR(36)&ZDOR(37)&ZDOR(38)&ZDOR(39)
		&ZDOR(24)&ZDOR(25)&ZDOR(26)&ZDOR(27)&ZDOR(28)&ZDOR(29)&ZDOR(30)&ZDOR(31)
		&ZDOR(16)&ZDOR(17)&ZDOR(18)&ZDOR(19)&ZDOR(20)&ZDOR(21)&ZDOR(22)&ZDOR(23)
		&ZDOR(8)&ZDOR(9)&ZDOR(10)&ZDOR(11)&ZDOR(12)&ZDOR(13)&ZDOR(14)&ZDOR(15)
		&ZDOR(0)&ZDOR(1)&ZDOR(2)&ZDOR(3)&ZDOR(4)&ZDOR(5)&ZDOR(6)&ZDOR(7)
		&Z_Address(0)&Z_Address(1)&Z_Address(2)&Z_Address(3)&Z_Address(4)&Z_Address(5)&Z_Address(6)&Z_RD;
Zarlink_BlockMode_Internal	<= Z_BM;
Zarlink_WriteEn_Internal <= Z_WE;

-- Tri - State Buffers: 
-- Zarlink Tri-State Output
Zarlink_Data_Out <= Zarlink_Data_Out_Internal when Zarlink_GetChannel = '1' else (others => 'Z');
Zarlink_WriteEn <= Zarlink_WriteEn_Internal when Zarlink_GetChannel = '1' else 'Z';
Zarlink_BlockMode <= Zarlink_BlockMode_Internal when Zarlink_GetChannel = '1' else 'Z';
Zarlink_Name <= Module_Name when Zarlink_GetChannel = '1' else (others => 'Z');

Data_Out	<= TriggerChannelsIn(15 downto 0) & The10KHzCounter 
				& Zarlink_Data_In_Copy(7 downto 0)
				& Zarlink_Data_In_Copy(15 downto 8)
				& Zarlink_Data_In_Copy(23 downto 16)
				& Zarlink_Data_In_Copy(31 downto 24)
				& Zarlink_Data_In_Copy(39 downto 32)
				& Zarlink_Data_In_Copy(47 downto 40)
				& Zarlink_Data_In_Copy(55 downto 48)
				& Zarlink_Data_In_Copy(63 downto 56)
				& Zarlink_Data_In_Copy(71 downto 64)
				& Zarlink_Data_In_Copy(79 downto 72)
				& Zarlink_Data_In_Copy(87 downto 80)
				& Zarlink_Data_In_Copy(95 downto 88)
				& Zarlink_Data_In_Copy(103 downto 96)
				& Zarlink_Data_In_Copy(111 downto 104);
Valid_Out <= Zarlink_Valid_In_Copy;

	FSM : process(Reset,Clk)
	begin
		if (Reset = '1') then
			-- Default:
				ZDOR <= (others => '0');
				Z_RD <= '0';
				Z_Address <= (others => '0');						
				Z_BM <= '0';
				Z_WE <= '0';
				Z_Done <= '0';
				Zarlink_RequestChannel <= '0';
				
				RestartClock <= '0';
				
				FSM_STATE <= STATE_RESET;
				PollingIsEnabled_Copy <= '0';
				ClockEvent_Copy <= '0';
				BufferStatus <= (others => '0');
				BufferStatus_Counter <= (others => '0');
				CleaningMode <= '0';				
				Zarlink_Valid_In_Copy <= '0';
				Zarlink_Data_In_Copy <= (others => '0');
				CleanUpCounter <= (others => '0');
				Zarlink_EnableBlockBufferCheck <= '1';
		elsif rising_edge(Clk) then

			-- Default:
				ZDOR <= (others => '0');
				Z_RD <= '0';
				Z_Address <= (others => '0');						
				Z_BM <= '0';
				Z_WE <= '0';
				Z_Done <= Z_Done;
				Zarlink_RequestChannel <= '0';
				
				RestartClock <= '0';
				
				PollingIsEnabled_Copy <= PollingIsEnabled;
				ClockEvent_Copy <= ClockEvent;
				
				BufferStatus <= BufferStatus;
				BufferStatus_Counter <= BufferStatus_Counter;
				
				CleaningMode <= CleaningMode;
				CleanUpCounter <= CleanUpCounter;
				
				Zarlink_Valid_In_Copy <= '0';
				Zarlink_Data_In_Copy <= (others => '0');
				
				Zarlink_EnableBlockBufferCheck <= '0';
				
				case FSM_STATE is
					when STATE_RESET => 
						
						FSM_STATE <= STATE_IDLE;
						Zarlink_EnableBlockBufferCheck <= '1';
						
					when STATE_IDLE => 
					
							-- We enter here wehen the polling was enabled 
							if PollingIsEnabled_Copy = '1' then 
								-- The recording started 
								FSM_STATE <= STATE_RECORDING;
								-- Reset the clock... 
								RestartClock <= '1';
							else
								-- Wait 
								FSM_STATE <= STATE_IDLE;
							end if;
							CleaningMode <= '0';
							Zarlink_EnableBlockBufferCheck <= '1';
					
					-- Is necessary for compensating the delay beween resetting the polling clock
					-- and the regarding reset of the polling clock signal
					when STATE_RECORDING_PRE => 
					
							Z_Done <= '0';
							Zarlink_EnableBlockBufferCheck <= '1';					
							FSM_STATE <= STATE_RECORDING_PRE2;
					-- Same reason as above... But here we deal with detail, that this module
					-- uses a copy of the orgignal sigal (which introduces a second delay of one)
					when STATE_RECORDING_PRE2 => 
					
							Z_Done <= '0';
							Zarlink_EnableBlockBufferCheck <= '1';					
							FSM_STATE <= STATE_RECORDING;
						
					when STATE_RECORDING => 
							
							-- If we enter this section, then recording was disabled... 
							if PollingIsEnabled_Copy <= '0' then
								-- The recordning ended...
								FSM_STATE <= STATE_CLEANUP;
								-- We set the mode for cleaning up the Zarlink buffer. 
								CleaningMode <= '1';
								CleanUpCounter <= (others => '0');
								Zarlink_RequestChannel <= '1';
								
							elsif ClockEvent_Copy = '1' then
								-- The clock rang... Do something... 
								-- like checking if there is data in the buffer 
								FSM_STATE <= STATE_ZARLINK_PREPARE;
								Zarlink_RequestChannel <= '1';
							-- Wait until somethink happens...
							else
								-- Wait 
								FSM_STATE <= STATE_RECORDING;
							end if;
							Z_Done <= '0';
							Zarlink_EnableBlockBufferCheck <= '1';

					when STATE_ZARLINK_PREPARE =>
						-- If the bell rang, we want to check if there is something in the buffer 
						Zarlink_RequestChannel <= '1';
						if Zarlink_GetChannel = '1' AND Zarlink_LinkQuality_Report = '0' then
							-- We have the channel to the zarlink. 
							FSM_STATE <= STATE_CHECKBUFFER;
						elsif Zarlink_GetChannel = '1' AND Zarlink_LinkQuality_Report = '1' then 
							-- We want information!!!! Information! 
							FSM_STATE <= STATE_DEBUG_START;
						else
							-- Continue waiting
							FSM_STATE <= STATE_ZARLINK_PREPARE;
						end if;
						Z_Done <= '0';
						Zarlink_EnableBlockBufferCheck <= '1';

				-- Debug stuff
					when STATE_DEBUG_START => 
					
						Zarlink_RequestChannel <= '1';
						if (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							FSM_STATE <= STATE_DEBUG_ECC;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_DEBUG_START;
							Z_Address <= std_logic_vector(to_unsigned(6,7));
							ZDOR(7 downto 0) <= "00000111";
							Z_RD <= '0';
							Z_BM <= '0';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_DEBUG_START;						
						end if;
						
					when STATE_DEBUG_ECC => 
					
						Zarlink_RequestChannel <= '1';
						if (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							FSM_STATE <= STATE_DEBUG_CRC;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_DEBUG_ECC;
							Z_Address <= std_logic_vector(to_unsigned(15,7));
							ZDOR <= (others => '0');
							Z_RD <= '1';
							Z_BM <= '0';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_DEBUG_ECC;						
						end if;
					
					when STATE_DEBUG_CRC =>

						Zarlink_RequestChannel <= '1';
						if (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							FSM_STATE <= STATE_DEBUG_CNT1;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_DEBUG_CRC;
							Z_Address <= std_logic_vector(to_unsigned(16,7));
							ZDOR <= (others => '0');
							Z_RD <= '1';
							Z_BM <= '0';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_DEBUG_CRC;						
						end if;
					
					when STATE_DEBUG_CNT1 =>

						Zarlink_RequestChannel <= '1';
						if (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							FSM_STATE <= STATE_DEBUG_CNT2;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_DEBUG_CNT1;
							Z_Address <= std_logic_vector(to_unsigned(17,7));
							ZDOR <= (others => '0');
							Z_RD <= '1';
							Z_BM <= '0';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_DEBUG_CNT1;						
						end if;					
						
					when STATE_DEBUG_CNT2 => 
					
						Zarlink_RequestChannel <= '1';
						if (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							FSM_STATE <= STATE_DEBUG_STOP;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_DEBUG_CNT2;
							Z_Address <= std_logic_vector(to_unsigned(18,7));
							ZDOR <= (others => '0');
							Z_RD <= '1';
							Z_BM <= '0';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_DEBUG_CNT2;						
						end if;		

					when STATE_DEBUG_STOP => 
					
						Zarlink_RequestChannel <= '1';
						if (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							FSM_STATE <= STATE_CHECKBUFFER;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_DEBUG_STOP;
							Z_Address <= std_logic_vector(to_unsigned(6,7));
							ZDOR(7 downto 0) <= "00000000";
							Z_RD <= '0';
							Z_BM <= '0';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_DEBUG_STOP;						
						end if;
							
					when STATE_CHECKBUFFER =>
						-- Is there something in the buffer? 
						Zarlink_RequestChannel <= '1';
						if (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							FSM_STATE <= STATE_CHECKBUFFER_WAIT;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_CHECKBUFFER;
							Z_Address <= std_logic_vector(to_unsigned(1,7));
							ZDOR <= (others => '0');
							Z_RD <= '1';
							Z_BM <= '0';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_CHECKBUFFER;						
						end if;
				
					when STATE_CHECKBUFFER_WAIT =>
						Zarlink_RequestChannel <= '1';					
						-- In the case the Zarlink delivered no new data, continue to wait 
						if Zarlink_Valid_In = '0' then
							FSM_STATE <= STATE_CHECKBUFFER_WAIT;
						-- Buffer is emtpy and the polling ended. Which means that the buffer is clear. 
						-- Go to idle and wait until polling is re-activated 
						elsif (Zarlink_Data_In (7 downto 0) = std_logic_vector(to_unsigned(0,8))) AND (CleaningMode = '1') then
							BufferStatus <= Zarlink_Data_In (7 downto 0);
							FSM_STATE <= STATE_IDLE;	

						-- Buffer is empty, which mean we go pack an wait again until the clock rangs again. 
						elsif (Zarlink_Data_In (7 downto 0) = std_logic_vector(to_unsigned(0,8))) AND (CleaningMode = '0') then
							BufferStatus <= Zarlink_Data_In (7 downto 0);
							RestartClock <= '1';
							FSM_STATE <= STATE_RECORDING_PRE;	

						-- There is something in the buffer 
						else 
							BufferStatus <= Zarlink_Data_In (7 downto 0);
							FSM_STATE <= STATE_DOWNLOAD_DATA;
							BufferStatus_Counter <= (others => '0');
						end if;
						Z_Done <= '0';

					
					when STATE_DOWNLOAD_DATA => 
					
						Zarlink_RequestChannel <= '1';
						if (Zarlink_Wait = '1') AND (Z_Done = '1') then
							-- Message was accepted and now continue 
							FSM_STATE <= STATE_DOWNLOAD_DATA_WAIT;
							Z_Done <= '0';
						elsif (Zarlink_Wait = '0') then
							-- Repeat the message until it was accepted 
							FSM_STATE <= STATE_DOWNLOAD_DATA;
							Z_Address <= std_logic_vector(to_unsigned(45,7));
							ZDOR <= (others => '0');
							Z_RD <= '1';
							Z_BM <= '1';
							Z_WE <= '1';
							Z_Done <= '1';
						else
							-- Wait until we can send our message 
							FSM_STATE <= STATE_DOWNLOAD_DATA;						
						end if;
							
					when STATE_DOWNLOAD_DATA_WAIT	=>
						Zarlink_RequestChannel <= '1';					
						if (Zarlink_Valid_In = '0') then
							FSM_STATE <= STATE_DOWNLOAD_DATA_WAIT;
						else
							FSM_STATE <= STATE_DOWNLOAD_DATA_CONTINUE;
							BufferStatus_Counter <= BufferStatus_Counter + 1;
							Zarlink_Data_In_Copy <= Zarlink_Data_In;
							-- If we are in the cleaning mode, we never produce valid output!
							Zarlink_Valid_In_Copy <= NOT CleaningMode;							
						end if;

					when STATE_DOWNLOAD_DATA_CONTINUE =>
						Zarlink_RequestChannel <= '1';
						if (std_logic_vector(BufferStatus_Counter) >= BufferStatus) AND (CleaningMode = '1') then
							FSM_STATE <= STATE_IDLE;
						elsif (std_logic_vector(BufferStatus_Counter) >= BufferStatus) AND (CleaningMode = '0') then
							FSM_STATE <= STATE_RECORDING_Pre;	
							RestartClock <= '1';
						else
							FSM_STATE <= STATE_DOWNLOAD_DATA;
						end if;
					
					when STATE_CLEANUP => 	
						-- After ending the recording wait a moment and then clean the buffer 
						Zarlink_RequestChannel <= '1';
						if std_logic_vector(CleanUpCounter) < "0" & CleanUpCounter_Maximal then 
							FSM_STATE <= STATE_CLEANUP;
							CleanUpCounter <= CleanUpCounter + 1;
						else
							FSM_STATE <= STATE_ZARLINK_PREPARE;
						end if;
						
					when others => 
						FSM_STATE <= STATE_RESET;
				
				end case;

		end if;
	end process FSM;

end architecture Behavioral;
