library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;

-- Error Code:
-- 0 : Everything was okay
-- 10 : TimeOut - Had problems with the tx buffer
-- 11 : TimeOut - Had problems with the rx buffer

entity ZarlinkEchoModule is
    Port ( 
		-- Connection to the Zarlink 
		Zarlink_Data_Out			: in 		STD_LOGIC_VECTOR(119 downto 0); 
		Zarlink_WriteEn			: in 		STD_LOGIC; 
		Zarlink_Data_In			: out 	STD_LOGIC_VECTOR(111 downto 0);
		Zarlink_Valid_In			: out 	STD_LOGIC;
		Zarlink_BlockMode			: in		STD_LOGIC; 
		Zarlink_Wait				: out  	STD_LOGIC;
		Zarlink_Name				: in 		STD_LOGIC_VECTOR (15 downto 0);
		
		Zarlink_Timeout			: in 		STD_LOGIC_VECTOR (27 downto 0);
		Zarlink_DebugMode			: in 		STD_LOGIC;
		Zarlink_Error 				: out		STD_LOGIC;
		
		-- Connection to the Network
		Network_Data				: out STD_LOGIC_VECTOR(15 downto 0); -- Tri-State 
		Network_WriteEn			: out STD_LOGIC; -- Tri-State
		Network_Wait				: in  STD_LOGIC;
		Network_RequestChannel  : out STD_LOGIC; 
		Network_GetChannel		: in  STD_LOGIC;
		CRC_Reset					: out STD_LOGIC;
		CRC_00						: in 	STD_LOGIC_VECTOR(15 downto 0);
		CRC_01						: in 	STD_LOGIC_VECTOR(15 downto 0);
		CRC_10						: in 	STD_LOGIC_VECTOR(15 downto 0);
		CRC_11						: in 	STD_LOGIC_VECTOR(15 downto 0);

		EnableBlockBufferCheck  : in STD_LOGIC;
		
		Cooldown_Counter_Max 	: in std_logic_vector(7 downto 0);

		The10KHzCounter 			: in STD_LOGIC_VECTOR (15 downto 0);
		The32KHzCounter 			: in STD_LOGIC_VECTOR (15 downto 0);
		
		-- Connection to the Zarlink 
		SPI_ZL_CS 					: out STD_LOGIC;
		SPI_SDI 						: out STD_LOGIC;
		SPI_SDO 						: in 	STD_LOGIC;
		SPI_CLK 						: out STD_LOGIC;		
		
		Length_Of_SPIClk			: in STD_LOGIC_VECTOR(7 downto 0);
		
		Zarlink_Retry 				: in STD_LOGIC_vector(18 downto 0);
		
		-- The usual ports 
		Reset 						: in  STD_LOGIC;
		Clk 							: in  STD_LOGIC
	 );
end ZarlinkEchoModule;

architecture Behavioral of ZarlinkEchoModule is
	signal Zarlink_BlockMode_Copy	: STD_LOGIC := '0';
	signal Zarlink_Data_Out_Copy : STD_LOGIC_VECTOR(119 downto 0) := (others => '0');
	signal Zarlink_Name_Copy : STD_LOGIC_VECTOR (15 downto 0) := (others => '0');
	-- Network Outputs
	signal Network_Data_Internal : STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
	signal Network_WriteEn_Internal : STD_LOGIC := '0';
	signal Network_GetChannel_Internal	: STD_LOGIC := '0';
	-- 0x55AA
	constant NET_HEADER : STD_LOGIC_VECTOR(15 downto 0) := std_logic_vector(to_unsigned(21930,16));
	-- 0x8002
	constant NET_RESPONSEID : STD_LOGIC_VECTOR(15 downto 0) := std_logic_vector(to_unsigned(32770,16));
	-- Not constant because the length can change 
	signal NET_LENGTH : STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
	signal CRC_Reset_Internal : STD_LOGIC := '0';
	
	type STATE_TYPE is (
		-- Reset State
		STATE_RESET,
		-- Idle State
		STATE_IDLE,
		STATE_BLOCK_BUFFER_CHECK,
		STATE_BLOCK_BUFFER_CHECK_WAIT,
		STATE_BLOCK_BUFFER_CHECK_PAUSE,
		STATE_TRANSMISSION,
		STATE_TRANSMISSION_WAIT,
		STATE_PREPARE_NETWORK,
		STATE_NETWORK_COMMUNICATE_HEADER,
		STATE_NETWORK_COMMUNICATE_LENGTH,
		STATE_NETWORK_COMMUNICATE_RESPONSEID,
		STATE_NETWORK_COMMUNICATE_ID,
		STATE_NETWORK_COMMUNICATE_ERROR,
		STATE_NETWORK_COMMUNICATE_TIMESTAMP_A,
		STATE_NETWORK_COMMUNICATE_TIMESTAMP_B,
		STATE_NETWORK_COMMUNICATE_WORD_0,
		STATE_NETWORK_COMMUNICATE_WORD_1,
		STATE_NETWORK_COMMUNICATE_WORD_2,
		STATE_NETWORK_COMMUNICATE_WORD_3,
		STATE_NETWORK_COMMUNICATE_WORD_4,
		STATE_NETWORK_COMMUNICATE_WORD_5,
		STATE_NETWORK_COMMUNICATE_WORD_6,
		STATE_NETWORK_COMMUNICATE_WORD_7,
		STATE_NETWORK_COMMUNICATE_WORD_8,
		STATE_NETWORK_COMMUNICATE_WORD_9,
		STATE_NETWORK_COMMUNICATE_WORD_10,
		STATE_NETWORK_COMMUNICATE_WORD_11,
		STATE_NETWORK_COMMUNICATE_WORD_12,
		STATE_NETWORK_COMMUNICATE_WORD_13,
		STATE_NETWORK_COMMUNICATE_WORD_14,
		STATE_NETWORK_COMMUNICATE_WAIT_A,
		-- Communicate with the network - WAIT B
		STATE_NETWORK_COMMUNICATE_WAIT_B,		
		-- Communicate with the network - WAIT C
		STATE_NETWORK_COMMUNICATE_WAIT_C,		
		-- Communicate with the network (CRC 00)
		STATE_NETWORK_COMMUNICATE_CRC00,
		-- Communicate with the network (CRC 01)
		STATE_NETWORK_COMMUNICATE_CRC01,
		-- Communicate with the network (CRC 10)
		STATE_NETWORK_COMMUNICATE_CRC10,
		-- Communicate with the network (CRC 11)
		STATE_NETWORK_COMMUNICATE_CRC11,		
		STATE_COOLDOWN_HOLDCONNECTION,
		STATE_COOLDOWN
	);
	signal FSM_STATE : STATE_TYPE;
	
	signal DummyCounter : unsigned(111 downto 0) := (others => '0');

	signal ErrorState : STD_LOGIC_VECTOR (15 downto 0) := (others => '0');

	signal CRC_01_Copy : STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
	signal CRC_10_Copy : STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
	signal CRC_11_Copy : STD_LOGIC_VECTOR(15 downto 0) := (others => '0');

	-- SPI
	signal Zarlink_Data_Out_SPI : STD_LOGIC_VECTOR(119 downto 0) := (others => '0'); 
	signal Zarlink_WriteEn_SPI : STD_LOGIC := '0'; 
	signal Zarlink_Data_In_SPI : STD_LOGIC_VECTOR(111 downto 0) := (others => '0');
	signal Zarlink_Valid_In_SPI	: STD_LOGIC := '0'; 
	signal Zarlink_BlockMode_SPI : STD_LOGIC := '0'; 
	signal Zarlink_Wait_SPI : STD_LOGIC := '0'; 
	signal Zarlink_Data_In_Copy : STD_LOGIC_VECTOR(111 downto 0) := (others => '0');
	signal Zarlink_Valid_In_Copy : STD_LOGIC := '0';
	signal TimeoutCounter	: unsigned (27 downto 0) := (others => '0');
	signal Reset_SPI : std_logic := '1';
	
	signal Zarlink_Retry_Counter : unsigned(18 downto 0) := (others => '0');
	
	signal Cooldown_Counter : unsigned(7 downto 0) := (others => '0');
	
	
begin

Zarlink_Data_In <= Zarlink_Data_In_Copy;
Zarlink_Valid_In <= Zarlink_Valid_In_Copy;

-- Zarlink Tri-State Output
Network_Data <= Network_Data_Internal when Network_GetChannel_Internal = '1' else (others => 'Z');
Network_WriteEn <= Network_WriteEn_Internal when Network_GetChannel_Internal = '1' else 'Z';
CRC_Reset <= CRC_Reset_Internal when Network_GetChannel_Internal = '1' else 'Z';

	FSM : process(Reset,Clk)
	begin
		if (Reset = '1') then
			FSM_STATE <= STATE_RESET;
			Zarlink_Wait <= '1';
			Network_RequestChannel <= '0';
			NET_LENGTH <= (others => '0');
			DummyCounter <= (others => '0');
			Network_GetChannel_Internal <= '0';
			Zarlink_BlockMode_Copy <= '0';
			Zarlink_Data_Out_Copy <= (others => '0');
			Zarlink_Name_Copy <= (others => '0');
			Network_Data_Internal <= (others => '0');
			Network_WriteEn_Internal <= '0';
			CRC_Reset_Internal <= '0';
			CRC_01_Copy <= (others => '0');
			CRC_10_Copy <= (others => '0');
			CRC_11_Copy <= (others => '0');	
			TimeoutCounter	<= (others => '0');		
			ErrorState <= (others => '0');
			Zarlink_Data_Out_SPI <= (others => '0'); 
			Zarlink_WriteEn_SPI <= '0'; 
			Zarlink_BlockMode_SPI <= '0'; 
			Reset_SPI <= '1';

			Zarlink_Valid_In_Copy <= '0';
			Zarlink_Data_In_Copy <= (others => '0');
			Zarlink_Error <= '0';
			
			Zarlink_Retry_Counter <= (others => '0');
			Cooldown_Counter <= (others => '0');
			
		elsif rising_edge(Clk) then
			Network_GetChannel_Internal <= Network_GetChannel;
			-- Default:
			Zarlink_Wait <= '1';
			Network_RequestChannel <= '1';
			DummyCounter <= DummyCounter + 1;
			NET_LENGTH <= NET_LENGTH;
			Network_Data_Internal <= (others => '0');
			Network_WriteEn_Internal <= '0';
			Zarlink_BlockMode_Copy <= Zarlink_BlockMode_Copy;
			Zarlink_Data_Out_Copy <= Zarlink_Data_Out_Copy;
			Zarlink_Name_Copy <= Zarlink_Name_Copy;
			
			CRC_Reset_Internal <= '0';
			CRC_01_Copy <= (others => '0');
			CRC_10_Copy <= (others => '0');
			CRC_11_Copy <= (others => '0');
			TimeoutCounter <= TimeoutCounter;
			ErrorState <= ErrorState;

			Zarlink_Data_Out_SPI <= (others => '0'); 
			Zarlink_WriteEn_SPI <= '0'; 
			Zarlink_BlockMode_SPI <= '0'; 
			Reset_SPI <= '0';

			Zarlink_Valid_In_Copy <= '0';
			Zarlink_Data_In_Copy <= Zarlink_Data_In_Copy;
			Zarlink_Error <= '0';
			
			Zarlink_Retry_Counter <= Zarlink_Retry_Counter;
			Cooldown_Counter <= (others => '0');
			
			case FSM_STATE is
				
				when STATE_RESET => 
					FSM_STATE <=  STATE_IDLE;
					Network_RequestChannel <= '0';
					ErrorState <= (others => '0');
					
				when STATE_IDLE =>
					
					TimeoutCounter	<= (others => '0');
					ErrorState <= (others => '0');
					
					-- Wait until something has to be done
					-- 
					if Zarlink_WriteEn = '1' then
						-- Do we need to check if there is something in the buffer / is there space in the buffer? 
						if (Zarlink_BlockMode = '1') AND (EnableBlockBufferCheck = '1') then
							-- Yes
							FSM_STATE <= STATE_BLOCK_BUFFER_CHECK;
						else
							-- NO
							FSM_STATE <= STATE_TRANSMISSION;
						end if;
						
						Zarlink_Wait <= '1';
						Network_RequestChannel <= '1';

						Zarlink_BlockMode_Copy <= Zarlink_BlockMode;
						Zarlink_Data_Out_Copy <= Zarlink_Data_Out;
						Zarlink_Name_Copy <= Zarlink_Name;
						
						-- How long would be the network response? 
						if Zarlink_Data_Out(0) = '0' then 
							if Zarlink_BlockMode = '1' then
								-- Write plus block
								NET_LENGTH <= std_logic_vector(to_unsigned(38,16));
							else
								-- Write plus single
								NET_LENGTH <= std_logic_vector(to_unsigned(24,16));
							end if;
						else
							if Zarlink_BlockMode = '1' then
								-- Read plus block
								NET_LENGTH <= std_logic_vector(to_unsigned(52,16));
							else
								-- Read plus sigle
								NET_LENGTH <= std_logic_vector(to_unsigned(26,16));
							end if;
						end if;
						
					else
						-- Nothing to do -> wait
						FSM_STATE <=  STATE_IDLE;
						Zarlink_Wait <= '0';
						Network_RequestChannel <= '0';
					end if;
					
					when STATE_BLOCK_BUFFER_CHECK =>
						-- Request how the status of the buffer is...
						Network_RequestChannel <= '0';
						TimeoutCounter <= TimeoutCounter + 1;
						
						if Zarlink_Data_Out_Copy(0) = '0' then
							-- < 64
							Zarlink_Data_Out_SPI(7 downto 0) <= "11000001";
							Zarlink_WriteEn_SPI <= '1';
							Zarlink_BlockMode_SPI <= '0';
						else			
							-- > 0
							Zarlink_Data_Out_SPI(7 downto 0) <= "10000001";
							Zarlink_WriteEn_SPI <= '1';
							Zarlink_BlockMode_SPI <= '0';
						end if;
						
						-- This took to long. Time Out!
						if std_logic_vector(TimeoutCounter) >= Zarlink_Timeout then
							FSM_STATE <= STATE_PREPARE_NETWORK;
							Reset_SPI <= '1';
							if Zarlink_Data_Out_Copy(0) = '0' then
								ErrorState <= std_logic_vector(to_unsigned(10,16));
							else
								ErrorState <= std_logic_vector(to_unsigned(11,16));
							end if;
							Zarlink_Error <= '1';
						else
							-- It is still time left
							FSM_STATE <= STATE_BLOCK_BUFFER_CHECK_WAIT;
						end if;
					
					when STATE_BLOCK_BUFFER_CHECK_WAIT =>
						Network_RequestChannel <= '0';
						TimeoutCounter <= TimeoutCounter + 1;

						if std_logic_vector(TimeoutCounter) >= Zarlink_Timeout then
							-- Time out. Lets report the error 
							FSM_STATE <= STATE_PREPARE_NETWORK;
							Reset_SPI <= '1';
							if Zarlink_Data_Out_Copy(0) = '0' then
								ErrorState <= std_logic_vector(to_unsigned(10,16));
							else
								ErrorState <= std_logic_vector(to_unsigned(11,16));
							end if;
							Zarlink_Error <= '1';
						elsif (Zarlink_Valid_In_SPI = '1') AND (Zarlink_Data_Out_Copy(0) = '0') then
							-- We got an answer to our status request 
							if Zarlink_Data_In_SPI(7 downto 0) < std_logic_vector(to_unsigned(60,8)) then
								-- We can continue
								FSM_STATE <= STATE_TRANSMISSION;
							else
								-- We have to re-check 
								FSM_STATE <= STATE_BLOCK_BUFFER_CHECK;
							end if;
						elsif (Zarlink_Valid_In_SPI = '1') AND (Zarlink_Data_Out_Copy(0) = '1') then
							-- We got an answer to our status request 
							
							if Zarlink_Data_In_SPI(7 downto 0) > std_logic_vector(to_unsigned(0,8)) then
								-- We can continue
								FSM_STATE <= STATE_TRANSMISSION;							
							else
								Zarlink_Retry_Counter <= (others => '0');
								-- We have to re-check 
								FSM_STATE <= STATE_BLOCK_BUFFER_CHECK_PAUSE;
							end if;
						else
							-- Wait for a response 
							FSM_STATE <= STATE_BLOCK_BUFFER_CHECK_WAIT;
						end if;
					
					when STATE_BLOCK_BUFFER_CHECK_PAUSE => 
						Network_RequestChannel <= '0';
						TimeoutCounter <= TimeoutCounter + 1;
						Zarlink_Retry_Counter <= Zarlink_Retry_Counter + 1;

						if std_logic_vector(TimeoutCounter) >= Zarlink_Timeout then
							-- Time out. Lets report the error 
							FSM_STATE <= STATE_PREPARE_NETWORK;
							Reset_SPI <= '1';
							if Zarlink_Data_Out_Copy(0) = '0' then
								ErrorState <= std_logic_vector(to_unsigned(10,16));
							else
								ErrorState <= std_logic_vector(to_unsigned(11,16));
							end if;
							Zarlink_Error <= '1';
						elsif std_logic_vector(Zarlink_Retry_Counter) >= Zarlink_Retry then
							FSM_STATE <= STATE_BLOCK_BUFFER_CHECK;
						else
							FSM_STATE <= STATE_BLOCK_BUFFER_CHECK_PAUSE;
						end if;
					
					when STATE_TRANSMISSION =>
							
							Network_RequestChannel <= '0';
							
							-- Do the transmission
							if Zarlink_Wait_SPI = '1' then
								FSM_STATE <= STATE_TRANSMISSION;
							else
								Zarlink_WriteEn_SPI <= '1';
								FSM_STATE <= STATE_TRANSMISSION_WAIT;
							end if;
							
					 		Zarlink_Data_Out_SPI <= Zarlink_Data_Out_Copy;
							Zarlink_BlockMode_SPI <= Zarlink_BlockMode_Copy;
							
					when STATE_TRANSMISSION_WAIT =>
						-- Wait until the transmission has been done 
						Network_RequestChannel <= '0';
						if Zarlink_Valid_In_SPI = '1' then 
							-- Done. Report the result 
							FSM_STATE <= STATE_PREPARE_NETWORK;
							
							-- Export the answer if this was a reading action 
							if Zarlink_Data_Out_Copy(0) = '1' then
								-- Reading
								Zarlink_Valid_In_Copy <= '1'; 
								Zarlink_Data_In_Copy <= Zarlink_Data_In_SPI;
							else
								-- Writing
								null;
							end if;
							
						else
							-- Continue to wait 
							FSM_STATE <= STATE_TRANSMISSION_WAIT;
						end if;
					
					--	We need the Network
					-- So... Lets get it. 
					when STATE_PREPARE_NETWORK => 
					
						if (Zarlink_DebugMode = '0') AND (ErrorState = std_logic_vector(to_unsigned(0,16))) then
							FSM_STATE <= STATE_COOLDOWN_HOLDCONNECTION;
							Network_RequestChannel <= '0';
						elsif Network_GetChannel_Internal = '1' then
							-- We have the channel to the network.
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_HEADER;
							CRC_Reset_Internal <= '1';
						else
							-- Continue waiting
							FSM_STATE <= STATE_PREPARE_NETWORK;
						end if;					

					-- Communicate with the network (HEADER)
					when STATE_NETWORK_COMMUNICATE_HEADER =>

						if Network_Wait = '0' then
							Network_Data_Internal <= NET_HEADER;
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_LENGTH;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_HEADER;
						end if;

					when STATE_NETWORK_COMMUNICATE_LENGTH =>

						if Network_Wait = '0' then
							Network_Data_Internal <= NET_LENGTH;
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_RESPONSEID;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_LENGTH;
						end if;

					when STATE_NETWORK_COMMUNICATE_RESPONSEID =>

						if Network_Wait = '0' then
							Network_Data_Internal <= NET_RESPONSEID;
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_ID;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_RESPONSEID;
						end if;

					when STATE_NETWORK_COMMUNICATE_ID =>

						if Network_Wait = '0' then
							Network_Data_Internal <= Zarlink_Name_Copy;
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_ERROR;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_ID;
						end if;

					when STATE_NETWORK_COMMUNICATE_ERROR =>

						if Network_Wait = '0' then
							Network_Data_Internal <= ErrorState;
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_TIMESTAMP_A;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_ERROR;
						end if;

					when STATE_NETWORK_COMMUNICATE_TIMESTAMP_A =>
						if Network_Wait = '0' then
							Network_Data_Internal <= The10KHzCounter;
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_TIMESTAMP_B;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_TIMESTAMP_A;
						end if;	
						
					when STATE_NETWORK_COMMUNICATE_TIMESTAMP_B => 
						if Network_Wait = '0' then
							Network_Data_Internal <= The32KHzCounter;
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_WORD_0;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_TIMESTAMP_B;
						end if;					
					when STATE_NETWORK_COMMUNICATE_WORD_0 =>

						if Network_Wait = '0' then
							Network_Data_Internal <= Zarlink_Data_Out_Copy(15 downto 0);
							Network_WriteEn_Internal <= '1';
							if Zarlink_BlockMode_Copy = '1' then
								-- Block
								FSM_STATE <= STATE_NETWORK_COMMUNICATE_WORD_1;
								
							elsif (Zarlink_BlockMode_Copy = '0') AND (Zarlink_Data_Out_Copy(0) = '1') then
								FSM_STATE <= STATE_NETWORK_COMMUNICATE_WORD_8;
								
							else							
								-- Single
								FSM_STATE <= STATE_NETWORK_COMMUNICATE_WAIT_A;
							end if;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_WORD_0;
						end if;					

					when STATE_NETWORK_COMMUNICATE_WORD_1 =>

						if Network_Wait = '0' then
							Network_Data_Internal <= Zarlink_Data_Out_Copy(31 downto 16);
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_WORD_2;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_WORD_1;
						end if;	

					when STATE_NETWORK_COMMUNICATE_WORD_2 =>

						if Network_Wait = '0' then
							Network_Data_Internal <= Zarlink_Data_Out_Copy(47 downto 32);
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_WORD_3;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_WORD_2;
						end if;					
						
					when STATE_NETWORK_COMMUNICATE_WORD_3 =>

						if Network_Wait = '0' then
							Network_Data_Internal <= Zarlink_Data_Out_Copy(63 downto 48);
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_WORD_4;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_WORD_3;
						end if;	

					when STATE_NETWORK_COMMUNICATE_WORD_4 =>

						if Network_Wait = '0' then
							Network_Data_Internal <= Zarlink_Data_Out_Copy(79 downto 64);
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_WORD_5;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_WORD_4;
						end if;	

					when STATE_NETWORK_COMMUNICATE_WORD_5 =>

						if Network_Wait = '0' then
							Network_Data_Internal <= Zarlink_Data_Out_Copy(95 downto 80);
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_WORD_6;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_WORD_5;
						end if;	

					when STATE_NETWORK_COMMUNICATE_WORD_6 =>

						if Network_Wait = '0' then
							Network_Data_Internal <= Zarlink_Data_Out_Copy(111 downto 96);
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_WORD_7;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_WORD_6;
						end if;	

					when STATE_NETWORK_COMMUNICATE_WORD_7 =>

						if Network_Wait = '0' then
							Network_Data_Internal <= "00000000" & Zarlink_Data_Out_Copy(119 downto 112);
							Network_WriteEn_Internal <= '1';
							if Zarlink_Data_Out_Copy(0) = '0' then
								-- Was writing action -> done
								FSM_STATE <= STATE_NETWORK_COMMUNICATE_WAIT_A;
							else
								-- Was reading action -> append the spi answer
								FSM_STATE <= STATE_NETWORK_COMMUNICATE_WORD_8;
							end if;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_WORD_7;
						end if;	

					when STATE_NETWORK_COMMUNICATE_WORD_8 =>

						if Network_Wait = '0' then
							Network_Data_Internal <= Zarlink_Data_In_Copy(15 downto 0);
							Network_WriteEn_Internal <= '1';
							if Zarlink_BlockMode_Copy = '1' then
								-- Block
								FSM_STATE <= STATE_NETWORK_COMMUNICATE_WORD_9;
							else
								-- Single 
								FSM_STATE <= STATE_NETWORK_COMMUNICATE_WAIT_A;
							end if;							
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_WORD_8;
						end if;	

					when STATE_NETWORK_COMMUNICATE_WORD_9 =>

						if Network_Wait = '0' then
							Network_Data_Internal <= Zarlink_Data_In_Copy(31 downto 16);
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_WORD_10;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_WORD_9;
						end if;	

					when STATE_NETWORK_COMMUNICATE_WORD_10 =>

						if Network_Wait = '0' then
							Network_Data_Internal <= Zarlink_Data_In_Copy(47 downto 32);
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_WORD_11;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_WORD_10;
						end if;	

					when STATE_NETWORK_COMMUNICATE_WORD_11 =>

						if Network_Wait = '0' then
							Network_Data_Internal <= Zarlink_Data_In_Copy(63 downto 48);
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_WORD_12;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_WORD_11;
						end if;	

					when STATE_NETWORK_COMMUNICATE_WORD_12 =>

						if Network_Wait = '0' then
							Network_Data_Internal <= Zarlink_Data_In_Copy(79 downto 64);
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_WORD_13;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_WORD_12;
						end if;	

					when STATE_NETWORK_COMMUNICATE_WORD_13 =>

						if Network_Wait = '0' then
							Network_Data_Internal <= Zarlink_Data_In_Copy(95 downto 80);
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_WORD_14;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_WORD_13;
						end if;	

					when STATE_NETWORK_COMMUNICATE_WORD_14 =>

						if Network_Wait = '0' then
							Network_Data_Internal <= Zarlink_Data_In_Copy(111 downto 96);
							Network_WriteEn_Internal <= '1';
								FSM_STATE <= STATE_NETWORK_COMMUNICATE_WAIT_A;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_WORD_14;
						end if;	

					-- Wait for the CRC - Values I.
					when STATE_NETWORK_COMMUNICATE_WAIT_A =>
						FSM_STATE <= STATE_NETWORK_COMMUNICATE_WAIT_B;

					-- Wait for the CRC - Values II.
					when STATE_NETWORK_COMMUNICATE_WAIT_B =>
						FSM_STATE <= STATE_NETWORK_COMMUNICATE_WAIT_C;						
					
					-- Wait for the CRC - Values III.
					when STATE_NETWORK_COMMUNICATE_WAIT_C =>
						FSM_STATE <= STATE_NETWORK_COMMUNICATE_CRC00;
						
					-- Communicate with the network (CRC 00)
					when STATE_NETWORK_COMMUNICATE_CRC00 => 
						CRC_01_Copy <= CRC_01;
						CRC_10_Copy <= CRC_10;
						CRC_11_Copy <= CRC_11;
						
						if Network_Wait = '0' then
							Network_Data_Internal <= CRC_00;
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_CRC01;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_CRC00;
						end if;
						
					-- Communicate with the network (CRC 01)
					when STATE_NETWORK_COMMUNICATE_CRC01 =>
						CRC_01_Copy <= CRC_01_Copy;
						CRC_10_Copy <= CRC_10_Copy;
						CRC_11_Copy <= CRC_11_Copy;		
						
						if Network_Wait = '0' then
							Network_Data_Internal <= CRC_01_Copy;
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_CRC10;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_CRC01;
						end if;
						
					-- Communicate with the network (CRC 10)
					when STATE_NETWORK_COMMUNICATE_CRC10 => 
						CRC_10_Copy <= CRC_10_Copy;
						CRC_11_Copy <= CRC_11_Copy;	
						if Network_Wait = '0' then
							Network_Data_Internal <= CRC_10_Copy;
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_CRC11;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_CRC10;
						end if;
						
					-- Communicate with the network (CRC 11)
					when STATE_NETWORK_COMMUNICATE_CRC11 => 

						CRC_11_Copy <= CRC_11_Copy;		
						
						if Network_Wait = '0' then
							Network_Data_Internal <= CRC_11_Copy;
							Network_WriteEn_Internal <= '1';
							FSM_STATE <= STATE_COOLDOWN_HOLDCONNECTION;
						else
							FSM_STATE <= STATE_NETWORK_COMMUNICATE_CRC11;
						end if;						
					
					when STATE_COOLDOWN_HOLDCONNECTION =>
						
						Cooldown_Counter <= Cooldown_Counter + 1;
						
						if Cooldown_Counter >= unsigned(Cooldown_Counter_Max) then
							FSM_STATE <= STATE_COOLDOWN;
						else
							FSM_STATE <= STATE_COOLDOWN_HOLDCONNECTION;
						end if;

					when STATE_COOLDOWN => 
							FSM_STATE <= STATE_IDLE;
							Zarlink_Wait <= '0';
							Network_RequestChannel <= '0';
					
					when others => 
						FSM_STATE <= STATE_RESET;
						Zarlink_Wait <= '0';
						Network_RequestChannel <= '0';
			end case;
		
		
		end if;
	end process FSM;

MyZarlinkSPIModule : entity work.Zarlink_SPI_Module 
    Port MAP ( 

		Zarlink_Data_Out => Zarlink_Data_Out_SPI,
		Zarlink_WriteEn => Zarlink_WriteEn_SPI,
		Zarlink_Data_In => Zarlink_Data_In_SPI,
		Zarlink_Valid_In => Zarlink_Valid_In_SPI,
		Zarlink_BlockMode	=> Zarlink_BlockMode_SPI,
		Zarlink_Wait => Zarlink_Wait_SPI,

		-- Connection to the Zarlink 
		SPI_ZL_CS => SPI_ZL_CS,
		SPI_SDI => SPI_SDI,
		SPI_SDO => SPI_SDO,
		SPI_CLK => SPI_CLK,
		
		Length_Of_SPIClk => Length_Of_SPIClk,
		
		-- The usual ports 
		Reset => Reset_SPI,
		Clk => Clk
	 );

end architecture Behavioral;

