library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;

entity Zarlink_SPI is
    Port ( 

		-- FIFO IN
		FIFO_IN_rd_en 					: OUT STD_LOGIC;
		FIFO_IN_dout 					: IN STD_LOGIC_VECTOR(29 DOWNTO 0);
		FIFO_IN_valid 					: IN STD_LOGIC;

		-- FIFO OUT
		FIFO_OUT_din 					: OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
		FIFO_OUT_wr_en 				: OUT STD_LOGIC;
		FIFO_OUT_prog_full 			: IN STD_LOGIC;
	 
		Reset								: in STD_LOGIC;
		Clk 								: in STD_LOGIC
	 
	 );
end Zarlink_SPI;

architecture Behavioral of Zarlink_SPI is

	-- The states of the SPI state machine
	type STATE_Type is (
		-- State after the reset 
		SPI_INIT,
		-- State where we wait for new data 
		SPI_IDLE,
		-- Start the SPI transfer
		--	(second&third start state for holding the CS down for a bit longer before begining with the ride),
		SPI_START_A,
		SPI_START_B,
		SPI_START_C,
		-- Handling the SPI Bit in the SPI_CLK = 0 phase
		SPI_BITHANLDE_1,
		-- Handling the SPI Bit in the SPI_CLK = 1 phase
		SPI_BITHANLDE_2,
		-- Ending the SPI transfer
		SPI_END,
		-- Cool down states for holding SPI_CS_B = 1 for > 250 ns (2x 125 = 375ns) 
		-- If neccessary, one of the cooldown states can (possibly) be removed 
		SPI_COOLDOWN_1,
		SPI_COOLDOWN_2,
		SPI_COOLDOWN_3
	);
	signal FSM_STATE : STATE_TYPE;
	
	signal Data_IN : std_logic_vector (23 downto 0) := (others => '0');
	signal Data_IN_Type : std_logic_vector (5 downto 0) := (others => '0');
	signal Data_IN_Read : std_logic := '1';
	signal Data_OUT : std_logic_vector (7 downto 0) := (others => '0');
	
begin
	StateMachine : process (Clk,Reset)
	begin
		
		if (Reset = '1') then
			FSM_STATE <= SPI_INIT;
			FIFO_IN_rd_en <= '0';
			Data_IN <= (others => '0');
			Data_IN_Type <= (others => '0');
			Data_IN_Read <= '1';
			Data_OUT <= (others => '0');
			FIFO_OUT_wr_en <= '0';
			FIFO_OUT_din <= (others => '0');
		elsif rising_edge(Clk) then
		
			-- Default:
			FSM_STATE <= FSM_STATE;
			FIFO_IN_rd_en <= '0';
			Data_IN <= Data_IN;
			Data_IN_Type <= Data_IN_Type;
			Data_IN_Read <= Data_IN_Read;
			Data_OUT <= Data_OUT;
			FIFO_OUT_wr_en <= '0';
			FIFO_OUT_din <= (others => '0');
			
			case FSM_STATE is

				when SPI_INIT =>
					FSM_STATE <= SPI_IDLE;
					
				when SPI_IDLE =>
					
					if (FIFO_IN_valid = '0') or (FIFO_OUT_prog_full = '1') then
						FSM_STATE <= SPI_IDLE;
					else
						FSM_STATE <= SPI_START_A;
						FIFO_IN_rd_en <= '1';
						
						Data_IN_Type <= FIFO_IN_dout(5 downto 0);
						-- Data_IN_Read nicht vergessen 					
						Data_IN <= FIFO_IN_dout(29 downto 6);
						
					end if;
					
				when SPI_START_A =>
					FSM_STATE <= SPI_START_B;
				
				when SPI_START_B =>
					FSM_STATE <= SPI_START_C;
				
				when SPI_START_C =>
					FSM_STATE <= SPI_BITHANLDE_1;
				
				when SPI_BITHANLDE_1 =>
					FSM_STATE <= SPI_BITHANLDE_2;

				when SPI_BITHANLDE_2 =>
					FSM_STATE <= SPI_END;
					FIFO_OUT_wr_en <= '1';
					FIFO_OUT_din <= "11111111" & Data_IN_Type & "00";
					
				when SPI_END =>
					FSM_STATE <= SPI_COOLDOWN_1;
				
				when SPI_COOLDOWN_1 =>
					FSM_STATE <= SPI_COOLDOWN_2;
				
				when SPI_COOLDOWN_2 =>
					FSM_STATE <= SPI_COOLDOWN_3;
	
				when SPI_COOLDOWN_3  =>
					FSM_STATE <= SPI_IDLE;

				when others => 
					FSM_STATE <= SPI_IDLE;
					
			end case;
	
		end if;
		
	end process StateMachine;

end Behavioral;

