library IEEE;
library UNISIM;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
use UNISIM.VComponents.all;

entity Zarlink_SPI_Module is
    Port ( 

		Zarlink_Data_Out			: in 		STD_LOGIC_VECTOR(119 downto 0); 
		Zarlink_WriteEn			: in 		STD_LOGIC; 
		Zarlink_Data_In			: out 	STD_LOGIC_VECTOR(111 downto 0);
		Zarlink_Valid_In			: out 	STD_LOGIC;
		Zarlink_BlockMode			: in		STD_LOGIC; 

		Zarlink_Wait				: out 	STD_LOGIC; 

		-- Connection to the Zarlink 
		SPI_ZL_CS 					: out STD_LOGIC;
		SPI_SDI 						: out STD_LOGIC;
		SPI_SDO 						: in 	STD_LOGIC;
		SPI_CLK 						: out STD_LOGIC;		

		Length_Of_SPIClk			: in STD_LOGIC_VECTOR(7 downto 0);

		-- The usual ports 
		Reset 						: in  STD_LOGIC;				
		Clk 							: in  STD_LOGIC
	 );
end Zarlink_SPI_Module;

architecture Behavioral of Zarlink_SPI_Module is

	signal WaitCounter : unsigned (7 downto 0) := (others => '0'); 
	signal BitCounter : unsigned (6 downto 0) := (others => '0'); 
	signal BitCounter_Max : unsigned (6 downto 0) := (others => '0'); 
	
	signal Zarlink_BlockMode_Copy : std_logic := '0';
	signal Zarlink_Data_Out_Copy	: STD_LOGIC_VECTOR(119 downto 0) := (others => '0');
	signal Zarlink_Data_In_Buffer	: STD_LOGIC_VECTOR(111 downto 0) := (others => '0');
	
	type STATE_TYPE is (
		-- Reset State
		STATE_RESET,
		-- IDLE State 
		STATE_IDLE,
		-- Start the SPI transfer
		--	(second&third start state for holding the CS down for a bit longer before begining with the ride),
		SPI_START,
		-- Handling the SPI Bit in the SPI_CLK = 0 phase
		SPI_BITHANLDE_1,
		-- Handling the SPI Bit in the SPI_CLK = 1 phase
		SPI_BITHANLDE_2_WAIT,
		SPI_BITHANLDE_2,
		-- Ending the SPI transfer
		SPI_END,
		-- Cool down states for holding SPI_CS_B = 1 for > 250 ns (2x 125 = 375ns) 
		-- If neccessary, one of the cooldown states can (possibly) be removed 
		SPI_COOLDOWN_1,
		SPI_COOLDOWN_2,
		SPI_COOLDOWN_WAIT
	);
	signal FSM_STATE : STATE_TYPE;	

begin


	FSM : process(Reset,Clk)
	begin
		if (Reset = '1') then
			FSM_STATE <= STATE_RESET;

			SPI_ZL_CS <= '1';
			SPI_SDI <= '0';
			SPI_CLK <= '0';

			BitCounter <= (others => '0');
			BitCounter_Max <= (others => '0');
			
			Zarlink_BlockMode_Copy <= '0';
			Zarlink_Data_Out_Copy <= (others => '0');
			Zarlink_Data_In_Buffer <= (others => '0');
			Zarlink_Wait <= '0'; 
			WaitCounter <= (others => '0'); 
			
			Zarlink_Data_In <= (others => '0');
			Zarlink_Valid_In <= '0';
			
		elsif rising_edge(Clk) then
		
			-- Default:
			BitCounter <= (others => '0');
			SPI_ZL_CS <= '1';
			SPI_SDI <= '0';
			SPI_CLK <= '0';
			Zarlink_Wait <= '1';
			Zarlink_BlockMode_Copy <= Zarlink_BlockMode_Copy;
			Zarlink_Data_Out_Copy <= Zarlink_Data_Out_Copy;
			WaitCounter <= (others => '0'); 

			Zarlink_Data_In <= (others => '0');
			Zarlink_Valid_In <= '0';
			Zarlink_Data_In_Buffer <= Zarlink_Data_In_Buffer;
		
			if Zarlink_BlockMode_Copy = '0' then
				BitCounter_Max <= to_unsigned(15,7);
			else
				BitCounter_Max <= to_unsigned(119,7);
			end if;
			
			case FSM_STATE is

				when STATE_RESET =>
					FSM_STATE <= STATE_IDLE;
					Zarlink_Wait <= '0';
					
				when STATE_IDLE =>
					if Zarlink_WriteEn = '1' then
						FSM_STATE <= SPI_START;
						Zarlink_BlockMode_Copy <= Zarlink_BlockMode;
						Zarlink_Data_Out_Copy <= Zarlink_Data_Out;
					else
						FSM_STATE <= STATE_IDLE;
						Zarlink_Wait <= '0';
					end if;
					
					
				when SPI_START =>
					if WaitCounter = to_unsigned(35,8) then
						FSM_STATE <= SPI_BITHANLDE_1;
					else
						FSM_STATE <= SPI_START;
						WaitCounter <= WaitCounter + 1;
					end if;
					SPI_ZL_CS <= '0';
					Zarlink_Data_In_Buffer <= (others => '0');					
				
				when SPI_BITHANLDE_1 =>
					if WaitCounter > unsigned(Length_Of_SPIClk) then
						FSM_STATE <= SPI_BITHANLDE_2_WAIT;
					else
						FSM_STATE <= SPI_BITHANLDE_1;
						WaitCounter <= WaitCounter + 1;
					end if;				

					BitCounter <= BitCounter;
					
					SPI_ZL_CS <= '0';
					SPI_SDI <= Zarlink_Data_Out_Copy(0);


			when SPI_BITHANLDE_2_WAIT =>
					if WaitCounter = unsigned(Length_Of_SPIClk) then
						FSM_STATE <= SPI_BITHANLDE_2;
					else
						FSM_STATE <= SPI_BITHANLDE_2_WAIT;
						WaitCounter <= WaitCounter + 1;
					end if;				
					
					BitCounter <= BitCounter;
					
					SPI_ZL_CS <= '0';
					SPI_SDI <= Zarlink_Data_Out_Copy(0);
					SPI_CLK <= '1';

				when SPI_BITHANLDE_2 =>
					-- Last Bit?
					if BitCounter = BitCounter_Max then
						-- Yes
						FSM_STATE <= SPI_END;
					else
						-- No
						FSM_STATE <= SPI_BITHANLDE_1;
					end if;
					BitCounter <= BitCounter + 1;
					
					SPI_ZL_CS <= '0';
					SPI_SDI <= Zarlink_Data_Out_Copy(0);
					SPI_CLK <= '1';
					Zarlink_Data_Out_Copy <= "0" & Zarlink_Data_Out_Copy(119 downto 1);
					Zarlink_Data_In_Buffer <= Zarlink_Data_In_Buffer(110 downto 0) & SPI_SDO;
					
				when SPI_END =>
					if WaitCounter = to_unsigned(35,8) then
						FSM_STATE <= SPI_COOLDOWN_1;
					else
						FSM_STATE <= SPI_END;
						WaitCounter <= WaitCounter + 1;
					end if;
					SPI_ZL_CS <= '0';
					
				when SPI_COOLDOWN_1 =>
					FSM_STATE <= SPI_COOLDOWN_2;
					
					-- If this was a register read then exterminate the 111 downto 0 bits. 
					-- It is not clear what it in there... 
					if Zarlink_BlockMode_Copy = '0' then
						Zarlink_Data_In_Buffer(111 downto 8) <= (others => '0');
					else
						null;
					end if;
				
				when SPI_COOLDOWN_2 =>
					FSM_STATE <= SPI_COOLDOWN_WAIT;
					Zarlink_Data_In <= Zarlink_Data_In_Buffer;
					Zarlink_Valid_In <= '1';
					
				when SPI_COOLDOWN_WAIT =>	 			
					if WaitCounter = to_unsigned(33,8) then
						FSM_STATE <= STATE_IDLE;
					else
						FSM_STATE <= SPI_COOLDOWN_WAIT;
						WaitCounter <= WaitCounter + 1;
					end if;
					
				when others =>
					FSM_STATE <= STATE_RESET;
					
			end case;
			
		end if;
	end process FSM;

end Behavioral;

